package crowdmodel;

import java.awt.Polygon;
import java.util.HashSet;
import java.util.Iterator;

import crowdmodel.agents.*;
import crowdmodel.environment.*;
import crowdmodel.test.BehaveTest;
import crowdmodel.test.PerceiveTest;
import crowdmodel.test.Test;
import crowdmodel.utils.Misc;
import crowdmodel.utils.MiscSingleton;
import repast.simphony.context.Context;
import repast.simphony.context.DefaultContext;
import repast.simphony.context.space.graph.NetworkFactoryFinder;
import repast.simphony.context.space.continuous.ContinuousSpaceFactoryFinder;
import repast.simphony.context.space.grid.GridFactoryFinder;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.dataLoader.SubContextCreator;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.parameter.Parameters;
import repast.simphony.space.continuous.RandomCartesianAdder;
import repast.simphony.space.grid.GridBuilderParameters;
import repast.simphony.space.grid.SimpleGridAdder;
import repast.simphony.valueLayer.GridValueLayer;


/**
 * Context creator is one of the main classes of the model.
 * It creates the context in which everything resides. This includes the objects,
 * the spaces in which objects/agents are places, but also
 * the relationships between objects can be defined here.
 * 
 * This (main) context created for the CROSS model represents the 'world' (physical objects) and 
 * a subcontext (agents). One can see it as an extra specification layer.
 * Main/master context: the one to rule them all, this one creates all the other
 * contexts and has all the elements that its subcontexts also has. Here the physical world is described. Walkable|non-walkable areas, 
 * grid, space and value layers.
 * Subcontext: agents, here the social structure is described as a subcontext 
 * of the main context. The social structure is defined by having agents and their relations (projections)
 * 
 * This is a part of the Repast structure, so for further information
 * read the Repast tutorial/website (repast.sourceforge.net).
 * 
0) CROSS has 2 contexts: main/world and agents. Where agents resides in world/main.
1) The projections in CROSS are: grid, continuous, value and network. Of which network belongs to the agent context and all the others to the world context.
2) In the score file every element needs to be registered, so Repast finds them. 
 * 
 * @author Nanda Wijermans
 *
 */

public class CContextCreator implements ContextBuilder<CObject>{
	
	
	/**
	 * Builds and returns a context. Building a context consists of filling it with
	 * agents, adding projections and so forth. When 'build' is executed, the master context
	 * the system will return a context based on information given in the
	 * model.score file. 
	 * 
	 * Includes:
	 * - Retrieving environment parameters from the GUI/batch-file
	 * - Physical element that make up the world context. These are areas of the festival scenario (
	 * -
	 */	
	
	public Context<CObject> build(Context<CObject> context){
		
		/*  
		 * The environment parameters contain the user-editable values that appear in the GUI.
		 * Retrieves the parameters p. 
		 */
		Parameters p = RunEnvironment.getInstance().getParameters();
		
//		int d_worldWidth = (Integer) p.getValue("sim_worldWidth");
//		int d_worldHeight = (Integer) p.getValue("sim_worldHeight");
		
		int worldWidth = Constants.WORLD_WIDTH;
		int worldHeight = Constants.WORLD_HEIGHT;
		
		int d_crowdsize = (Integer) p.getValue("group_crowdsize");
//		int d_numLeaders = (Integer) p.getValue("group_nrLeaders");
		int d_ratioLeaders = (Integer) p.getValue("group_ratioLeaders");
		int d_numLeaders = (int) Math.rint(d_crowdsize * ((double) d_ratioLeaders / 100.0)); 
//		int d_numWLeaders = (Integer) p.getValue("group_warLeaders") * d_numLeaders;	// percentage of leaders that direct in an aggressive way
//		int d_numPLeaders = d_numLeaders - d_numWLeaders;   // percentage of leaders that direct in a peaceful way	
		String d_safetyPref_dist = (String) p.getValueAsString("safety_pref_dist");
		
		int densityCondition = Misc.getDensityCondition(d_crowdsize);
		int safetyPrefCondition = Misc.getSafetyCondition(d_safetyPref_dist);
		int leadershipCondition = Misc.getLeadershipCondition(d_ratioLeaders);
			
		SubContextCreator agentcontextcreator = new AgentContextCreator();		
		Context<CrowdAgent> agentContext = agentcontextcreator.build(new DefaultContext<CrowdAgent>("Agent Subcontext", Constants.ID_AGENT_CONTEXT));
		agentContext.setId(Constants.ID_AGENT_CONTEXT);
		context.addSubContext(agentContext);
		
		NetworkFactoryFinder.createNetworkFactory(null).createNetwork(
		"Behaviour Network", context, false);
 
			
		/*
		 * Creates a new 2D grid to model the discrete areas of the festival
		 * Areas are: toilet, bar, stage and the rest (walkable area)
		 *  
		 * The inputs to the GridFactory include the grid name, 
		 * the context in which to place the grid, and the grid parameters. 
		 * Grid parameters include the border specification, <setter for placing the areas on the grid>
		 * boolean for multiple occupancy, and the dimensions of the grid.
		 */
		
		GridFactoryFinder.createGridFactory(null).
			createGrid("Simple Grid", context, 
				new GridBuilderParameters<CObject>( 
					new repast.simphony.space.grid.StickyBorders(),
						new SimpleGridAdder<CObject>(), true, worldWidth, worldHeight)); 
		
		
		/** 
		 * Creates a new 2D continuous space. Continuous space is used to model the physical space
		 * on which the agents will move. The inputs for the Space Factory include the space name,
		 * the context in which to place the space, border specification, random adder for populating the grid with agents,
		 * and the dimensions of the grid.
		 * 
		 * 
		 */
		
		ContinuousSpaceFactoryFinder.createContinuousSpaceFactory(null).
			createContinuousSpace("Continuous Space", context, 
				new RandomCartesianAdder<CObject>(),
					new repast.simphony.space.continuous.StickyBorders(), worldWidth, worldHeight, 1);

		/* 
		 * Create a new 2D value layer to store the areas of the stage and bar/toilets. This is 
		 * only used for visualisation since it's faster to draw the value layer
		 * in 2D displays compared with rendering each environmental patch as an agent.
	     * However the dynamic agent representation is still present, which might be useful for other things
	     */
		
		GridValueLayer valueLayer = new GridValueLayer("Terrain", true,
				new repast.simphony.space.grid.StickyBorders(),worldWidth, worldHeight);
				context.addValueLayer(valueLayer);						


		for (int i = 0; i < worldWidth; ++i){
					for (int j = 0; j < worldHeight; ++j){
						CellObject cell = new CellObject(context, i, j, MiscSingleton.getInstance().nextID()); // create an empty terrain cell
					}	
		}
		
		/* defines the scenario areas */
		Polygon stagePolygon = new Polygon(Constants.stageX, Constants.stageY, 4);
		Polygon barPolygon = new Polygon(Constants.barX, Constants.barY, 4);
		Polygon toiletPolygon = new Polygon(Constants.toiletX, Constants.toiletY, 4);
		
		/* stage */
		Area d_stage = new Area(context, "stage", stagePolygon, false);
		d_stage.addPoi(Constants.POI_STAGE);
 		d_stage.setPOIs();
 		
		/* bar */
		Area d_bar = new Area(context, "bar", barPolygon, false);
		d_bar.addPoi(Constants.POI_BAR);
		d_bar.addPoi(Constants.POI_BAR1);
		d_bar.addPoi(Constants.POI_BAR2);
		d_bar.setPOIs();
		
		/* toilet */
		Area d_toilet = new Area(context, "toilet", toiletPolygon, false);
		d_toilet.addPoi(Constants.POI_TOILET);
		d_toilet.addPoi(Constants.POI_TOILET1);
		d_toilet.addPoi(Constants.POI_TOILET2);
 		d_toilet.setPOIs();
 	
 /* define the agent (sub)context */		
		
		/* Populate the agent subcontext (and thus also the world and root context) with the initial agents
		 * Iterate over the number of agents. Agent are related to the continuous layer in terms of location and movement */
				
		for (int i = 0; i < d_crowdsize; ++i) {
			CrowdAgent cagent = new CrowdAgent(MiscSingleton.getInstance().nextID(), context, agentContext); // create a new crowd agent
			agentContext.add(cagent);
		}
		
		/* Set Leadership, by randomly assigning some agents as leaders by
		 * making them see themselves as a leader and other see this person as a leader
		 * 
		 * Perceiving oneself as a leader doesn't involve active change of an agent. As it's social environment has an
		 * equal influence upon a leader.
		 * Perceiving someone else as a leader on the other hand does have an active effect. As the influence of
		 * perceiving a behaviour is differentiated, ie depends on who you are observing what effect is has.
		 */
	
		if (d_numLeaders < d_crowdsize + 1){
			int numLeaders = d_numLeaders;
			HashSet<CrowdAgent> leaders = new HashSet<CrowdAgent>();
			while ( numLeaders > 0 ) {
				CrowdAgent testDummy = (CrowdAgent) agentContext.getRandomObject(); //is trekken met terugleggen (hoe in het engels? drawing & returning..)
				while (testDummy != null && testDummy.isLeader()){
					testDummy = (CrowdAgent) agentContext.getRandomObject();
				}					
				if (testDummy == null){
					System.out.println("dummy is null - cannot happen");
					//Syst.err
					numLeaders = 0;
				} else {
					testDummy.setLeadership(); // become a leader
					leaders.add(testDummy);
				    --numLeaders;
				}
			}
			/* let all other agents perceive these selected individuals as leaders */
			
			Iterator<CrowdAgent> agents1 = (Iterator<CrowdAgent>) agentContext.iterator();
			while(agents1.hasNext()){
				CrowdAgent ca = agents1.next();
				ca.setMyLeaders(leaders);
			}
		} else {  // more leaders than agents
			System.out.println("Cannot create more leaders than agents");
			//Syst.err 
		}

		int duoPerc = (Integer) p.getValue("group_percDuo");
		int trioPerc = (Integer) p.getValue("group_percTrio");
		int quartetPerc = (Integer) p.getValue("group_percQuartet");
			
		int[][] friends = Misc.makeFriends(d_crowdsize, duoPerc, trioPerc, quartetPerc);
		Misc.setSocialNetwork(agentContext, friends);
			
		Iterator<CrowdAgent> agents = (Iterator<CrowdAgent>) agentContext.iterator();
		while(agents.hasNext()){
			CrowdAgent ca = agents.next();
			ca.init(densityCondition, safetyPrefCondition, leadershipCondition);   // this can only be done after the social network matrix has been filled
		}
		
		
		/* Testing code/model */
		Iterator<CrowdAgent> agents1 = (Iterator<CrowdAgent>) agentContext.iterator();
		CrowdAgent testDummy = (agentContext.isEmpty() ? null : (CrowdAgent) agentContext.getRandomObject());

		PerceiveTest pTest = new PerceiveTest(testDummy);
		BehaveTest bTest = new BehaveTest(testDummy, d_crowdsize, agents1);
		Test test = new Test(); 
		
		context.add(pTest);
		context.add(bTest);
		context.add(test);

		
		/* Outputters */
		CrowdAgent rndAgent1 = (agentContext.isEmpty() ? null : (CrowdAgent) agentContext.getRandomObject());
		CrowdAgent rndAgent2 = (agentContext.isEmpty() ? null : (CrowdAgent) agentContext.getRandomObject());
		CrowdAgent rndAgent3 = (agentContext.isEmpty() ? null : (CrowdAgent) agentContext.getRandomObject());
		
		Cluster cluster = new Cluster(densityCondition, safetyPrefCondition, leadershipCondition);
		context.add(cluster);
		
		cluster.setAgent4Tracking(rndAgent1,rndAgent2,rndAgent3);
		
		/* When running in batch mode, tell the scheduler when to end each run. */ 
		if (RunEnvironment.getInstance().isBatch()){
			double endAt = (Double) p.getValue("runlength");
			RunEnvironment.getInstance().endAt(endAt);
		}
		
		return context;
	}

}
	
