package crowdmodel;

import java.util.ArrayList;
import java.util.Iterator;
import repast.simphony.annotate.AgentAnnot;
import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.environment.RunState;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.essentials.RepastEssentials;
import repast.simphony.parameter.Parameters;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.graph.Network;
import repast.simphony.space.graph.RepastEdge;
import crowdmodel.agents.CrowdAgent;
import crowdmodel.utils.MathUtils;
import crowdmodel.utils.Misc;
import crowdmodel.utils.MiscSingleton;

/**
 * Cluster is a class that contains the functions to update the output measure: clusters
 * 
 * A cluster is defined as a network structure that indicates a set of agents that either
 * 1) perform the same behaviour
 * 2) perform a specific behaviour
 * 3) whatever way you would like to define a cluster..
 * For all connections counts that the agents are in each others physical vicinity (here
 * range of sight). 
 *  
 * It is used as an output measure as this should represent observable group patterns that
 * can also be compared in the real world.
 * 
 * These clusters are represented as a graph (social network). Every time step the connections
 * are created or updated between agents based on the current definition of a cluster. given the definition (1,2, ..)
 * individuals are connected and can have a certain weight. This network is an unidirectional graph/network.
 * 
 * We specify the connections in terms of:
 * a) adding/removing edges given the current cluster definition, the clusters represent the current status
 * b) updating the weights of the connections, given the definition weights are increased, which shows a development during time
 * 
 * Note: I can also make this function independent from Repast by outputting in a file every time step
 * still need to figure out what is the most convenient way to do so.
 * 
 * 
 * @author Nanda Wijermans
 *
 */

@AgentAnnot(displayName = "Cluster")

public class Cluster extends CObject {

	int d_crowdsize;
	int d_numEdges;
	int d_cntEdges;
	
	int d_n;
	double d_sumDistances;
	double d_meanAgentDistance;     // mean of the measured distances between agents: mean = sum(d_i) / n
	double d_varAgentDistance;		// variance of distance between agents: variance = sum(d_i - mean)^2 / (n - 1)
	
	double d_maxDensity;
	
	double[] d_distances;
	
//	final double d_runlength; // use this variable in batch run	 
	final int d_runlength;    // use this variable in GUI run
	final int d_seed;
	final String d_safety_dist;
	
	
	final int d_exp_dens;  // indicates the type of condition is chosen for this experiment
	final int d_exp_safe;  
	final int d_exp_leadership;

	/* variables for the life history of agents - detailed info of individuals */
	CrowdAgent d_dummy1, d_dummy2, d_dummy3;    // agents that are being tracked during a run for their individual life history - track their behaviour on the inter and intra level

	public double d_x1, d_x2, d_x3;  
	public double d_y1, d_y2, d_y3;
	
	public double d_arousal1, d_arousal2, d_arousal3;
	public double d_behaviour1, d_behaviour2, d_behaviour3;
	
	public double d_idDom1, d_idDom2, d_idDom3;
	public double d_socDom1, d_socDom2, d_socDom3;
	public double d_safeDom1, d_safeDom2, d_safeDom3;
	public double d_subsDom1, d_subsDom2, d_subsDom3;
	
	public double d_walkAct1, d_walkAct2, d_walkAct3;
	public double d_runAct1, d_runAct2, d_runAct3;
	public double d_danceAct1, d_danceAct2, d_danceAct3;

	public double d_walkExp1, d_walkExp2, d_walkExp3;
	public double d_runExp1, d_runExp2, d_runExp3;
	public double d_danceExp1, d_danceExp2, d_danceExp3;
	
	public double d_walkUtil1, d_walkUtil2, d_walkUtil3;
	public double d_runUtil1, d_runUtil2, d_runUtil3;
	public double d_danceUtil1, d_danceUtil2, d_danceUtil3;
	
	public double d_ci1, d_ci2, d_ci3; //cluster index 
	public double d_leadercnt1, d_leadercnt2, d_leadercnt3; //perceived leaders
	public double d_walkingLeaders1, d_walkingLeaders2, d_walkingLeaders3; //to track the type of behaviour leaders show
	public double d_runningLeaders1, d_runningLeaders2, d_runningLeaders3; //to track the type of behaviour leaders show
	public double d_dancingLeaders1, d_dancingLeaders2, d_dancingLeaders3; //to track the type of behaviour leaders show


	/**
	 * Constructor for cluster, initialises the variables
	 */
	
	public Cluster(int densityCondition, int safetyPrefCondition, int leadershipCondition){
		d_id = MiscSingleton.getInstance().nextID();
		Parameters p = RunEnvironment.getInstance().getParameters();
		d_crowdsize = (Integer) p.getValue("group_crowdsize"); 
//		d_runlength = (Double) p.getValue("runlength");  // in batch run
		d_runlength = (Integer) p.getValue("runlength"); // in GUI run
		d_seed = (Integer) RepastEssentials.GetParameter("randomSeed");
		d_safety_dist = (String) p.getValueAsString("safety_pref_dist");
		d_numEdges = 0;
		d_cntEdges = 0;
		
		d_n = 0;
		d_sumDistances = 0;
		d_meanAgentDistance = 0;
		d_varAgentDistance = 0;
		d_maxDensity = 0;
		
		int numDs = (int) Math.ceil(0.5 * (Math.pow(d_crowdsize, 2) - d_crowdsize));  // number of distances between agents : 1/2(n^2-n)
		d_distances = new double[numDs];
		
		for (int i = 0; i < d_distances.length; i++) {
			d_distances[i] = 0;
		}
		
		d_exp_dens = densityCondition;
		d_exp_safe = safetyPrefCondition;	
		d_exp_leadership = leadershipCondition;
	}
	
	
	/**
	 * Sets the agents that will be tracked during the current run of the simulation.
	 * The individual state (internally) are recorded, which is to much to do that
	 * for all agents
	 * 
	 * @param rndAgent1
	 * @param rndAgent2
	 * @param rndAgent3
	 */
	public void setAgent4Tracking(CrowdAgent rndAgent1, CrowdAgent rndAgent2,
			CrowdAgent rndAgent3) {
		
		d_dummy1 = rndAgent1;
		d_dummy2 = rndAgent2;
		d_dummy3 = rndAgent3;	
		
		d_dummy1.setToDummy(1);
		d_dummy2.setToDummy(2);
		d_dummy3.setToDummy(3);
		
		d_x1 = 0;
		d_y1 = 0;
		d_arousal1 = 0;
		d_behaviour1 = 0;
		d_idDom1 = 0;
		d_socDom1 = 0;
		d_safeDom1 = 0;
		d_subsDom1 = 0;
		d_walkAct1 = 0;
		d_runAct1 = 0;
		d_danceAct1 = 0;
		d_walkExp1 = 0;
		d_runExp1 = 0;
		d_danceExp1 = 0;
		d_walkUtil1 = 0;
		d_runUtil1 = 0;
		d_danceUtil1 = 0;
		d_ci1 = 0; 
		d_leadercnt1 = 0;
		d_walkingLeaders1 = 0;
		d_runningLeaders1 = 0;
		d_dancingLeaders1 = 0;
		
		d_x2 = 0;
		d_y2 = 0;
		d_arousal2 = 0;
		d_behaviour2 = 0;
		d_idDom2 = 0;
		d_socDom2 = 0;
		d_safeDom2 = 0;
		d_subsDom2 = 0;
		d_walkAct2 = 0;
		d_runAct2 = 0;
		d_danceAct2 = 0;
		d_walkExp2 = 0;
		d_runExp2 = 0;
		d_danceExp2 = 0;
		d_walkUtil2 = 0;
		d_runUtil2 = 0;
		d_danceUtil2 = 0;
		d_ci2 = 0;
		d_leadercnt2 = 0; 
		d_walkingLeaders2 = 0;
		d_runningLeaders2 = 0;
		d_dancingLeaders2 = 0;
		
		d_x3 = 0;
		d_y3 = 0;
		d_arousal3 = 0;
		d_behaviour3 = 0;
		d_idDom3 = 0;
		d_safeDom3 = 0;
		d_socDom3 = 0;
		d_subsDom3 = 0;
		d_walkAct3 = 0;
		d_runAct3 = 0;
		d_danceAct3 = 0;
		d_walkExp3 = 0;
		d_runExp3 = 0;
		d_danceExp3 = 0;
		d_walkUtil3 = 0;
		d_runUtil3 = 0;
		d_danceUtil3 = 0;
		d_ci3 = 0;
		d_leadercnt3 = 0;
		d_walkingLeaders3 = 0;
		d_runningLeaders3 = 0;
		d_dancingLeaders3 = 0;
	}
	
	/**
	 * Function that makes sure the network of clusters is updated.
	 * A cluster is the group measure, however the definition of the cluster
	 * can differ, this set here.
	 * 
	 * Furthermore, here the update is assigned to the scheduler.
	 * The priority parameter makes sure this function is called last! (high number means high priority, and vise versa)
	 * As the agents need to be finished first.
	 * @param crowdsize
	 */
	
	public void updateClusters(){
		resetNetwork();    /* all previous connections are removed */	
		
		resetCounters();
		Context context = RunState.getInstance().getMasterContext();		
		double tick = RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
		updateBClusters();
		updateLifeHistory();
		
		//Context context = RunState.getInstance().getMasterContext();
		//Network<CrowdAgent> behaviourNetwork = (Network<CrowdAgent>) context.getProjection("Behaviour Network");
//		d_numEdges = behaviourNetwork.numEdges();
		//System.out.println("Cluster.updateClusters After updating countEdges: " + d_cntEdges + " numEdges: " + d_numEdges);
}

	
	
@ScheduledMethod(start = 0, interval = 1, priority = Integer.MIN_VALUE, shuffle = false)  // high priority = high number; low priority = low number
	public void updateClustersWithoutNetwork(){
		//System.out.println("Cluster.updateClusters() : prio = laatste");
		resetCounters();
		updateBClustersWithoutNetwork();
		updateLifeHistory();
	}

/* Test method */
//@ScheduledMethod(start = 0, interval = 5, priority = 10, shuffle = false)  // high priority = high number; low priority = low number
//public void perceiveTest(){
//	System.out.println("Cluster.perceiveTest() : prio = 10");
//	Context context = RunState.getInstance().getMasterContext();
//	d_pTest.testIt(context);
//}

//@ScheduledMethod(start = 0, interval = 5, priority = 10, shuffle = false)  // high priority = high number; low priority = low number
//public void behaveTest(){
//	System.out.println("Cluster.behaveTest() : prio = 4");
//	d_bTest.testIt();
//}

	public void resetCounters(){
		d_cntEdges = 0;
		d_n = 0;
		
		d_sumDistances = 0;
		
		d_meanAgentDistance = 0;
		d_varAgentDistance = 0;
		
		d_maxDensity = 0;
	}


/**
 * Removes all the existing connections (edges) in a given network.
 * Returns a network without connections (only nodes)
 * 
 * @param network
 * @return network without edges
 * 
 */
private void resetNetwork() {
	
	Context context = RunState.getInstance().getMasterContext();
	Context agentContext = context.getSubContext(Constants.ID_AGENT_CONTEXT);

	//Network<CrowdAgent> network = (Network<CrowdAgent>) context.getProjection("Behaviour Network");

	// approach 1: remove all the edges, while the object (network) remains to exist		
	//Iterator edges = network.getEdges().iterator();

	//System.out.println("Cluster.reset: huidige edges in netwerk: " + network.numEdges());

	/*
	 * This while loop has an oddity. When the Iterator (edges) isn't refreshed is will give error while
	 * running. I suspect that this is due to an object reference of the iterator that doesn't exist anymore, as
	 * the iterator removes the edges it is pointing to. The odd thing for me is that after the removal of the
	 * edge, it will never address this edge as it moves to the next one. Maybe it is logical, however I am sure
	 * that I will make the same mistake again in future
	 * 
	 */
//	while (edges.hasNext()){
//		RepastEdge edge = (RepastEdge) edges.next();
//		network.removeEdge(edge);
//		edges = network.getEdges().iterator(); 
//	}

	// approach 2: remove the whole network and create a new one
	// Couldn't make it work, don't have time to figure this out. But isn't cheaper as the previous, at it involves even more steps
	// removal of edges, nodes, and projection out of a context.
		
	agentContext.removeProjection("Behaviour Network");
//	NetworkFactoryFinder.createNetworkFactory(null).createNetwork(
//			"Behaviour Network", agentContext, false);

}

	/**
	 * Counts the similiar behaviours without representing this in a network.
	 * Just for testing 
	 */

	public void updateBClustersWithoutNetwork(){
	
		Context context = RunState.getInstance().getMasterContext();
		Context<CrowdAgent> agentContext = context.getSubContext(Constants.ID_AGENT_CONTEXT);
		ArrayList<CrowdAgent> agentArray = Misc.iterator2array(agentContext.iterator(), d_crowdsize);
	
		for (int i = 0; i < agentArray.size(); i++) {
			CrowdAgent ca = (CrowdAgent) agentArray.get(i);
			for (int j = i+1; j < d_crowdsize - 1; j++) {			
				CrowdAgent ca2 = (CrowdAgent) agentArray.get(j);
				updateSameBehaviourCnt(ca, ca2);   // same behaviour == connection 
				updateDistance(ca, ca2);
			}
			double pdens = ca.getDensity();
			d_maxDensity = (pdens > d_maxDensity ? pdens : d_maxDensity);
		}
		calcDistanceIndicators();
	}


	/**
	 * Updates the connections between agents in terms of a social network structure (clusters).
	 * This functions describes a type of social structure in which a cluster is defined as: <br>
	 * 1) behaviour shown by the connected agents are the same <br>
	 	  a) Where the structure represents the current state; meaning that presence of a connection
	 *    is based on the current behaviour
	 *    
	 * With the note that when agents move in a similar direction, we will perceive them as the same. However
	 * when not, they are not accounted for as similar or synchronised. We make use of the heading of the agents
	 * to indicate this.
	 * 
	 * @param agentContext
	 * @param behaviour
	 */
	
	public void updateBClusters(){
		
		Context context = RunState.getInstance().getMasterContext();
		Context agentContext = context.getSubContext(Constants.ID_AGENT_CONTEXT);				
		ArrayList agentArray = Misc.iterator2array(agentContext.iterator(), d_crowdsize);
		
		for (int i = 0; i < agentArray.size(); i++) {
			CrowdAgent ca = (CrowdAgent) agentArray.get(i);
			String b1 = ca.getObservableB();
			for (int j = i+1; j < d_crowdsize - 1; j++) {
				CrowdAgent ca2 = (CrowdAgent) agentArray.get(j);
				updateSameBehaviour(ca, ca2);	// same behaviour == connection 
				updateDistance(ca, ca2);
			}
		}
		calcDistanceIndicators();
	}

	/**
	 * Checks whether two agents show the same behaviour.
	 * In case they are showing the same behaviour the counter is increased
	 * and a networkEdge is added between the two agents in the behaviourNetwork (social network structure)
	 * 
	 * @param ca
	 * @param ca2
	 */
	public void updateSameBehaviour(CrowdAgent ca, CrowdAgent ca2){
		Context context = RunState.getInstance().getMasterContext();
		Network<CrowdAgent> behaviourNetwork = (Network<CrowdAgent>) context.getProjection("Behaviour Network");
		ContinuousSpace space = (ContinuousSpace) context.getProjection("Continuous Space");
		
		String b1 = ca.getObservableB();
		String b2 = ca2.getObservableB();
		
		if (!b1.equalsIgnoreCase("nothing") & b1.equalsIgnoreCase(b2)){	 // same behaviour
			NdPoint p1 = space.getLocation(ca);
			NdPoint p2 = space.getLocation(ca2);
			double dist = space.getDistance(p1, p2);
			if (dist < Constants.GAZE_DEPTH){ // within viewing distance
				boolean syncWalk = b1.equalsIgnoreCase(Constants.B_NAME_WALK);
				boolean syncRun = b1.equalsIgnoreCase(Constants.B_NAME_RUN);
				if (syncWalk || syncRun){
					double heading1 = ca.getHeading();
					double heading2 = ca2.getHeading();
					if (MathUtils.getAngleDiff(heading1, heading2) < 180) { // sync movement
						behaviourNetwork.addEdge(ca, ca2);
						++d_cntEdges;
					}
				} else {							
					behaviourNetwork.addEdge(ca, ca2);
					++d_cntEdges;
					}
			}
		}
	}
	
	/**
	 * Checks whether two agents show the same behaviour.
	 * In case they are showing the same behaviour the counter is increased
	 * (same as the updateSameBehaviour function without the network representation)
	 * 
	 * @param ca
	 * @param ca2
	 */
	public void updateSameBehaviourCnt(CrowdAgent ca, CrowdAgent ca2){
		Context context = RunState.getInstance().getMasterContext();
		Network<CrowdAgent> behaviourNetwork = (Network<CrowdAgent>) context.getProjection("Behaviour Network");
		ContinuousSpace space = (ContinuousSpace) context.getProjection("Continuous Space");
		
		String b1 = ca.getObservableB();
		String b2 = ca2.getObservableB();
		
		if (b1.equalsIgnoreCase(b2)) {  // same behaviour
			NdPoint p1 = space.getLocation(ca);
			NdPoint p2 = space.getLocation(ca2);
			double dist = space.getDistance(p1, p2);
			if (dist < Constants.GAZE_DEPTH){ // within viewing distance
				boolean syncWalk = b1.equalsIgnoreCase(Constants.B_NAME_WALK);
				boolean syncRun = b1.equalsIgnoreCase(Constants.B_NAME_RUN);
				if (syncWalk || syncRun){
					double heading1 = ca.getHeading();
					double heading2 = ca2.getHeading();
					if (MathUtils.getAngleDiff(heading1, heading2) < 180) { // sync movement
						++d_cntEdges;
					}
				} else {							
					++d_cntEdges;
					}
			}
		}
	}
	
		
	
	/**
	 * Updates the connections between agents in terms of a social network structure (clusters).
	 * This functions describes a type of social structure in which a cluster is defined as: <br>
	 * 1) behaviour shown by the connected agents are the same <br>
	 *    b) Where the structure represents the evolved state; meaning that the presence and weight of 
	 *    a connection is the result of the past an now. 
	 * 
	 */

	
	
	/**
	 * Updates the connections between agents in terms of a social network structure (clusters).
	 * This functions describes a type of social structure in which a cluster is defined as:
	 * 2) the behaviour they show is of a specific type
	 *    a) Where the structure represents the current state; meaning that presence of a connection
	 *    is based on the behaviour of the agents in the current time step
	 * 
	 * @param behaviour
	 */
	
	public void updateBSpecClusters(String behaviour){
		
		Context context = RunState.getInstance().getMasterContext();
		Context agentContext = context.getSubContext(Constants.ID_AGENT_CONTEXT);
		Network<CrowdAgent> behaviourNetwork = (Network<CrowdAgent>) context.getProjection("Behaviour Network");
		ContinuousSpace space = (ContinuousSpace) context.getProjection("Continuous Space");
		
		Iterator<CrowdAgent> agents = (Iterator<CrowdAgent>) agentContext.iterator();
		ArrayList agentArray = Misc.iterator2array(agents, d_crowdsize);
		

		/* same behaviour ==  specified behaviour ==> connection */
		
		int index = 0;
		resetNetwork();    /* all previous connections are removed */
		while (agents.hasNext()){
			++index;
			CrowdAgent ca = agents.next();
			String b1 = ca.getObservableB();
			if (b1.equalsIgnoreCase(behaviour)){
				for (int i = index; i < d_crowdsize - 1; i++) {
					CrowdAgent ca2 = (CrowdAgent) agentArray.get(i);
					String b2 = ca2.getObservableB();
					if (b1 == b2) {  
						NdPoint p1 = space.getLocation(ca);
						NdPoint p2 = space.getLocation(ca2);
						double dist = space.getDistance(p1, p2);
						if (dist < Constants.GAZE_DEPTH){
							behaviourNetwork.addEdge(ca, ca2);
							//System.out.println("Cluster.updateBSpecCluster: edge is added");
						}
					}
				}
			}
		}
	}
	
	/**
	 * Updates the connections between agents in terms of a social network structure (clusters).
	 * This functions describes a type of social structure in which a cluster is defined as:
	 * a) the behaviour they show is of a specific type  
	 * b) Where the structure represents the evolved state; meaning that the presence and weight of 
	 *    a connection is the result of the past an now. 
	 * 
	 * @param behaviour
	 */
	
	public void updateBSpecEvolvedClusters(String behaviour){
		
		Context context = RunState.getInstance().getMasterContext();
		Context agentContext = context.getSubContext(Constants.ID_AGENT_CONTEXT);
		Network<CrowdAgent> behaviourNetwork = (Network<CrowdAgent>) context.getProjection("Behaviour Network");
		ContinuousSpace space = (ContinuousSpace) context.getProjection("Continuous Space");
		
		Iterator<CrowdAgent> agents = (Iterator<CrowdAgent>) agentContext.iterator();
		ArrayList agentArray = Misc.iterator2array(agents, d_crowdsize);
		
		/* same behaviour == specified behaviour ==> ++weight connection */
		int index = 0;
		while (agents.hasNext()){
			++index;
			CrowdAgent ca = agents.next();
			String b1 = ca.getObservableB();
			if (b1.equalsIgnoreCase(behaviour)){
				for (int i = index; i < d_crowdsize - 1; i++) {
					CrowdAgent ca2 = (CrowdAgent) agentArray.get(i);
					String b2 = ca2.getObservableB();
					if (b1 == b2) {  
						NdPoint p1 = space.getLocation(ca);
						NdPoint p2 = space.getLocation(ca2);
						double dist = space.getDistance(p1, p2);
						if (dist < Constants.GAZE_DEPTH){
							RepastEdge edge = behaviourNetwork.getEdge(ca, ca2);
							if (edge == null){
								behaviourNetwork.addEdge(ca, ca2, 1);
							} else {
								double w = edge.getWeight() + 1;
								edge.setWeight(w);
							}
						}
					}
				}
			}
		}
	}
	
	
	/**
	 * Returns the number of edges that are present in the current time step.
	 * This indicates the behaviour clustering of the crowd. We developed a group measure
	 * clusters that indicates a cohesion or local behaviour clustering based on bilateral relations.
	 * In describing it as a group  measure we aim to relate the group level (patterns) with
	 * the underlying processes. (with a correction for the crowd size)
	 * 
	 * @return number of edges in the BehaviourNetwork
	 */
	public double getSynchronicity(){ //getBclustering
		double sync = ((double) d_cntEdges) / d_crowdsize;   //correction for the crowdsize
		return sync;
	}
	
	public int getSyncEdgeCnt(){  //getBclusteringEdgeCnt
		return d_cntEdges;
	}

	/**
	 * Returns the number of edges that are present in the current time step.
	 * This indicates the behaviour clustering of the crowd. We developed a group measure
	 * clusters that indicates a cohesion or local behaviour clustering based on bilateral relations.
	 * In describing it as a group  measure we aim to relate the group level (patterns) with
	 * the underlying processes.
	 * 
	 * @return number of edges in the BehaviourNetwork
	 */
	public int testBclustering(){
		return d_cntEdges;
	}
	
	/**
	 * Updates the mean and variance of the measured distances between agents
	 * given a new distance of two agents.
	 * @param ca
	 * @param ca2
	 */
	private void updateDistance(CrowdAgent ca, CrowdAgent ca2){
			
		Context context = RunState.getInstance().getMasterContext();
		Context agentContext = context.getSubContext(Constants.ID_AGENT_CONTEXT);
		
		ContinuousSpace space = (ContinuousSpace) context.getProjection("Continuous Space");
		
		NdPoint pos1 = space.getLocation(ca);
		NdPoint pos2 = space.getLocation(ca2);
		
		double distance = space.getDistance(pos1, pos2) * Constants.UNIT2METER;
		
		d_distances[d_n] = distance;
		d_sumDistances += distance;
		++d_n;
	}
	
	/**
	 * Sets the distance indicators for this time-step
	 * - Mean of the distances between agents : sum(distances) / n
	 * - Variance of the distances between agents : sum((distances - mean)^2) / (n-1)
	 */
	private void calcDistanceIndicators(){
		d_meanAgentDistance = d_sumDistances / d_n;
		double sumVar = 0;
		for (int i = 0; i < d_n - 1 ; i++) {
			double diff = d_distances[i] - d_meanAgentDistance;
			sumVar += Math.pow(diff, 2) ;
		}
		d_varAgentDistance = sumVar / (d_n - 1);
	}
	
	
	private void updateLifeHistory(){
		/* agent dummy 1 */
		d_x1 = d_dummy1.getMyPosx();
		d_y1 = d_dummy1.getMyPosy();
		d_arousal1 = d_dummy1.getArousalLevel();
		d_behaviour1 = d_dummy1.getObservableBNum();
		d_idDom1 = d_dummy1.getIDdominance(); 
		d_socDom1 = d_dummy1.getSocDominance();
		d_safeDom1 = d_dummy1.getSafeDominance();
		d_subsDom1 = d_dummy1.getSubsDominance();
		d_walkAct1 = d_dummy1.getActivationBruleWalk();
		d_runAct1 = d_dummy1.getActivationBruleRun();
		d_danceAct1 = d_dummy1.getActivationBruleDance();
		d_walkExp1 = d_dummy1.getWalkExp();
		d_runExp1 = d_dummy1.getRunExp();
		d_danceExp1 = d_dummy1.getDanceExp();
		d_walkUtil1 = d_dummy1.getWalkUtil();
		d_runUtil1 = d_dummy1.getRunUtil();
		d_danceUtil1 = d_dummy1.getDanceUtil();
		d_ci1 = d_dummy1.clusterIndex();
		
		d_leadercnt1 = d_dummy1.getCntVisibleLeaders();
		d_walkingLeaders1 = d_dummy1.getNumWalkingLeaders();
		d_runningLeaders1 = d_dummy1.getNumRunningLeaders();
		d_dancingLeaders1 = d_dummy1.getNumDancingLeaders();
		
		
		/* agent dummy 2 */
		d_x2 = d_dummy2.getMyPosx();
		d_y2 = d_dummy2.getMyPosy();
		d_arousal2 = d_dummy2.getArousalLevel();
		d_behaviour2 = d_dummy2.getObservableBNum();
		d_idDom2 = d_dummy2.getIDdominance(); 
		d_socDom2 = d_dummy2.getSocDominance();
		d_safeDom2 = d_dummy2.getSafeDominance();
		d_subsDom2 = d_dummy2.getSubsDominance();
		d_walkAct2 = d_dummy2.getActivationBruleWalk();
		d_runAct2 = d_dummy2.getActivationBruleRun();
		d_danceAct2 = d_dummy2.getActivationBruleDance();
		d_walkExp2 = d_dummy2.getWalkExp();
		d_runExp2 = d_dummy2.getRunExp();
		d_danceExp2 = d_dummy2.getDanceExp();
		d_walkUtil2 = d_dummy2.getWalkUtil();
		d_runUtil2 = d_dummy2.getRunUtil();
		d_danceUtil2 = d_dummy2.getDanceUtil();
		d_ci2 = d_dummy2.clusterIndex();
		
		d_leadercnt2 = d_dummy2.getCntVisibleLeaders();
		d_walkingLeaders2 = d_dummy2.getNumWalkingLeaders();
		d_runningLeaders2 = d_dummy2.getNumRunningLeaders();
		d_dancingLeaders2 = d_dummy2.getNumDancingLeaders();
		
		/* agent dummy 3 */
		d_x3 = d_dummy3.getMyPosx();
		d_y3 = d_dummy3.getMyPosy();
		d_arousal3 = d_dummy3.getArousalLevel();
		d_behaviour3 = d_dummy3.getObservableBNum();
		d_idDom3 = d_dummy3.getIDdominance(); 
		d_socDom3 = d_dummy3.getSocDominance();
		d_safeDom3 = d_dummy3.getSafeDominance();
		d_subsDom3 = d_dummy3.getSubsDominance();
		d_walkAct3 = d_dummy3.getActivationBruleWalk();
		d_runAct3 = d_dummy3.getActivationBruleRun();
		d_danceAct3 = d_dummy3.getActivationBruleDance();
		d_walkExp3 = d_dummy3.getWalkExp();
		d_runExp3 = d_dummy3.getRunExp();
		d_danceExp3 = d_dummy3.getDanceExp();
		d_walkUtil3 = d_dummy3.getWalkUtil();
		d_runUtil3 = d_dummy3.getRunUtil();
		d_danceUtil3 = d_dummy3.getDanceUtil();
		d_ci3 = d_dummy3.clusterIndex();
		
		d_leadercnt3 = d_dummy3.getCntVisibleLeaders();
		d_walkingLeaders3 = d_dummy3.getNumWalkingLeaders();
		d_runningLeaders3 = d_dummy3.getNumRunningLeaders();
		d_dancingLeaders3 = d_dummy3.getNumDancingLeaders();
		
	}
	
	/**
	 * @return the d_x1
	 */
	public double getD_x1() {
		return d_x1;
	}


	/**
	 * @return the d_x2
	 */
	public double getD_x2() {
		return d_x2;
	}


	/**
	 * @return the d_x3
	 */
	public double getD_x3() {
		return d_x3;
	}


	/**
	 * @return the d_y1
	 */
	public double getD_y1() {
		return d_y1;
	}


	/**
	 * @return the d_y2
	 */
	public double getD_y2() {
		return d_y2;
	}


	/**
	 * @return the d_walkingLeaders1
	 */
	public double getD_walkingLeaders1() {
		return d_walkingLeaders1;
	}


	/**
	 * @return the d_walkingLeaders2
	 */
	public double getD_walkingLeaders2() {
		return d_walkingLeaders2;
	}


	/**
	 * @return the d_walkingLeaders3
	 */
	public double getD_walkingLeaders3() {
		return d_walkingLeaders3;
	}

	/**
	 * @return the d_runningLeaders1
	 */
	public double getD_runningLeaders1() {
		return d_runningLeaders1;
	}


	/**
	 * @return the d_runningLeaders2
	 */
	public double getD_runningLeaders2() {
		return d_runningLeaders2;
	}


	/**
	 * @return the d_runningLeaders3
	 */
	public double getD_runningLeaders3() {
		return d_runningLeaders3;
	}


	/**
	 * @return the d_dancingLeaders1
	 */
	public double getD_dancingLeaders1() {
		return d_dancingLeaders1;
	}


	/**
	 * @return the d_dancingLeaders2
	 */
	public double getD_dancingLeaders2() {
		return d_dancingLeaders2;
	}


	/**
	 * @return the d_dancingLeaders3
	 */
	public double getD_dancingLeaders3() {
		return d_dancingLeaders3;
	}

	

	/**
	 * @return the d_y3
	 */
	public double getD_y3() {
		return d_y3;
	}


	/** 
	 * Returns the current sample mean
	 * @return
	 */
	public double getMeanAgentDistance(){
		return d_meanAgentDistance;
	}
	
	/** 
	 * Returns the current sample mean
	 * @return
	 */
	public double getVarAgentDistance(){
		return d_varAgentDistance;
	}

	
	/**
	 * @return run length
	 */
	public double getRunLength(){
		return d_runlength;
	}
	
	/**
	 * @return crowd size of this run
	 */
	public double getCrowdSize(){
		return d_crowdsize;
	}
	
	/**
	 * @return seed of this run
	 */
	public double getSeed(){
		return d_seed;
	}
	
	/**
	 * @return distribution for the safety preference setting in this run
	 */
	public String getSafetyDist(){
		return d_safety_dist;
	}

	/**
	 * the experiment condition of the density
	 * 1 : low (1)
	 * 2 : medium (3)
	 * 3 : high (5.5)
	 * 
	 * @return the experiment condition of the density
	 * 
	 */
	public int getDensityCondition() {
		return d_exp_dens;
	}


	/**
	 * the experiment condition of the safety preference
	 * 1 : low safety preference
	 * 2 : control condition, medium safety pref
	 * 3 : high safety preference
	 * 
	 * @return the experiment condition of the safety preference
	 * 
	 */
	public int getSafetyPrefCondition() {
		return d_exp_safe;
	}

	/**
	 * the experiment condition of the leadership
	 * 0 : zero (0) leaders (0)
	 * 1 : low (25) leaders (1:28)
	 * 2 : medium (50) leaders (1:14)
	 * 3 : high (100) leaders (1:7)
	 * 
	 * @return the experiment condition of the leadership setting
	 * 
	 */
	public int getLeadershipCondition() {
		return d_exp_leadership;
	}

	/**
	 * @return the d_dummy1
	 */
	public CrowdAgent getD_dummy1() {
		return d_dummy1;
	}


	/**
	 * @return the d_dummy2
	 */
	public CrowdAgent getD_dummy2() {
		return d_dummy2;
	}


	/**
	 * @return the d_dummy3
	 */
	public CrowdAgent getD_dummy3() {
		return d_dummy3;
	}


	/**
	 * @return the d_arousal1
	 */
	public double getD_arousal1() {
		return d_arousal1;
	}


	/**
	 * @return the d_arousal2
	 */
	public double getD_arousal2() {
		return d_arousal2;
	}


	/**
	 * @return the d_arousal3
	 */
	public double getD_arousal3() {
		return d_arousal3;
	}


	/**
	 * @return the d_idDom1
	 */
	public double getD_idDom1() {
		return d_idDom1;
	}


	/**
	 * @return the d_idDom2
	 */
	public double getD_idDom2() {
		return d_idDom2;
	}


	/**
	 * @return the d_idDom3
	 */
	public double getD_idDom3() {
		return d_idDom3;
	}


	/**
	 * @return the d_socDom1
	 */
	public double getD_socDom1() {
		return d_socDom1;
	}


	/**
	 * @return the d_socDom2
	 */
	public double getD_socDom2() {
		return d_socDom2;
	}


	/**
	 * @return the d_socDom3
	 */
	public double getD_socDom3() {
		return d_socDom3;
	}


	/**
	 * @return the d_safeDom1
	 */
	public double getD_safeDom1() {
		return d_safeDom1;
	}


	/**
	 * @return the d_safeDom2
	 */
	public double getD_safeDom2() {
		return d_safeDom2;
	}


	/**
	 * @return the d_safeDom3
	 */
	public double getD_safeDom3() {
		return d_safeDom3;
	}


	/**
	 * @return the d_subsDom1
	 */
	public double getD_subsDom1() {
		return d_subsDom1;
	}


	/**
	 * @return the d_subsDom2
	 */
	public double getD_subsDom2() {
		return d_subsDom2;
	}


	/**
	 * @return the d_subsDom3
	 */
	public double getD_subsDom3() {
		return d_subsDom3;
	}


	/**
	 * @return the d_walk1
	 */
	public double getD_walk1() {
		return d_walkAct1;
	}


	/**
	 * @return the d_walk2
	 */
	public double getD_walk2() {
		return d_walkAct2;
	}


	/**
	 * @return the d_walk3
	 */
	public double getD_walk3() {
		return d_walkAct3;
	}


	/**
	 * @return the d_run1
	 */
	public double getD_run1() {
		return d_runAct1;
	}


	/**
	 * @return the d_run2
	 */
	public double getD_run2() {
		return d_runAct2;
	}


	/**
	 * @return the d_run3
	 */
	public double getD_run3() {
		return d_runAct3;
	}


	/**
	 * @return the d_dance1
	 */
	public double getD_dance1() {
		return d_danceAct1;
	}

	/**
	 * @return the d_dance2
	 */
	public double getD_dance2() {
		return d_danceAct2;
	}


	/**
	 * @return the d_dance3
	 */
	public double getD_dance3() {
		return d_danceAct3;
	}


	/**
	 * @return the d_ci1
	 */
	public double getD_ci1() {
		return d_ci1;
	}


	/**
	 * @return the d_ci2
	 */
	public double getD_ci2() {
		return d_ci2;
	}


	/**
	 * @return the d_ci3
	 */
	public double getD_ci3() {
		return d_ci3;
	}


	public double getBehaviour1(){
		return d_behaviour1;
	}

	public double getBehaviour2(){
		return d_behaviour2;
	}
	public double getBehaviour3(){
		return d_behaviour3;
	}

	/**
	 * @return the d_leadercnt1
	 */
	public double getD_leadercnt1() {
		return d_leadercnt1;
	}


	/**
	 * @return the d_leadercnt2
	 */
	public double getD_leadercnt2() {
		return d_leadercnt2;
	}


	/**
	 * @return the d_leadercnt3
	 */
	public double getD_leadercnt3() {
		return d_leadercnt3;
	}

	/**
	 * @return the d_walkExp1
	 */
	public double getD_walkExp1() {
		return d_walkExp1;
	}


	/**
	 * @return the d_walkExp2
	 */
	public double getD_walkExp2() {
		return d_walkExp2;
	}


	/**
	 * @return the d_walkExp3
	 */
	public double getD_walkExp3() {
		return d_walkExp3;
	}


	/**
	 * @return the d_walkUtil1
	 */
	public double getD_walkUtil1() {
		return d_walkUtil1;
	}


	/**
	 * @return the d_walkUtil2
	 */
	public double getD_walkUtil2() {
		return d_walkUtil2;
	}


	/**
	 * @return the d_walkUtil3
	 */
	public double getD_walkUtil3() {
		return d_walkUtil3;
	}


	/**
	 * @return the d_runExp1
	 */
	public double getD_runExp1() {
		return d_runExp1;
	}


	/**
	 * @return the d_runExp2
	 */
	public double getD_runExp2() {
		return d_runExp2;
	}


	/**
	 * @return the d_runExp3
	 */
	public double getD_runExp3() {
		return d_runExp3;
	}


	/**
	 * @return the d_danceExp1
	 */
	public double getD_danceExp1() {
		return d_danceExp1;
	}


	/**
	 * @return the d_danceExp2
	 */
	public double getD_danceExp2() {
		return d_danceExp2;
	}


	/**
	 * @return the d_danceExp3
	 */
	public double getD_danceExp3() {
		return d_danceExp3;
	}


	/**
	 * @return the d_runUtil1
	 */
	public double getD_runUtil1() {
		return d_runUtil1;
	}


	/**
	 * @return the d_runUtil2
	 */
	public double getD_runUtil2() {
		return d_runUtil2;
	}


	/**
	 * @return the d_runUtil3
	 */
	public double getD_runUtil3() {
		return d_runUtil3;
	}


	/**
	 * @return the d_danceUtil1
	 */
	public double getD_danceUtil1() {
		return d_danceUtil1;
	}


	/**
	 * @return the d_danceUtil2
	 */
	public double getD_danceUtil2() {
		return d_danceUtil2;
	}


	/**
	 * @return the d_danceUtil3
	 */
	public double getD_danceUtil3() {
		return d_danceUtil3;
	}
	
}

	
