package crowdmodel;

import java.awt.Polygon;

import org.hsqldb.lib.Set;

import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.parameter.Parameters;
import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.grid.GridPoint;

/**
 * The Constants class defines all the constants that
 * are not necessarily belonging to another class but
 * to the project. For instance for initialisation, or
 * possible options in specifying a variable.
 * 
 * This involves:
 * - World constants: width, height, translation sim2real world
 * - Motion constants: different displacement velocities
 * - Scenario dependent constants: areas, points of interest
 * 
 * @author Nanda Wijermans
 *
 */

public class Constants {

/*
 * 	World constants
 *  Constants related to the world agents and other objects reside in.
 *
 *  Link sim2real world:
 *  Physical space is related to a real physical space
 *  1 grid cell = 40x40cm (0.4x0.4m)
 *  grid of 40x40 = area of 16x16m = 256m^2
 *  based on average size of humans (50x30cm||0,15m^2) and the preference for a square
 *  instead of a rectangle in space & computations (Keith Still, 2000 | De menselijke maat, 1994 | Ulrich Weidmann, 1993)
 *  
 *  World width and height are default values, however can be overridden by the user.
 *  
 */
	public static final int WORLD_WIDTH = 40; 		// The x dimension of the physical space
	public static final int WORLD_HEIGHT = 40; 		// The y dimension of the physical space
	public static final int WORLD_MAX_DIST = 57;	// ceiling(sqrt(world_width^2 + world_height^2))
	
	public static final int UNIT_MOVEMENT = 1;   	// link with realness = 0.4 m; need to check
	public static final int DEGREES = 360;
	public static final int MODULO = 90;			// given the heading one needs to get the right angle (alpha) for calculation of x1y1 given heading and x0y0
  public static final double METER2UNIT = 2.5; 	// 1 meter corresponds with 2.5 simulation units 
	public static final double UNIT2METER = 0.4;	// 1 unit corresponds with 0.4 meters
	public static final double HUMAN_DIAMETER = 1;	// link with real = 0.4 m 
	public static final double HUMAN_DANCE_DIAMETER = 1.5;  // link with real = 0.6 m (educated guess)

	public static final double MAX_DENSITY = 10;	// the maximal density  TODO add arguments for this number
	
	public static final String ID_AGENT_CONTEXT = "agentContext";	// The typeID of the agent subcontext
	
/*
 * Perception constants	
 */
	
	public static final int GAZE_WIDTH = 180;  // in degrees 
	public static final int GAZE_DEPTH = 2;    // in simulation units

	/*
	 * 	nearby = # persons in surrounding
	 *  maxNearby =  max. possible persons in surrounding = area of 1 person / area of the surrounding
	 * personArea = area of 1 person = 0.4 * 0.4 = 0.16 m^2
	 * surroundingArea = area of the surrounding = area of a circle =  pi * r^2 = pi * gaze-depth^2 =  pi * (2 * unit2meter)^2 = 3,14 * (2 * 0,4)^2 = 2,01 
	 * maxSurrounding = 2,01/0.16 = 12,6 
	 * 
	*/
	
	public static final double MAX_SURROUNDING = 12.6; // max # persons in the surrounding Area
	
	/* bodily constants */
	public static final double LOW2M_AROUSAL = 0.3;
	public static final double MEDIUM2H_AROUSAL = 0.7;

	public static final String BLADDER = "bladder";
	public static final String STOMACH = "stomach";
	
	/* Cognitive constants */
	public static final double BRULE_RETRIEVE_TIME = 0.5;
	
	
	/* Default initialisation settings 
	 * Used for default initialisation of variables that can also be user or batch-defined
	 * See DataSets for the variables	*/
	public static final int DEFAULT_CROWDSIZE = 20;
	public static final int DEFAULT_DENSITY = 4;
	public static final int DEFAULT_NR_LEADERS = 0;
	public static final int DEFAULT_PEACE_LEADERS = 20;
	public static final int DEFAULT_WAR_LEADERS = 100 - DEFAULT_PEACE_LEADERS;

	public static final double DEFAULT_INIT_IDENTITY = 0.0;
	public static final double DEFAULT_INIT_SOCIAL = 0.0;
	public static final double DEFAULT_INIT_SAFETY = 0.5;
	
	public static final double DEFAULT_PREF_IDENTITY = 0.7;
	public static final double DEFAULT_PREF_SOCIAL = 0.8;
	public static final double DEFAULT_PREF_SAFETY = 0.3;
	public static final double DEFAULT_PREF_SUBSISTENCE = 0.8;	
	
	
	public static final int NEUTRAL_LEADER = 0; 	// indicates the type direction a leader influences  
	public static final int PEACEFUL_LEADER = 1;
	public static final int AGGRESSIVE_LEADER = 2;
			
	/* Numeric values for strings/names */
	public static final int GOAL_IDENTITY = 0;
	public static final int GOAL_SOCIAL = 1;
	public static final int GOAL_SAFETY = 2;
	public static final int GOAL_SUBSISTENCE = 3;
	
	public static final int IN_GROUP = 0;
	public static final int OUT_GROUP = 1;
	public static final int UNKNOWN = 2;


	/* String values for names or labels */
	public static final String GOAL_NAME_IDENTITY = "identity";
	public static final String GOAL_NAME_SOCIAL = "social";
	public static final String GOAL_NAME_SAFETY = "safety";
	public static final String GOAL_NAME_SUBSISTENCE = "subsistence";

	
	/* String values for behaviour names/labels */
	public static final String B_NAME_INHIBITED = "inhibited";
	public static final String B_NAME_WALK = "walk";
	public static final String B_NAME_RUN = "run";
	public static final String B_NAME_DANCE = "dance";
	public static final String B_NAME_STANDUP = "standup";
	public static final String B_NAME_SITDOWN = "sitdown";
	public static final String B_NAME_KNEEL = "kneel";
	public static final String B_NAME_LIEDOWN = "liedown";
	
	public static final String B_NAME_CARRY = "carry";
//	public static final String B_NAME_CARRY_OBJECT = "carryObject";
//	public static final String B_NAME_CARRY_PERSON = "carryPerson";
	public static final String B_NAME_LIFT = "lift";
	public static final String B_NAME_PASS = "pass";
	public static final String B_NAME_STRIKE = "strike";
	public static final String B_NAME_THROW = "throw";
	public static final String B_NAME_PULL = "pull";
	public static final String B_NAME_PUSH = "push";	
	public static final String B_NAME_EMBRACE = "embrace";
	public static final String B_NAME_RESTRAIN = "restrain";
	public static final String B_NAME_CLAPPING = "clapping";
	public static final String B_NAME_SNAPPING = "snapping";
	public static final String B_NAME_OTHER = "other";
	public static final String B_NAME_NOTHING = "nothing";
	
	/* String values for behaviour expectation names/labels*/
	public static final String F_NAME_EWALK = "walkExp";
	public static final String F_NAME_ERUN = "runExp";
	public static final String F_NAME_EDANCE = "danceExp";
	
	public static final String F_NAME_ESTANDUP = "standupExp";
	public static final String F_NAME_ESITDOWN = "sitdownExp";
	public static final String F_NAME_EKNEEL = "kneelExp";
	public static final String F_NAME_ELIEDOWN = "liedownExp";
	
	public static final String F_NAME_ECARRY = "carryExp";
//	public static final String F_NAME_ECARRY_OBJECT = "carryObject";
//	public static final String F_NAME_ECARRY_PERSON = "carryPerson";
	public static final String F_NAME_ELIFT = "liftExp";
	public static final String F_NAME_EPASS = "passExp";
	public static final String F_NAME_ESTRIKE = "strikeExp";
	public static final String F_NAME_ETHROW = "throwExp";
	public static final String F_NAME_EPULL = "pullExp";
	public static final String F_NAME_EPUSH = "pushExp";	
	public static final String F_NAME_EEMBRACE = "embraceExp";
	public static final String F_NAME_ERESTRAIN = "restrainExp";
	public static final String F_NAME_ECLAPPING = "clappingExp";
	public static final String F_NAME_ESNAPPING = "snappingExp";
	
	public static final String F_NAME_EOTHER = "otherExp";
	public static final String F_NAME_ENOTHING = "nothingExp";
	
	
	/* String values for person names/labels */
	public static final String P_NAME_SELF = "self";
	public static final String P_NAME_UNKNOWN = "unknown";
	public static final String P_NAME_FRIEND = "friend";
	public static final String P_NAME_LEADER = "leader";
	public static final String P_NAME_INGROUP = "ingroup";
	public static final String P_NAME_OUTGROUP = "outgroup";
	
	
	/* String values for distribution names */
	public static final String DIST_UNIFORM = "uniform";
	public static final String DIST_NORMAL = "normal";
	public static final String DIST_CUSTOM = "customised";
	public static final String DIST_LEFT_SKEW = "left-skew";
	public static final String DIST_RIGHT_SKEW = "right-skew";
	
	
/* Motion constants
 * Different options in velocity
 */	
	public static final double V_DEFAULT = 1;   // default velocity: moving 1 simulation unit (distance) per time step	
	public static final double V_WALK = 1;
	public static final double V_RUN = 1.2;
	
/*
 * Scenario constants
 * Defines default scenario dependent areas and point of interests (poi), can be overruled or complemented by a user.
 * An area is defined by the corner points is is build from.
 * 
 * Festival scenario: stage, bar, toilet area
 *
 * stage area: P1(25,0); P2(75,0); P3(25,75); P4(75,100)
 * these are relative values given the width/height of the world
 * 
 */
	
	public static int stagex0 = (int)Math.floor(0.25 * (WORLD_WIDTH)); // 9
	public static int stagey0 = (int)Math.floor(0.75 * (WORLD_HEIGHT)); // 29
	public static int stagex1 = (int)Math.floor(0.75 * (WORLD_WIDTH));  // 29
	public static int stagey1 = WORLD_HEIGHT - 1; //39
	
	public static int[] stageX = new int[]{stagex0, stagex0, stagex0, stagex1};
	public static int[] stageY = new int[]{stagey0, stagey1, stagey0, stagey1};
	
	public static NdPoint POI_STAGE = new NdPoint((stagex0 * 2), stagey0, 0);
	
/* bar area : P1(90,0); P2(100,0); P3(90,40); P4(100,40)
 * these are relative values given the width/height of the world */
	public static int barx0 = (int)Math.floor(0.90 * (WORLD_WIDTH)); 
	public static int barx1 = WORLD_WIDTH - 1;
	public static int bary0 = 0;
	public static int bary1 = (int)Math.floor(0.40 * (WORLD_HEIGHT));

	public static int[] barX = new int[]{barx0, barx0, barx0, barx1};
	public static int[] barY = new int[]{bary0, bary1, bary0, bary1};

	public static NdPoint POI_BAR = new NdPoint(35.0, 2.0, 0);
	public static NdPoint POI_BAR1 = new NdPoint(35.0, 6.0, 0);
	public static NdPoint POI_BAR2 = new NdPoint(35.0, 10.0, 0);

	public static GridPoint GPOI_BAR = new GridPoint(35, 2, 0);
	public static GridPoint GPOI_BAR1 = new GridPoint(35, 6, 0);
	public static GridPoint GPOI_BAR2 = new GridPoint(35, 10, 0);

	/* toilet area: P1(0,0); P2(10,0); P3(0,15); P4(10,15) 
	 * these are relative values given the width/height of the world */
	public static int toiletx0 = 0;
	public static int toiletx1 = (int)Math.floor(0.10 * (WORLD_WIDTH)); 
	public static int toilety0 = 0;
	public static int toilety1 = (int)Math.floor(0.15 * (WORLD_HEIGHT));

	public static int[] toiletX = new int[]{toiletx0, toiletx0, toiletx0, toiletx1};
	public static int[] toiletY = new int[]{toilety0, toilety1, toilety0, toilety1};
	
	public static NdPoint POI_TOILET = new NdPoint(5.0, 1.0, 0);
	public static NdPoint POI_TOILET1 = new NdPoint(5.0, 3.0, 0);
	public static NdPoint POI_TOILET2 = new NdPoint(5.0, 5.0, 0);
	
	public static GridPoint GPOI_TOILET = new GridPoint(5, 1, 0);
	public static GridPoint GPOI_TOILET1 = new GridPoint(5, 3, 0);
	public static GridPoint GPOI_TOILET2 = new GridPoint(5, 5, 0);
}
