package crowdmodel.agents;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Vector;

import crowdmodel.CObject;
import crowdmodel.Constants;
import crowdmodel.gui.GUIparams;
import crowdmodel.utils.*;
import repast.simphony.annotate.AgentAnnot;
import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunState;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.parameter.Parameter;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.graph.Network;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridDimensions;
import repast.simphony.space.grid.GridPoint;


/**  
 * 
 * Base class for a crowd agent (individual)
 * An agent in this simulation refers to an computational individual in a crowd.
 * An agent consists out of physiological properties and a memory.
 * 
 * @author Nanda Wijermans
 *
 */

@AgentAnnot(displayName = "CrowdAgent")

public class CrowdAgent extends CObject {
	
	private final int d_agentID;
	private final Context<CrowdAgent> d_agent_context;
	public final Memory d_memory;
	public final Physiology d_physiology;
	
	private double d_internalTime;  // [0,10] directly coupled to the arousal level of an individual
/*
 * Variables that support local perception of the world
 * so it is possible that the world has changed however these changes will not
 * have any effect as they aren't observed or processed yet.
 */
	
	final private double d_visibleArea;
	
	private NdPoint d_mypos;
	private double d_heading;
	private double d_occupation_diameter;
	
	private boolean d_leader; 
	private int d_leaderType;
	public boolean d_groupA;

	private boolean d_cannotMove;
	private boolean d_satisfied;
	
	private int d_numVisibleOthers;
	private Vector<CrowdAgent> d_visibleOthers;
	private int d_numOthers; // number of agents that surround the current agent (Moores neighborhood)
	private Vector<CrowdAgent> d_nearbyAgents; //agent surrounding the current agent (360 degrees)
	private double d_density;    //local density, the density that is perceived by the individual
	private int d_numVisibleLeaders; // number of leaders that an agent perceives
	
	private String d_POI;	// current point of interest
	public String d_myBehaviour;  // behaviour an individual is currently exposing
	public int d_walk;
	public int d_run;
	public int d_dance;
	
	public int d_sameAsLeader;   // 0 = no ; 1 = yes ; it is no true/false boolean to be able to SUM it
	public double d_walkUtil;
	public double d_runUtil;
	public double d_danceUtil;
	
	/* 
	 * vars for testing
	 */
	
	private boolean d_dummy;  // whether this agent is a dummy or not
	private int d_dummyColor;  // colour that this dummy will be displayed with (0 for non-dummies)
	
	private double d_prevIdSatis, d_prevSocSatis, d_prevSafeSatis, d_prevSubsSatis;
	private NdPoint d_prevPos;
	private Vector<CrowdAgent> d_prevNearbyAgents, d_prevAgentsInSight;

	public double d_distNearestAgent;
	public CrowdAgent d_nearestAgent;
	
	public NdPoint heading2point;
	
	/*
	 * vars for output
	 */
	public int d_densityCondition;
	public int d_safetyCondition;
	public int d_leadershipCondition;
	
	private int d_numWalkingLeaders, d_numRunningLeaders, d_numDancingLeaders; // behaviour an observed leader shows
	
	
	/**
	 *  This constructor creates and initialises an agent
	 */
	public CrowdAgent(int agentID, Context<CObject> worldContext, Context<CrowdAgent> agentContext){
		
		d_cannotMove = false;
		d_satisfied = true;
		
		/* subsystems of the crowd agent */	

		d_agentID = agentID;
		d_id = agentID;  
		
		d_world_context = worldContext;
		d_agent_context = agentContext;   /* gives the context an object resides in. In this case the subcontext here */
					
		d_mypos = new NdPoint(0,0,0); //dummy position 
		d_prevPos = d_mypos;
		heading2point = new NdPoint(0,0,0);
		
		/* perception */	
		
		d_visibleOthers = new Vector<CrowdAgent>();
		d_numVisibleOthers = 0;
		d_nearbyAgents = new Vector<CrowdAgent>();
		d_distNearestAgent = 100;
		d_nearestAgent = null;
		d_density = 0;
		d_numVisibleLeaders = 0;
		d_numWalkingLeaders = 0; // behaviour an observed leader shows
		d_numRunningLeaders = 0;
		d_numDancingLeaders = 0;
		d_sameAsLeader = 0;
		
		
		/* physiology */
		d_physiology = new Physiology();
		
		d_heading = Math.random() * Constants.DEGREES;
		d_occupation_diameter = Constants.HUMAN_DIAMETER;
		
		double r =  Constants.GAZE_DEPTH * Constants.UNIT2METER; // in meters
		d_visibleArea =  0.5 * Math.PI * Math.pow(r, 2.0); // area of an 1/2 circle	(1/2 * pi * r^2) [m2]
		
		/* cognitive part */
		d_memory = new Memory();
		d_leader = false;
		d_leaderType = 0;
		d_groupA = true;
		d_internalTime = 10; // max amount of time for behaviour selection
		d_myBehaviour = Constants.B_NAME_NOTHING;  
		d_walk = 0;
		d_run = 0;
		d_dance = 0;
		
		d_walkUtil = 0;
		d_runUtil = 0;
		d_danceUtil = 0;
	
		d_prevIdSatis = 0;
		d_prevSocSatis = 0;
		d_prevSafeSatis = 0;
		d_prevSubsSatis = 0;
		
		d_dummy = false;
		d_dummyColor = 0;

	}
	
	/**
	 * Initialisation function of the agents. 
	 * - Sets within the memory to which agent it belongs. 
	 * - Dito for the memory elements. So every memoryElement needs to know to what memory it belongs and 
	 * what agent. (This because the memory is a context on it's own and not part of the overall context 
	 * as it is only accessible for and owned by an agent.)
	 * - Sets the social scenario (tells the agents who their friends are by adding PersonFacts to memory)
	 * - Sets the initial position if the current (random) position is not on walkable terrain and not in the
	 * vicinity of friends
	 * 
	 */	
	public void init(int exp_dens, int exp_safe, int exp_leader){
		d_physiology.setThisAgent(this);
		d_memory.setThisAgent(this);
		d_memory.setElementsWithAgentAndMemory();
		
		d_densityCondition = exp_dens;
		d_safetyCondition  = exp_safe;
		d_leadershipCondition = exp_leader;
		
		Network<CrowdAgent> socNetwork = (Network<CrowdAgent>) d_agent_context.getProjection("Social Network");
		d_memory.setSocialScenario(socNetwork);
		initPos(socNetwork); 
		
		double initSubs = d_physiology.getDomPhysioLevel();
		d_memory.initSubs(initSubs);
		
		//for testing whether memory is filled
//		String elements = "";
//		Set<Map.Entry<Integer, MemoryElement>> memorySet = d_memory.getMemoryElements().entrySet();
//		for (Map.Entry<Integer, MemoryElement> element : memorySet) {
//		 	elements += element.getValue().getName() + ", ";
//		}
//			System.out.println(elements);
	}
	
	/**
	 * Initially agents are placed randomly in the world. 
	 * However based on the scenario some positions are non-walkable areas
	 * so those agents are relocated.<br>
	 * 
	 * Furthermore, agents that belong to a group, are placed in each others
	 * vicinity as this group representation indicates the persons who the
	 * agent joined to come to this event.
	 * <br><br>
	 * 
	 */
	
	public void initPos(Network socNetwork){

		ContinuousSpace space = (ContinuousSpace) d_world_context.getProjection("Continuous Space");
		Grid patch = (Grid) d_world_context.getProjection("Simple Grid");
		
		NdPoint target = space.getLocation(this);
		GridPoint targetPatch = Misc.nd2gridpoint(target);
		
		/* move to a walkable area */ 
		while (!Misc.walkablePatch(d_world_context, this, target)){
			target = getRandomLocation();
		}
		

			targetPatch = Misc.nd2gridpoint(target);
			
			if (target != d_mypos) {
			// Move the agent on the space by one unit according to its new heading
			space.moveTo(this, target.getX(), target.getY(), 0);
			// Move the agent to its new patch (note the patch may not actually change)
			patch.moveTo(this, targetPatch.getX(), targetPatch.getY()); 
			d_mypos = target;
			}
		// else : no need to change the initial position
		
	}	
	
	/**
	 * The step function defines what an agent does every simulation step/tick.
	 * Here the main processes are called: perception and behaving.
	 * 
	 * This function needs to be called first by the scheduler, that is why priority has a high number.
	 * This is relative to the other functions that are assigned to the scheduler, which are
	 * currently Cluster.updateClusters() and Tests.<nameTest>.testIt()
	 */
	
	
@ScheduledMethod(start = 0, interval = 1, priority = 13.0, shuffle = false) // high priority (needs to be done first)= high number; low priority = low number
	public void updateTestVars(){
		//System.out.println("CrowdAgent.updateTestVars() prio 13");
		d_prevIdSatis = getIDlevel();
		d_prevSocSatis = getSoclevel();
		d_prevSafeSatis = getSafelevel();
		d_prevSubsSatis = getSubslevel();
		ContinuousSpace space = (ContinuousSpace) d_world_context.getProjection("Continuous Space");
		d_prevPos = space.getLocation(this);
		d_prevNearbyAgents = Misc.agentsInCone(d_world_context, this, 360, 1);
		d_prevAgentsInSight = Misc.agentsInCone(d_world_context, this, Constants.GAZE_WIDTH, Constants.GAZE_DEPTH);
	}


/**
 * In this model perception is 'hard-coded', this means that no intelligent
 * algorithm is behind perception. Instead we use the knowledge at hand to define
 * what is observed and how this effects an individual. The internal world can be
 * affected in different ways. Perception is used in the broad sense of the word meaning
 * that everything that can be sensed, including internal updates based on metabolism changes.
 * 
 */
@ScheduledMethod(start = 0, interval = 1, priority = 11.0, shuffle = false) // high priority = high number; low priority = low number	
public void perceive(){
	//System.out.println("CrowdAgent.perceive(): prio 11, id = " + d_agentID);
	resetCounters();
	getInfo();
	primeMemory();
	updatePhysiology();
	updateGoals();
	updateFacts();
}

/**
 * A behaviour is chosen and executed. The behaviour selection is based on the 
 * current internal settings: dominance of goals, activation of the behaviours
 * 
 * Behaviour is executed = the effects it has in the real world are registered
 * need to check here how one deals with multiple effect on the same object in the real world.
 * necessary to notice this and let the changes override each other ruled by the order of the agent 
 * in which they behave
 * 
 * Behaviour selection is depending on time. The time an individual has to choose a behaviour
 * is implemented as a direct link between the arousal level and internal time to choose and compare
 * behaviours with each other. This gives rise to automatic/heuristic vs. reasoned action in a continuous range.
 * The general procedure of behaviour selection is as follows: <br>
 * 0) set time (done by resetInternalTime())<br>
 * 1) set current B (behaviour with the highest activation (and if considered the one that has the highest satisfaction of current goal dominance)<br>
 * 2) while (time > 0 ) { 
 *        compare B with the next (= activation based) behaviour)<br>
 *        update B accordingly (B will be the one with the highest utility (= highest satisfaction of goals in the current tick)} <br>
 * 3) execute B 
 *
 * The steps that cost time are:
 * - retrieving the next behaviour (getNextActive()): time costs = a constant value = Constants.BRULE_RETRIEVE_TIME <br>
 * - retrieving a fact from memory to calculate the utility (compare()): time costs = activation-based function (bfact.retrievalTime())
 */

@ScheduledMethod(start = 0, interval = 1, priority = 5.0, shuffle = false) // high priority = high number; low priority = low number	
	public void ooBehave() {

	/* behaviour selection */
	if (d_memory.getDomGoal().getD_dominance() != 0.0){    // only choose a behaviour if at least one of the goals is not satisfied	
		setSatisfied(false);
		d_memory.setBehaviourIterator();
	
		resetInternalTime();
		BehaviourRule b = d_memory.getNextActiveB();
			
		while (d_internalTime > 0 & d_memory.hasNextB()){
			b = compare(b);
		}

	/* behaviour execution */
		b.execute();
		setObservableB(b.getName());
		
		// used for testing whether social walking is working properly
		if (b instanceof LocomotionRule){
			LocomotionRule loco = (LocomotionRule) b;
			heading2point = loco.getHpos();
		}
		
		if (b instanceof LocomotionRule){
			Goal g = d_memory.getDomGoal();
			if (g.getName().equalsIgnoreCase(Constants.GOAL_NAME_SOCIAL)){
				LocomotionRule locoB = (LocomotionRule) b;
				setNearestAgent(locoB.getNearestAgent());
				setDistNearestAgent(locoB.getMostClose());	
			}
		}
	}	
	else {
		setSatisfied(true);
		leaveQueue();
		setObservableB("nothing");
	}
}


	/**
	 * Makes sure agents don't stand on or in front of the bar or toilet after they are completely satisfied
	 * 
	 */
	private void leaveQueue(){
		Context context = RunState.getInstance().getMasterContext();
		Grid patch = (Grid) context.getProjection("Simple Grid");
		ContinuousSpace space = (ContinuousSpace) context.getProjection("Continuous Space");
		
		GridPoint patchPos = Misc.nd2gridpoint(space.getLocation(this));
		GridPoint pos = null;
		double dist = 20;
		
		int x = patchPos.getX();
		int y = patchPos.getY();
		if (x == 35 & (y == 2 || y == 6 || y == 10) || (x == 5 & (y == 1 || y == 3 || y == 5))){
			pos = Misc.findWalkableNeighborPatch(d_world_context, patchPos);
		} else if (space.getDistance(d_mypos, Constants.POI_TOILET) < dist ||
						space.getDistance(d_mypos, Constants.POI_TOILET1) < dist ||
						space.getDistance(d_mypos, Constants.POI_TOILET2) < dist ) {
			if (Misc.crowdedness(space, d_mypos, 1) > 1){
				pos = Misc.findWalkableNeighborPatch(d_world_context, patchPos);   
			}
		} else if (space.getDistance(d_mypos, Constants.POI_BAR) < dist ||
				space.getDistance(d_mypos, Constants.POI_BAR1) < dist ||
				space.getDistance(d_mypos, Constants.POI_BAR2) < dist ) {
			if (Misc.crowdedness(space, d_mypos, 1) > 1){
				pos = Misc.findWalkableNeighborPatch(d_world_context, patchPos);   
			}	
		}
		
		if (pos != null){					
			space.moveTo(this, pos.getX(), pos.getY(), 0);
			// Move the agent to its new patch (note the patch may not actually change)
			patch.moveTo(this, pos.getX(), pos.getY()); 
		}
	}

	private void setDistNearestAgent(double dichtstebije) {
		d_distNearestAgent = dichtstebije;
	
}

	public void setNearestAgent(CrowdAgent nearest) {
		d_nearestAgent = nearest;
}

	/**
	 * Resets the counters
	 */
	
	public void resetCounters(){
		d_numVisibleLeaders = 0;
		d_numWalkingLeaders = 0;
		d_numRunningLeaders = 0;
		d_numDancingLeaders = 0;
	}
	
	
	/**
	 * Retrieves the information from the external world based on the
	 * constraints upon perception. This guarantees local and restricted perception.
	 * Plus it already sets the variables for the functions used later in perception.
	 */
		
	public void getInfo() {
	/*	information is retrieved from the world sensor settings are updated */ 
		Context context = RunState.getInstance().getMasterContext();
		ContinuousSpace space = (ContinuousSpace) context.getProjection("Continuous Space");
		d_mypos = space.getLocation(this); 
		
		d_visibleOthers = Misc.agentsInCone(context, this, Constants.GAZE_WIDTH, Constants.GAZE_DEPTH);
		d_numVisibleOthers = d_visibleOthers.size();  
		d_density = d_numVisibleOthers / d_visibleArea; // density [persons/m^2]
		d_nearbyAgents = Misc.agentsInCone(d_world_context, this, 360, 1); //moores neighborhood
	
		
	/* extract the info from the environment based on individual settings of perception range */
	/* info = current location, vicinity other agents, vicinity scenario dependent areas (festival: bar/toilet, stage) */

	/* info used for data output */
		d_numOthers = d_nearbyAgents.size();
}

	/**
	 * When an agents observes the world around it, information enters the cognitive system that directly affect the agent in terms
	 * of activation or dominance. Here we limit ourselves to what can be perceived with our eyes (however in future the role of
	 * audible, tactile, nasal perception should be incorporated)
	 * In this model when an agent observes objects and actions in the world they are primed, meaning their activation
	 * in memory is increased.
	 * 
	 * <ul>
	 * 	<li> Observed behaviours: primes corresponding BehaviourRules and BehaviourFacts</li>
	 *  <li> Observed persons: primes the corresponding PersonFacts. </li>
	 * </ul>
	 * 
	 * This is important as we assume that those elements that are dominant (i.e. active) in the cognitive system plays a role 
	 * in the behaviour an individual will show. Meaning that, something can be relevant but inactive and therefore not a behavioural option.
	 * 	
	 */
	public void primeMemory(){
		
		
		/*
		 * priming, self and observed behaviours and facts	
		 */

			if (d_myBehaviour != Constants.B_NAME_INHIBITED & d_myBehaviour != "nothing"){
				d_memory.getElement(d_myBehaviour).prime();   // prime behaviour
				d_memory.getElement(Misc.bName2fName(d_myBehaviour)).prime(); // prime behaviour fact
			}
				
			if (d_numVisibleOthers > 0){
				Vector<CrowdAgent> perceivedAgents = Misc.agentsInCone(d_world_context, this, Constants.GAZE_WIDTH, Constants.GAZE_DEPTH);
				Iterator<CrowdAgent> pAgents = perceivedAgents.iterator();
				while (pAgents.hasNext()){
					CrowdAgent ca = pAgents.next();
					String observedBehaviour = ca.getObservableB();
					String agentName = Constants.P_NAME_FRIEND + ca.getAgentID();
					
					if (observedBehaviour != Constants.B_NAME_INHIBITED & observedBehaviour != "nothing"){
						String bFact = Misc.bName2fName(observedBehaviour);
						d_memory.getElement(observedBehaviour).prime(); // primes behaviour rule
						d_memory.getElement(bFact).prime();  // primes behaviourFact
					}
					
					if (d_memory.contains(agentName)){
						d_memory.getElement(agentName).prime(); // prime personFact
					}
					if (ingroup(ca)){
						d_memory.getElement(Constants.P_NAME_INGROUP).prime(); // prime groupFact
					} 
					else {
						d_memory.getElement(Constants.P_NAME_OUTGROUP).prime();
					}	
					
				}
			}
			d_memory.step();  //updates the total activation value of the memory elements 
	}
	
 
	/**
	 * Describes the effect every time step upon physiology. Meaning the role of perception on physiology, but also the internal updates as metabolism changes the internal settings.
	 * We incorporated the following effects:
	 * <ul>
	 * <li>Density-Arousal: here we directly implement the link between the perception of density levels and the arousal level
	 * of an individual. </li>
	 * <li>Toilet-Bladder: being at a certain location representing the toilet has a direct effect on the bladder level</li>
	 * <li>Bar-Stomach: being at a certain location representing the bar has a direct effect on the stomach level</li>
	 * <li> internal update: every time step the internal settings are updated given time, this is independent from the
	 * external changes that may or may not occur and effect physiology. This is done for the physiology attributes bladder and stomach level.</li>
	 * </ul>
	 * 
	 */
	public void updatePhysiology(){
		
		ContinuousSpace space = (ContinuousSpace) d_world_context.getProjection("Continuous Space");
		Grid patch = (Grid) d_world_context.getProjection("Simple Grid");
		
		/* update physiology - arousal, bladder and stomach */ 
		
		d_physiology.updateArousal(d_density);	
		 
		GridPoint patchPos = Misc.nd2gridpoint(space.getLocation(this));
		int x = patchPos.getX();
		int y = patchPos.getY();
			
		if (x == 35 && (y == 2 || y == 6 || y == 10)){
			d_physiology.setStomachLevel(1);
			d_physiology.updateBladder();
		} 
		else if (x == 5 && (y == 1 || y == 3 || y == 5)){
			d_physiology.setBladderLevel(0);
			d_physiology.updateStomach();
		}
		else {
			d_physiology.updateBladder();
			d_physiology.updateStomach();
		}
		
	}
		
	/**
	 *  The update of the goals describe the effect the current situation has (external and internal state/perception)
	 *  has on the satisfaction of the goals an individual has. This is based on the scenario chosen, for our festival setting
	 *  we chose to following links between perception and satisfaction of goals:
	 *  <ul>
	 * <li> Identity: distance from Stage (point of interest); the closer --> the higher id satisfaction (stepfunction | range-based)</li>
	 * <li> Social: physical presence of other agents; the higher --> the higher the social satisfaction (stepfunction | range-based)</li>
	 * <li> Safety: local perceived density; the higher --> the lower the safety perception (sigmoid)</li>
	 * <li> Subsistence is not perceived subjective (only objective) via physiology</li>
	 * </ul>
	 * 
	 * In future we also want to update the expectation (facts about behaviours) based on the person who is performing a behaviour.
	 * As follows:
	 * <ul>
	 * <li>friends exibiting a behaviour x: making it increase social expectation or decrease safety expectation..</li>
	 * <li>leader exibiting a behaviour x: increases social expectation of this particular behaviour   </li>
	 * </ul>
	 */
	public void updateGoals(){
		
		ContinuousSpace space = (ContinuousSpace) d_world_context.getProjection("Continuous Space");
		Grid patch = (Grid) d_world_context.getProjection("Simple Grid");
		
		/* subsistence */
		if (GUIparams.getInstance().getSubsGoal()){
			d_physiology.setDomPhysiology();
			double subsSatis = d_physiology.getDomPhysioLevel();
			Goal subsistence = d_memory.getSubsistence();
			subsistence.setSatisfaction(subsSatis);	
		}
		
		/* safety */
		if (GUIparams.getInstance().getSafeGoal()){ //check whether the safety goal is present and thus has is influenced by perception
			Goal safety = d_memory.getSafety();
			double density = (d_density > Constants.MAX_DENSITY ? Constants.MAX_DENSITY : d_density);
			double safeSatis = (1 - (1 / (1 + Math.exp(-((density * (12 / Constants.MAX_DENSITY))-6)))));
			safety.setSatisfaction(safeSatis);
		} 
		
		/* social */
		if (GUIparams.getInstance().getSocGoal()){ //check whether the social goal is present and thus has is influenced by perception
			
			Goal social = d_memory.getSocial();
			double socSatis = social.getSatisfaction();
			if (d_nearbyAgents.size() > 0){			
				ArrayList<PersonFact> friends = d_memory.getFriends();
				boolean closeFriend = false;
				Iterator<CrowdAgent> agentsNearby = d_nearbyAgents.iterator();
				while (agentsNearby.hasNext()){
					if (friends.contains(agentsNearby.next())){
						closeFriend = true;	
					}
				}
				if (closeFriend){
					socSatis = 1;
				} else {
					socSatis += 0.3;
				}
			} else {
				socSatis -= .1;
			}
			social.setSatisfaction(socSatis);
			checkRange(social);			
		}

		/* identity */
		if (GUIparams.getInstance().getIdGoal()){  //check whether the identity goal is present and thus has is influenced by perception
			double dist = space.getDistance(d_mypos, Constants.POI_STAGE);
			Goal identity = d_memory.getIdentity();
			double idSatis = identity.getSatisfaction();
			if (dist < 3){
				idSatis += .05;
			} else if (dist < 7){
				idSatis += .01;
			} else if (dist < 20){
				idSatis -= .0025;
			} else { /* dist > 20 */
				idSatis -= .01;
			}
			identity.setSatisfaction(idSatis);
			checkRange(identity);
		}
			
		/* and lastly, dominance update */
		
		d_memory.setDomGoal();
	}
	
	/**
	 * Function that checks and when necessary corrects the range of the satisfaction value of a goal
	 * to stay between [0,1]
	 * @param goal
	 */
	
	public void checkRange(Goal goal){
		double goalSatis = goal.getSatisfaction();
		
		goalSatis = (goalSatis > 1) ? 1 : goalSatis;
		goalSatis = (goalSatis < 0) ? 0 : goalSatis;
		
		goal.setSatisfaction(goalSatis);
	}
	
	/**
	 * Facts are updated according to the current situation.
	 * This can be regarded as the effect of experience or learning in a very simple form.
	 * 
	 * Currently this only involves the differentation between the individuals one perceives.
	 * The behaviour shown by a leader will effect the expectation of this behaviour of 
	 * satisfying the social goal.
	 * 
	 * 1) the expectation of the social goal of the specific behaviour is set to 0.4
	 * 2) this value will decrease in 5 time steps to the initial value of that behaviour expectation
	 */
	
	public void updateFacts(){
		
		double leaderEffect = 0.4; // expectation value due to perceiving a leader
		final double decay = 0.05;
		
		boolean walkIsUpdated = false;
		boolean runIsUpdated = false;
		boolean danceIsUpdated = false;
		
        /* perceiving a leader */
		ArrayList<PersonFact> leaders = d_memory.getLeaders();
		Iterator<CrowdAgent> visibleAgents = d_visibleOthers.iterator();
		while (visibleAgents.hasNext()){				// for all visible agents
			CrowdAgent ca = visibleAgents.next();
			int caID = ca.d_agentID;							
			Iterator<PersonFact> myLeaders = leaders.iterator();	
			while(myLeaders.hasNext()){
				PersonFact lfact = myLeaders.next();
				if (lfact.d_personID == caID){				// if the visible other is a leader
					String myB = ca.d_myBehaviour;			// retrieve the behviour that leader shows
					if (myB != Constants.B_NAME_INHIBITED) {	// if b != inhibited change my social expectation of the type of behaviour the leader shows 
						BehaviourFact b = d_memory.getBehaviourFact(myB);
						b.setExpectancy(Constants.GOAL_NAME_SOCIAL, leaderEffect);
						++d_numVisibleLeaders;
						if (myB == Constants.B_NAME_WALK) {
							//++d_numWalkingLeaders;
							d_numWalkingLeaders = d_numWalkingLeaders + 1;
						} else if (myB == Constants.B_NAME_RUN){
							//++d_numRunningLeaders;
							d_numRunningLeaders = d_numRunningLeaders + 1;
						} else if (myB == Constants.B_NAME_DANCE){
							//++d_numDancingLeaders;
							d_numDancingLeaders = d_numDancingLeaders + 1;
						} else {
							//syst.err
						}
						walkIsUpdated = myB.equalsIgnoreCase(Constants.B_NAME_WALK);
						runIsUpdated = myB.equalsIgnoreCase(Constants.B_NAME_RUN);
						danceIsUpdated = myB.equalsIgnoreCase(Constants.B_NAME_DANCE);
					}
				}
			}
		}
		
		/* Decrease of expectation when not seeing a leader to the base value */
		if (!walkIsUpdated) {
			BehaviourFact walk = d_memory.getBehaviourFact(Constants.B_NAME_WALK);
			double socExp = walk.getExpectancy(Constants.GOAL_NAME_SOCIAL)* decay;
			double exp = (socExp < 0.3 ? 0.3 : socExp);
			walk.setExpectancy(Constants.GOAL_NAME_SOCIAL, exp);
		}
		if (!runIsUpdated) {
			BehaviourFact run = d_memory.getBehaviourFact(Constants.B_NAME_RUN);
			double socExp = run.getExpectancy(Constants.GOAL_NAME_SOCIAL)* decay;
			double exp = (socExp < 0.1 ? 0.1 : socExp);
			run.setExpectancy(Constants.GOAL_NAME_SOCIAL, exp);
		}
		if (!danceIsUpdated) {
			BehaviourFact dance = d_memory.getBehaviourFact(Constants.B_NAME_DANCE);
			double socExp = dance.getExpectancy(Constants.GOAL_NAME_SOCIAL)* decay;
			double exp = (socExp < 0.3 ? 0.3 : socExp);
			dance.setExpectancy(Constants.GOAL_NAME_SOCIAL, exp);
		}
	}
	
	

	/**
	 * Compares the current behaviour ready for execution
	 * with the next behaviour. In which the next behaviour
	 * is based on the current hierarchy among behaviours in
	 * terms of activationLevel.
	 *  
	 * @param b
	 * @return the best behaviour b or b1
	 */
	public BehaviourRule compare(BehaviourRule b){
		BehaviourRule b1 = d_memory.getNextActiveB();
		//System.out.println("CAgent.compare: vergelijk nu " + b.getName() + " met " + b1.getName());
		double utilb = calcUtil(b);   
		double utilb1 = calcUtil(b1);
		//System.out.println("util " +  b.getName() + "= " + utilb);
		//System.out.println("util " +  b1.getName() + "= " + utilb1);
		return ((utilb > utilb1) ? b : b1);
	}
	
	/**
	 * Calculates a value (utility value) to compare two behaviours with each other.
	 * This utility function is based on the expectancy an individual has performing a behaviour
	 * in fulfilling a specific goal. <br>
	 * 
	 * This is a function that can be 'played' with. What makes a certain behaviour more
	 * valuable compared to others. Or under what circumstances does the valuation change
	 * etc. Here theories can differ and thus this is the place to let them have effect.
	 * 
	 * @param b
	 * @return
	 */
	
	public double calcUtil(BehaviourRule b){
		/* doesn't cost any time */
		double idDom = d_memory.getIdentity().getD_dominance();
		double socDom = d_memory.getSocial().getD_dominance();
		double safeDom = d_memory.getSafety().getD_dominance();
		double subsDom = d_memory.getSubsistence().getD_dominance();

		String bfactName = Misc.bName2fName(b.getName());
		
		BehaviourFact bfact = (BehaviourFact) d_memory.getElement(bfactName);
		decreaseInternalTime(bfact.getRetrievalTime());
		
		double utilb = (idDom * (bfact.getExpectancy(Constants.GOAL_NAME_IDENTITY) * 100)) + 
					   (socDom * (bfact.getExpectancy(Constants.GOAL_NAME_SOCIAL) * 100)) + 
					   (safeDom * (bfact.getExpectancy(Constants.GOAL_NAME_SAFETY) * 100)) +
					   (subsDom * (bfact.getExpectancy(Constants.GOAL_NAME_SUBSISTENCE) * 100));
		
		return utilb;
		
	}
	
	/**
	 * Function that checks whether another agent is part of the same group as this agent.
	 * (in-group buddy check) 
	 * 
	 * @param otherAgent
	 * @return being part of the same group
	 */
	
	public boolean ingroup(CrowdAgent otherAgent){
		return (d_groupA == otherAgent.d_groupA);
	}
	
	/**
	 * Function that sets the internal time an agent has to select a behaviour
	 * This is based upon the arousal level of an individual. 
	 * High arousal implies less time to choose a behaviour, and vise versa
	 * <br><br>
	 * We let time be a factor that has a range of  [0,10]
	 * Where arousal is ranging between [0,1], this only differs a factor 10,
	 * This function returns the time by retrieving the arousal level and return
	 * this times 10.
	 * 
	 * 
	 */
	private void resetInternalTime() {
		double arousalLevel = d_physiology.getArousal();
		d_internalTime =  arousalLevel * 10;
	}

	public void decreaseInternalTime(double val){
		d_internalTime -= val;
	}
	
//	
//	/**
//	 * A function used for testing whether movement is going alright.
//	 * 
//	 */
//	public void moveTest(){
//		Grid patch = (Grid) d_world_context.getProjection("Simple Grid");
//		ContinuousSpace space = (ContinuousSpace) d_world_context.getProjection("Continuous Space");
//		
//		NdPoint testPos = new NdPoint(20,20);
//		NdPoint pos = space.getLocation(this);
//		//NdPoint pos = getMyPos();
//		setHeading(Misc.heading2point(pos, testPos));
//		
////		if (pos.getY() > testPos.getY()){
////			d_heading = 180 + Math.atan((pos.getX() - testPos.getX()) / (pos.getY() - testPos.getY())) / Math.PI * 180;
////		} else {
////			d_heading = Math.atan((pos.getX() - testPos.getX()) / (pos.getY() - testPos.getY())) / Math.PI * 180;
////		}
//		
//		NdPoint target = MathUtils.calcNewPosition(pos.getX(), pos.getY(), Constants.V_DEFAULT, d_heading);
//		GridPoint targetPatch = Misc.nd2gridpoint(target);
//		
//		if (Misc.walkablePatch(d_world_context, this, target)){
//			space.moveTo(this, target.getX(), target.getY(),0);
//			patch.moveTo(this, targetPatch.getX(), targetPatch.getY()); 
//		}
//	}

	public NdPoint getRandomLocation(){
		Grid patch = (Grid) d_world_context.getProjection("Simple Grid");
		ContinuousSpace space = (ContinuousSpace) d_world_context.getProjection("Continuous Space");
		GridDimensions dim = patch.getDimensions();
		double x = Math.random() * (dim.getWidth() - 1);  
		double y = Math.random() * (dim.getHeight() - 1);
		return new NdPoint(x, y, 0);
	  }
	
	/**
	 * Returns the ID of an agent
	 * 
	 * @return the id of the agent
	 */
	
	@Parameter(usageName="agentID", displayName="Agent id")  //makes this property visible in the display 
	public int getAgentID(){
		return d_agentID;
	}
	
	
	/**
	 * Returns the name of the point of interest an agent is heading towards??
	 * @return
	 */
//	@Parameter(usageName="POI", displayName="Agent POI") //makes this property visible in the display
	public String getAgentPOI(){
		return d_POI;
	}
	
	/**
	 * Sets the heading of the current CrowdAgent	
	 */
	public void setHeading(double heading){
		d_heading = heading;
	}

	
	/**
	* Returns the heading of the current CrowdAgent
	* 
	* @Parameter indicates that this variable is also displayed/send  
	* in/to the gui.	
	*/	
	@Parameter(usageName="d_heading", displayName="Agent heading") //makes this property visible in the display
	public double getHeading(){
		return d_heading;
	}

	/**
	 * Returns the dominant goal in a integer notation
	 * identity (0); social (1); safety (2); subsistence (3)
	 * 
	 * @return integer
	 */
	public int getDomGoal(){
		d_memory.setDomGoal(); 
		Goal g = d_memory.getDomGoal();
		String name = g.getName();
		if (name.equalsIgnoreCase("identity"))
			return Constants.GOAL_IDENTITY;
		else if (name.equalsIgnoreCase("social"))
			return Constants.GOAL_SOCIAL;
		else if (name.equalsIgnoreCase("safety"))
			return Constants.GOAL_SAFETY;
		else if (name.equalsIgnoreCase("subsistence"))
			return Constants.GOAL_SUBSISTENCE;
		else
			System.out.println("domGoal : " + g);
			
			//N: syst.err??
			return 4;
		}
	
	/**
	 * Returns the dominant goal 
	 * 
	 * @return name of the dominant goal (String)
	 */
	@Parameter(usageName="domGoal", displayName="Dom-Goal") //makes this property visible in the display
	public String getDomGoalString(){
		return d_memory.getDomGoal().getName();
	}
	
	
	/**
	 * Returns the current goal dominance of the identity goal
	 * @return double - identity goal dominance
	 */
	@Parameter(usageName="identityDom", displayName="Dom-ID") //makes this property visible in the display
	public double getIDdominance(){
		Goal d = d_memory.getIdentity();
		if (!d.getName().equalsIgnoreCase("empty")){
			return d.getD_dominance();
		}
		else 
			return 0;
	}
	
	/**
	 * @return the d_numWalkingLeaders
	 */
	public int getNumWalkingLeaders() {
		return d_numWalkingLeaders;
	}

	/**
	 * @return the d_numRunningLeaders
	 */
	public int getNumRunningLeaders() {
		return d_numRunningLeaders;
	}

	/**
	 * @return the d_numDancingLeaders
	 */
	public int getNumDancingLeaders() {
		return d_numDancingLeaders;
	}

	/**
	 * Returns the current goal dominance of the social goal
	 * @return double - social goal dominance
	 */
	@Parameter(usageName="socialDom", displayName="Dom-Social") //makes this property visible in the display
	public double getSocDominance(){
		Goal d = d_memory.getSocial();
		if (!d.getName().equalsIgnoreCase("empty")){
			return d.getD_dominance();
		}
		else 
			return 0;
	}
	

	
	/**
	 * Returns the current goal dominance of the safety goal
	 * @return double - safety goal dominance
	 */
	@Parameter(usageName="safetyDom", displayName="Dom-Safety") //makes this property visible in the display
	public double getSafeDominance(){
		Goal d = d_memory.getSafety();
		if (!d.getName().equalsIgnoreCase("empty")){
			return d.getD_dominance();
		}
		else 
			return 0;
	}
	
	/**
	 * Returns the current goal dominance of the subsistence goal
	 * @return double - subsistence goal dominance
	 */
	@Parameter(usageName="subsistenceDom", displayName="Dom-Subs") //makes this property visible in the display
	public double getSubsDominance(){
		Goal d = d_memory.getSubsistence();
		if (!d.getName().equalsIgnoreCase("empty")){
			return d.getD_dominance();
		}
		else 
			return 0;
	}
	
	/**
	 * Returns the current level of satisfaction of the identity goal
	 * @return double
	 */
@Parameter(usageName="identity", displayName="Inter-IDlevel") //makes this property visible in the display
	public double getIDlevel(){
		Goal d = d_memory.getIdentity();
		if (!d.getName().equalsIgnoreCase("empty")){
			return d.getSatisfaction();
		}
		else 
			return 0;
	}
	
	/**
	 * Returns the current level of satisfaction of the social goal
	 * @return double
	 */
@Parameter(usageName="social", displayName="Intern-SocLevel") //makes this property visible in the display
	public double getSoclevel(){
		Goal d = d_memory.getSocial();
		if (!d.getName().equalsIgnoreCase("empty")){
			return d.getSatisfaction();
		}
		else 
			return 0;
	}
	
	/**
	 * Returns the current level of satisfaction of the safety goal
	 * @return double
	 */
@Parameter(usageName="safety", displayName="Intern-safeLevel") //makes this property visible in the display
	public double getSafelevel(){
		Goal d = d_memory.getSafety();
		if (!d.getName().equalsIgnoreCase("empty")){
			return d.getSatisfaction();
		}
		else 
			return 0;
	}
	
	/**
	 * Returns the current level of satisfaction of the subsistence goal
	 * @return double
	 */
@Parameter(usageName="subsistence", displayName="Intern-subsLevel") //makes this property visible in the display
	public double getSubslevel(){
		Goal d = d_memory.getSubsistence();
		if (!d.getName().equalsIgnoreCase("empty")){
			return d.getSatisfaction();
		}
		else 
			return 0;
	}
	
	/**
	 * Sets an agent to be a leader and the the type of influence 
	 * the agent will steer to this can be: peaceful or warful/toward aggression
	 * mind that this is the perception of the agent itself as being 
	 * a leader.
	 * 
	 * These are current options, however this is a point
	 * for further development. This can be done in many ways:
	 * - manipulate the positions/spreading of leader(s) given an area of people  
	 * - A leader can steer in many different ways and this can also change during time.
	 * - Perception of other agents effect whether the influence of this agent will play a role, even further
	 * it can change the way an agent perceives itself as being a leader.
	 * - etc.
	 * 
	 * @param leaderType
	 */
	public void setLeadership(int leaderType){
		d_leader = true;
		d_leaderType = leaderType;
	}
	
	/**
	 * Sets an agent to be a leader
	 */
	public void setLeadership(){
		d_leader = true;
	}
	
	/**
	 * Returns whether this agent is a leader or not
	 * note: this is a perception an agent of itself being a leader or not
	 * At this point in my simulation this also means that others view this
	 * agent as a leader, however this is setup as a perception of those others
	 * too. This is a starting point and made easy for further development.
	 * 
	 * @return d_leader - a boolean value
	 */
@Parameter(usageName="leader", displayName="Attribute-Is a leader") //makes this property visible in the display
	public boolean getLeadership(){
		return d_leader;
	}
	
	/**
	 * Returns the type of leader an agent is.
	 * An leader can direct peaceful or warful (aggressive/violent)
	 * not used now
	 * 
	 * @return d_leaderType - an integer indicating 0 (neutral), 1 (peaceful), 2 (warful/aggressive)
	 */
//	@Parameter(usageName="leaderType", displayName="Attribute-leaderType") //makes this property visible in the display
	public String getLeadershipType(){
		if (d_leaderType == Constants.AGGRESSIVE_LEADER){
			return "warful";
		}
		else if (d_leaderType == Constants.PEACEFUL_LEADER){
			return "peaceful";
		}
		else {
			return "no leader";
		}
	}
	
	
	/**
	 * Function returns whether an agent is member of a group
	 * At this point specifically to belong to group (A) or not
	 * 
	 * @return d_groupA
	 */
//@Parameter(usageName="groupMember", displayName="Attribute-Is a groupmember:") //makes this property visible in the display
	public boolean getGroupMembership(){
		return d_groupA;
	}
	
	/**
	 * @return current position of the agent in continuous space
	 */
	public NdPoint getMyPos(){
		return d_mypos;
	}
	
	/**
	 * returns the number of people surrounding this individual within 1 simulation unit (=0.4 meters)
	 */
	
	public int getSurroundedness(){
		return d_numOthers;
	}
	/**
	 * @return number of agents that are observed at this time
	 */
	
@Parameter(usageName="perceivedOthers", displayName="perceived others") //makes this property visible in the display
	public int getPerceivedOthers(){
		return d_numVisibleOthers;
	}

	/**
	 * Indicates whether the agent in inhibited by it's physical surroundings in 
	 * moving
	 * 
	 * @param b
	 */
	public void setCannotMove(boolean b){
		d_cannotMove = b;
	}
	
	/**
	 * Returns whether an agent is inhibited in its current movement or not
	 * @return ability to move
	 */
	public boolean getCannotMove(){
		return d_cannotMove;
	}
	
	/**
	 * Sets a variable with the explicit behaviour performed by this agent
	 * It checks whether the behaviour that was intended to executed has really been
	 * executed, otherwise it says that is was obstructed while trying.
	 * 
	 * This is done to easily observe the behaviour of another agent.
	 * 
	 * TODO make this obstruction sensitive for all behaviours, not only for movement behaviours
	 * 
	 * @param name
	 */
	public void setObservableB(String name){
		d_myBehaviour = (d_cannotMove ? Constants.B_NAME_INHIBITED : name);
		d_walk = (name.equalsIgnoreCase(Constants.B_NAME_WALK) ? 1 : 0);
		d_run = (name.equalsIgnoreCase(Constants.B_NAME_RUN) ? 1 : 0);
		d_dance = (name.equalsIgnoreCase(Constants.B_NAME_DANCE) ? 1 : 0);
	}
	
	/**
	 * Returns the behaviour of that this agent is currently showing.
	 * This is not the behaviour an agent chose to perform, but the behaviour
	 * that is actually performed. As the agent might be inhibited in showing
	 * its behaviour
	 * 
	 * @return
	 */
	
@Parameter(usageName="myBehaviour", displayName="Extern-my visible behaviour") //makes this property visible in the display
	public String getObservableB(){
		return d_myBehaviour;
	}

	/**
	 * Returns the behaviour an individual is showing in a numerical value
	 * 1 = walk
	 * 2 = run
	 * 3 = dance
	 * 4 = inhibited no behaviour - system.out.err!
	 * @return
	 */
	public int getObservableBNum(){
		if (d_myBehaviour.equalsIgnoreCase(Constants.B_NAME_WALK)){
			return 1;
		}
		if (d_myBehaviour.equalsIgnoreCase(Constants.B_NAME_RUN)){
			return 2;
		} 
		if (d_myBehaviour.equalsIgnoreCase(Constants.B_NAME_DANCE)){
			return 3;
		}
	return 4;
}

	/**
	 * Returns the current activation of the memory element: walk (behaviour rule)
	 * @return
	 */

	public double getActivationBruleWalk(){
		MemoryElement bRuleWalk = (MemoryElement) d_memory.getElement(Constants.B_NAME_WALK);
		return bRuleWalk.getActivation();
	}
	
	
	/**
	 * Returns the current activation of the memory element: run (behaviour rule)
	 * @return
	 */

	public double getActivationBruleRun(){
		MemoryElement bRuleRun = (MemoryElement) d_memory.getElement(Constants.B_NAME_RUN);
		return bRuleRun.getActivation();
	}
	
	/**
	 * Returns the current activation of the memory element: dance (behaviour rule)
	 * @return
	 */

	public double getActivationBruleDance(){
		MemoryElement bRuleDance = (MemoryElement) d_memory.getElement(Constants.B_NAME_DANCE);
		return bRuleDance.getActivation();
	}
	
	
	/**
	 * Returns the diameter of the area an individual occupies.
	 * 
	 * @return the d_occupation_diameter
	 */
	public double getOccupationDiameter() {
		return d_occupation_diameter;
	}

	/**
	 * Sets the area an individual occupies. (diameter of the occupation circle) 
	 * Default this is 40x40cm ; 1x1 simulation unit.
	 * However for some behaviours (such as dancing) more space is taken.
	 * 
	 * @param d_occupation_diameter 
	 */
	public void setOccupationDiameter(double d_occupation_diameter) {
		this.d_occupation_diameter = d_occupation_diameter;
	}
	
	/**
	 * Indicates whether an agent is satisfied.
	 * Meaning that all its goals are currently fulfilled.
	 * 
	 * @param bool
	 */
	public void setSatisfied(boolean bool){
		d_satisfied = bool;
	}
	
	/**
	 * Returns the whether an agent is satisfied
	 * meaning whether all its goals are currently fulfilled or not
	 * 
	 * @return
	 */
	public boolean getSatisfied(){
		return d_satisfied;
	}

	/**
	 * Returns the dominant influence given (subsistence-related) physiology
	 * 
	 * @return dominant physical factor: bladder or stomach
	 */
	public String getDomPhysio(){
		return d_physiology.getDomPhysiology();
	}
	
	/**
	 * Returns the local density perceived by this individual
	 * 
	 * @return local (perceived) density level 
	 */

@Parameter(usageName="perceivedDensity", displayName="perceived density") //makes this property visible in the display
	public double getPerceivedDensity(){
		return d_density;
	}

	/**
	 * @return the prevIdSatis
	 */
	public double getPrevIdSatis() {
		return d_prevIdSatis;
	}

	/**
	 * @return the prevSocSatis
	 */
	public double getPrevSocSatis() {
		return d_prevSocSatis;
	}

	/**
	 * @return the prevSafeSatis
	 */
	public double getPrevSafeSatis() {
		return d_prevSafeSatis;
	}

	/**
	 * @return the prevSubsSatis
	 */
	public double getPrevSubsSatis() {
		return d_prevSubsSatis;
	}
	
	/**
	 * @return the prevPos
	 */
	public NdPoint getPrevPos() {
		return d_prevPos;
	}
	
	/**
	 * 
	 * @return nearby agents of last timeStep (before moving)
	 */
	public Vector getPrevNearbyAgents(){
		return d_prevNearbyAgents;
	}
	

	/**
	 * 
	 * @return agents insight of last timeStep (before moving)
	 */
	public Vector getPrevAgentsInsight(){
		return d_prevAgentsInSight;
	}
	
	/**
	 * Function indicates the relative crowdedness around an agent.
	 * In which the clustering refers to the pure physical presence of others
	 * 
	 * clusterIndex = other agents (moores neigborhood) / max_agents that can surround an agent
	 * As currently only one person can stand on a patch the maximum of agents that can stand in the area of
	 * an agent is:  
	 * 
	 * ci = # persons in surrounding / max. possible persons in surrounding
	 * 
	 * nearby = # persons in surrounding
	 * maxNearby =  max. possible persons in surrounding = area of 1 person / area of the surrounding
	 * personArea = area of 1 person = 0.4 * 0.4 = 0.16 m^2
	 * surroundingArea = area of the surrounding = area of a circle =  pi * r^2 = pi * gaze-depth^2 =  pi * (2 * unit2meter)^2 = 3,14 * (2 * 0,4)^2 = 2,01 
	 * maxSurrounding = 2,01/0.16 = 12,6 
	 * 
	 * circle area = pi* r^2.
	 * 
	 * @return
	 */
	
	public double clusterIndex(){
		return d_numOthers / Constants.MAX_SURROUNDING;
	}

@Parameter(usageName="arousalLevel", displayName="Inter-arousal level") //makes this property visible in the display
	public double getArousalLevel(){
		return d_physiology.getArousal();
	}

	/**
	 * 
	 * @return the local density (density perceived by this agent)
	 */
	public double getDensity(){
		return d_density;
	}
	
	/**
	 * @return the agents that are nearby (360, surrounding the agent)
	 */
	public Vector<CrowdAgent> getNearbyAgents(){
		return d_nearbyAgents;
	}
	
	/**
	 * @return the distClosestAgent
	 */

//@Parameter(usageName="distNearestAgent", displayName="mostclose") //makes this property visible in the display
	public double getDistNearestAgent() {
		return d_distNearestAgent;
	}
	
	/**
	 * @return ID of nearest agent
	 */

	@Parameter(usageName="nearestAgent", displayName="Agent - nearest") //makes this property visible in the display
	public int getIDNearestAgent() {
		if (d_nearestAgent != null){
			return d_nearestAgent.getID();
		} else {
			return 0;
		}
	}

	
	/**
	 * @return nearest agent
	 */
	public CrowdAgent getNearestAgent() {
		return d_nearestAgent;
	}

	/**
	 * @return the d_numVisibleOthers
	 */
	public int getNumVisibleOthers() {
		return d_numVisibleOthers;
	}

	/**
	 * @return the d_visibleOthers
	 */
	public Vector<CrowdAgent> getVisibleOthers() {
		return d_visibleOthers;
	}
	
	/*
	 * extra getters for initial parameters 
	 */
	
	/**
	 * Satisfaction preference of the Identity goal for this agent (heterogeneous & static attribute of the agent)
	 * @return preference level of the Identity goal
	 */
	public double getIDpref(){
		return d_memory.getIDPref();
	}

	/**
	 * Satisfaction preference of the Social goal for this agent (heterogeneous & static attribute of the agent)
	 * @return preference level of the Social goal
	 */
	public double getSocialpref(){
		return d_memory.getSocialPref();
	}

	/**
	 * Satisfaction preference of the Safety goal for this agent (heterogeneous & static attribute of the agent)
	 * @return preference level of the Safety goal
	 */
	public double getSafepref(){	
		return d_memory.getSafePref();
	}

	/**
	 * satisfaction preference of the Subsistence goal for this agent (heterogeneous & static attribute of the agent)
	 * @return preference level of the Subsistence goal
	 */
@Parameter(usageName="subsPref", displayName="Inter-subsPref") //makes this property visible in the display
	public double getSubspref(){	
		return d_memory.getSubsPref();
	}
	
	/**
	 * @return stomach level [0,1] (empty-full) 
	 */

@Parameter(usageName="stomachLevel", displayName="Inter-stomach") //makes this property visible in the display
	public double getStomachLevel(){
		return (d_physiology.getStomachLevel());
	}
	
	/**
	 * @return bladder level [0,1] (empty-full)
	 */
@Parameter(usageName="bladderLevel", displayName="Inter-bladder") //makes this property visible in the display
	public double getBladderLevel(){
		return d_physiology.getBladderLevel();
	}

//@Parameter(usageName="heading2point", displayName="Head to") //makes this property visible in the display
//	public NdPoint getHeading2point(){
//		return heading2point;
//	}

@Parameter(usageName="agentMove2x", displayName="Agent - Move2x") //makes this property visible in the display
	public double getAgentMove2x(){
		return heading2point.getX();
	}

@Parameter(usageName="agentMove2y", displayName="Agent - Move2y") //makes this property visible in the display
	public double getAgentMove2y(){
		return heading2point.getY();
	}

@Parameter(usageName="myx", displayName="Agent - my x") //makes this property visible in the display
public double getMyPosx(){
	return d_mypos.getX();
}

@Parameter(usageName="myy", displayName="Agent - my y") //makes this property visible in the display
public double getMyPosy(){
	return d_mypos.getY();
}

@Parameter(usageName="walkexp", displayName="socExp_walk") //makes this property visible in the display
public double getWalkExp() {
	BehaviourFact walk = d_memory.getBehaviourFact(Constants.B_NAME_WALK);
	return walk.getExpectancy(Constants.GOAL_NAME_SOCIAL);
}

@Parameter(usageName="runexp", displayName="socExp_run") //makes this property visible in the display
public double getRunExp() {
	BehaviourFact run = d_memory.getBehaviourFact(Constants.B_NAME_RUN);
	return run.getExpectancy(Constants.GOAL_NAME_SOCIAL);
}

@Parameter(usageName="dancexp", displayName="socExp_dance") //makes this property visible in the display
public double getDanceExp() {
	BehaviourFact dance = d_memory.getBehaviourFact(Constants.B_NAME_DANCE);
	return dance.getExpectancy(Constants.GOAL_NAME_SOCIAL);
}

/**
 * the experiment condition of the density
 * 1 : low (1)
 * 2 : medium (3)
 * 3 : high (5.5)
 * 
 * @return the experiment condition of the safety preference
 * 
 */
public int getDensityCondition() {
	return d_densityCondition;
}

/**
 * the experiment condition of the safety preference
 * 1 : low safety preference
 * 2 : control condition, medium safety pref
 * 3 : high safety preference
 * 
 * @return the experiment condition of the safety preference
 * 
 */
public int getSafetyCondition() {
	return d_safetyCondition;
}

/**
 * the experiment condition of the number of leaders
 * 1 : zero leaders (0)
 * 2 : low amount of leaders (25) 1:28
 * 3 : medium amount of leaders (50) 1:14
 * 4:  high amount of leaders (100) 1:7
 * @return the experiment condition of the safety preference
 * 
 */
public int getLeadershipCondition() {
	return d_leadershipCondition;
}

/**
 * Sets this agent to be a dummy.
 * An agent who is tracked during a run to use in the life histories
 * that explain what happens on the individual
 * 
 *  todo - turn this var in a final 
 */
public void setToDummy(int color){
	d_dummy = true; 
	d_dummyColor = color;
}
/**
 * 
 * @return whether this agent is a dummy
 */
public boolean isDummy(){
	return d_dummy;
}

/**
 * 
 * @return the visualisation color of dummy
 */
public int getDummyColor(){
	return d_dummyColor;
}

/**
 * 
 * @return whether the agent is a leader
 */
public boolean isLeader(){
	return d_leader;
}

/**
 * Sets the agent with the persons it perceives as a leader
 * @param leaders
 */
public void setMyLeaders(HashSet<CrowdAgent> leaders){
	d_memory.setMyLeaders(leaders);
}

/**
 * 
 * @return whether this agent is walking 
 */
public int getIsWalking(){
	return d_walk;
}
/**
 * 
 * @return whether this agent is running
 */
public int getIsRunning(){
	return d_run;
}

/**
 * 
 * @return whether this agent is dancing
 */
public int getIsDancing(){
	return d_dance;
}

/**
 * 
 * @return amount of leaders that this agent perceived in this time-step
 */
public int getCntVisibleLeaders(){
	return d_numVisibleLeaders;
}

/**
 * 
 * @return whether an agent performs the same behaviour as it saw by its leader
 */
public int getSameBasLeader(){
	return d_sameAsLeader;
}

public double getDanceUtil(){
	return calcUtil(d_memory.getBehaviour(Constants.B_NAME_DANCE));
}

public double getRunUtil(){
	return calcUtil(d_memory.getBehaviour(Constants.B_NAME_RUN));
}

public double getWalkUtil(){
	return calcUtil(d_memory.getBehaviour(Constants.B_NAME_WALK));
}

}
