package crowdmodel.agents;

/**
 * Goal is a type of memoryElement. 
 * Memory has 3 types of elements: Goals, Facts, and Rules
 * A Goal represents the drives/desired stated of an agent
 * 
 * In this structure an agent has four abstract goals (identity, social, safety, subsistence)
 * 
 * Identity : represents the goal to have an individual identity
 * Social : represents the goal to belong to a group
 * Safety : represents the goal to remain safe
 * Subsistence : represent the goal to keep energy (sleep, eat, drink, etc)
 * 
 * @author Nanda Wijermans
 * 
 */


public class Goal extends MemoryElement {

	
	private double d_satisfaction;    // Fulfilment of the current goal [0,1]
	private double d_preference;	  // preference level of fulfilment (=satisfaction)
	private double d_dominance;		  // Current dominance value of this goal

	/**
	 * Constructor creates a goal 
	 * and initialises it with a initial level of satisfaction
	 * and a preferred level of satisfaction of this goal
	 */

	public Goal (String name, double satis, double pref){
		d_name = name;
		setSatisfaction(satis);
		setPreference(pref);
		updateDominance();
	}

	/**
	 * Calculates the dominance of the current goal 
	 * based on the preferred level of fulfilment and the real level of fulfilment
	 * 
	 * dominance = preferred level - current level 
	 */
	
	public void updateDominance(){
		d_dominance = d_preference - d_satisfaction;
		if (d_dominance > 1) { d_dominance = 1; }
		if (d_dominance < 0) { d_dominance = 0; }
		d_activation = d_dominance;					
	}
	
	/**
	 * Sets the current level of satisfaction of this goal
	 * and also calculates the new value for dominance
	 */
	public void setSatisfaction(double satis) {
		d_satisfaction = satis;
		updateDominance();
	}
	
	/**
	 * Returns the current level of satisfaction of this goal
	 */
	public double getSatisfaction() {
		return d_satisfaction;
	}

	/**
	 * Sets the preferred fulfilment of this goal
	 */
	public void setPreference (double pref) {
		d_preference = pref;
	}
	
	/**
	 * Returns the preferred level of fulfilment of this goal
	 */
	public double getPreference () {
		return d_preference;
	}
	
	/**
	 * Returns the name label of goal
	 */
	public String getName(){
		return d_name;
	}
	
	/**
	 * Returns the dominance of the current goal
	 * @return
	 */
	public double getD_dominance() {
		return d_dominance;
	}
	
}