package crowdmodel.agents;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Vector;

import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunState;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.graph.Network;
import repast.simphony.space.grid.GridPoint;
import repast.simphony.space.grid.Grid;
import repast.simphony.util.ContextUtils;
import crowdmodel.Constants;
import crowdmodel.environment.*;
import crowdmodel.utils.*;


/**
 * A LocomotionRule describes the movement of
 * the object that is asking for it
 * 
 * Types of locomotion are
 * 
 * person:
 * A) vertical/body position: (not included)
 * standing/upright
 * sitting/seated
 * kneeling/knelt
 * lying/prone
 * other
 * 
 * B)horizontal: 
 * walking
 * running/jogging
 * marching (not included)
 * dancing
 * vehicular motion (not included)
 * other
 * 
 * object:
 * move/transpose/displace
 *
 * @author Nanda Wijermans
 */

public class LocomotionRule extends BehaviourRule{

		
	public static final String[] objectLocomotions = {"transpose"};
	public static final String[] personLocomotions = {"standup", "sitdown", "kneel", "liedown", "other", "walk", "run", "dance", "vehicle"};

	public String d_domGoal;
	
	private ArrayList<String> objectLocomotionList;// ["transpose"];
	private ArrayList<String> personLocomotionList;//["standup","sitdown","kneel","liedown","other","walk","run","dance","vehicle"];

	
	//for testing only
	public double mostclose;
	public CrowdAgent d_nearest;
	private NdPoint d_hpos;
	
	/**
	 * Constructor creates a locomotion rule with a predefined set of locomotion options
	 * 
	 * @param name
	 */
	LocomotionRule(String name){
		objectLocomotionList = Misc.fill(objectLocomotions);
		personLocomotionList = Misc.fill(personLocomotions);
		d_name = name;

		d_hpos = new NdPoint(0,0,0);
	}
		
	
	/**
	 * Executes the current locomotionRule
	 * At this point this is done by relating the name of the this rule
	 * with the corresponding function (predefined locomotion) for execution
	 * But can be reconsidered in doing this differently as all functions
	 * are described in every instantiation LocomotionRule class, however
	 * only one is associated/used. 
	 * 
	 * @param domGoal
	 * @param heading
	 * @param pos
	 */
	public void execute(){		
		
		//System.out.println("Execute(): " + d_name);
		if (d_name == Constants.B_NAME_WALK) { walk(); }
		else if (d_name == Constants.B_NAME_RUN){ run(); }
		else if (d_name == Constants.B_NAME_DANCE){ dance(); }
		else
			nothing();
	}
	
	/**
	 * Do nothing behaviour :)
	 * 
	 */
	public void nothing(){
		// do nothing
	}
	
	
	/**
	 * Describes the execution of a walking behaviour
	 * Given a certain heading the agent is walking in that direction.
	 */
	public void walk(){
	
		String domGoal = d_thisMemory.getDomGoal().getName();
		//System.out.println("Verander mijn heading : " + domGoal);
		
		setGoalHeading(domGoal);
		//d_thisAgent.setHeading(270);
		
		//System.out.println("Ga nu lopen: ");
		move(Constants.V_WALK);
		//moveTest();
	}
	
	
	/**
	 * Describes a moving behaviour: running
	 * Given a heading (based on current dominant goal) the agent 
	 * moves with a velocity of 1.5 simulation units/tick (0.6 m/s)
	 * 
	 */
	public void run(){
		String domGoal = d_thisMemory.getDomGoal().getName();		
		setGoalHeading(domGoal);
		
		move(Constants.V_RUN);
		move(Constants.V_RUN);
	}
	
	/**
	 * Describes a behaviour on the spot: dancing
	 * At this point it only implies that an individual occupies 
	 * a larger area in space
	 * 
	 */
	public void dance(){
		
		Context context = RunState.getInstance().getMasterContext();
		Grid patch = (Grid) context.getProjection("Simple Grid");
		ContinuousSpace space = (ContinuousSpace) context.getProjection("Continuous Space");
		
		d_thisAgent.setOccupationDiameter(Constants.HUMAN_DANCE_DIAMETER);
		
		double dist = 0.25; // distance of movement is 10 cm, that corresponds with 0.25 sim units
		double heading = d_thisAgent.getHeading();
		boolean swingleft = MiscSingleton.getInstance().getMyUniform().nextBoolean();
		double swingheading = 0;
		
		NdPoint pos = d_thisAgent.getMyPos();
		double x = pos.getX();
		double y = pos.getY();
		
		swingheading = (swingleft ? heading - 90 : heading + 90);
		swingheading = swingheading + 360 % 360; // correct for values outside [0,360]
		swingheading = swingheading * Math.PI / 180; // heading in radians
		
		double dx = Math.sin(swingheading) * dist; // 
	    double dy = Math.cos(swingheading) * dist;
		
	    NdPoint target = new NdPoint (x + dx, y + dy);
	    x = target.getX();
	    y = target.getY();
		if (Misc.withinWorld(target)){
			boolean walkable = true;
			for (Object o : patch.getObjectsAt((int)x, (int)y)) {
			/* discern walkable from non-walkable patches */ 
				if (o instanceof CellObject) {
					CellObject cell = (CellObject) o;
					walkable = cell.getWalkable();
				} 
			}
			if (walkable){ 
				space.moveTo(d_thisAgent, x, y, 0);
				patch.moveTo(d_thisAgent, (int) x, (int) y); 
			}
		}
		
	}
	
	/**
	 * Changes the heading of an agent based on the dominant goal
	 * This is a pre-movement part of an action.
	 *  
	 * Dominant ID: head towards the stage
	 * 
	 * Dominant Soc: head towards others 
	 *   driven by the need to belong to a group (social goal)
	 *   that is fulfilled by being in the vicinity of others
	 *   At this point social movement is headed to others (in general)
	 *   however needs to be refined:
	 *   - moving towards friends/group-members
	 *   - moving towards a specific person(s) 
	 *
	 * Dominant Safety: head to low density areas
	 * 	 This involves moving away from high density areas, 
	 * 	 driven by the need to feel safe. (safety goal)
	 *   that is fulfilled by non-crowded surroundings
	 *   Implemented by walking in a arbitrary direction at this point
	 *   TODO(needs refinement)
	 * 
	 * Dominant Subsistence: head to toilet or bar
	 *   driven by the need for energy (Energy goal)
	 *   At this point movement toward the toilet or bar
	 *   depends on which one is more close
	 */	
	public void setGoalHeading(String domGoal){

		if (domGoal  == Constants.GOAL_NAME_IDENTITY){
			d_thisAgent.setHeading(Misc.heading2point(d_thisAgent.getMyPos(), Constants.POI_STAGE));
		} else if (domGoal == Constants.GOAL_NAME_SOCIAL){
			d_thisAgent.setHeading(socialHeading());
		} else if (domGoal == Constants.GOAL_NAME_SAFETY){
			d_thisAgent.setHeading((Math.random() * Constants.DEGREES));
		} else if (domGoal == Constants.GOAL_NAME_SUBSISTENCE){
			d_thisAgent.setHeading(subsHeading());
		}
	}
			
	
	/**
	 * Function describes the heading change of an agent
	 * as the pre-movement part of moveSocial behaviour
	 * This involves moving towards others, driven by the
	 * need to fulfil the vicinity of others
	 * 
	 * At this point social movement is headed to others
	 * however needs to be refined:
	 * - moving towards friends/group-members
	 * - moving towards a specific person(s) 
	 * 	
	 * @return heading
	 */
	public double socialHeading(){

		Context context = RunState.getInstance().getMasterContext(); 
		Context agent_scontext = ContextUtils.getContext(d_thisAgent);
		ContinuousSpace space = (ContinuousSpace) context.getProjection("Continuous Space");
		Network<CrowdAgent> socNetwork = (Network<CrowdAgent>) agent_scontext.getProjection("Social Network");
		
		NdPoint mypos = d_thisAgent.getMyPos();
		double myheading = d_thisAgent.getHeading();
		
		NdPoint pos = new NdPoint(0,0,0);
		int visibleAgents = d_thisAgent.getNumVisibleOthers();
		
		if (visibleAgents > 0){   // there are agents in my sight
			Vector<CrowdAgent> agents = d_thisAgent.getVisibleOthers();
			Iterable<CrowdAgent> myFriends = socNetwork.getAdjacent(d_thisAgent);

			CrowdAgent friend = null;
			double dist2friend = Constants.WORLD_MAX_DIST;
			for (CrowdAgent myfriend : myFriends) {
				if (agents.contains(myfriend) && d_thisAgent != myfriend){
					NdPoint posFriend = space.getLocation(myfriend);
					double dist  = space.getDistance(mypos, posFriend);
					if (dist < dist2friend) {
						friend = myfriend;
						dist2friend = dist;
					}											
				}
			}
			
			if (friend != null){  //  check whether a friend is near
				pos = space.getLocation(friend);    // head to friend position
			} else {
				d_nearest = Misc.nearestAgent(context, d_thisAgent, agents.iterator());
				pos = space.getLocation(d_nearest);
				}
		}
		else {  
			Iterator<CrowdAgent> allAgents = agent_scontext.iterator();
			pos = Misc.posNearestAgent(context, mypos, allAgents, d_thisAgent);	
			}	
		
		setHpos(pos);
		return MathUtils.calcHeading(myheading, mypos, pos);
	}
	
	
		
	/**
	 * Turn to the agent that is the closest to you.
	 * Is not used now, is the simple, 1st version of socialHeading()
	 * 	
	 * @return heading
	 */
	public double socialHeadingSimple(){

		Context context = RunState.getInstance().getMasterContext(); 
		Context agent_scontext = ContextUtils.getContext(d_thisAgent);
		ContinuousSpace space = (ContinuousSpace) context.getProjection("Continuous Space");
		Network<CrowdAgent> socNetwork = (Network<CrowdAgent>) agent_scontext.getProjection("Social Network");
		
		NdPoint mypos = d_thisAgent.getMyPos();
		double myheading = d_thisAgent.getHeading();
		
		NdPoint pos = new NdPoint(0,0);
		int visibleAgents = d_thisAgent.getNumVisibleOthers();
		
		if (visibleAgents > 0){   // there are agents in my sight
			Vector<CrowdAgent> agents = Misc.agentsInCone(context, d_thisAgent, Constants.GAZE_DEPTH, Constants.GAZE_WIDTH);			
			pos = Misc.posNearestAgent(context, mypos, agents.iterator(), d_thisAgent);   // head to nearest agent position
			mostclose = space.getDistance(mypos, pos);	
		}
		else {  // ok then the expensive way and check all agents to see which one is more close
			Iterator<CrowdAgent> allAgents = agent_scontext.iterator();
			pos = Misc.posNearestAgent(context, mypos, allAgents, d_thisAgent);	
			mostclose = space.getDistance(mypos, pos);	
		}	
		
		return Misc.heading2point(mypos, pos);
	}
	
	
	/**
	 * Function describes the heading change of an agent
	 * as the pre-movement part of moveSubs behaviour
	 * This involves moving towards the toilet or bar, 
	 * driven by the need for subsistence satisfaction
	 * This can be achieved by going to the toilet or go to the bar.
	 * Depending on which of them will bring satisfaction to the agent.
	 * 
	 * In going to the bar or toilet, the agent has 3 points where it can
	 * Fulfil its needs. It selects the one that is most nearby, however
	 * when it is crowded the agent will move to the next one.
	 * 
	 * returns the heading suitable leading to subsistence goal satisfaction
	 * 
	 */
	public double subsHeading(){
		Context context = RunState.getInstance().getMasterContext(); 
		ContinuousSpace space = (ContinuousSpace) context.getProjection("Continuous Space");

		NdPoint mypos = d_thisAgent.getMyPos();	
		
		double distT0 = space.getDistance(mypos, Constants.POI_TOILET);
		double distT1 = space.getDistance(mypos, Constants.POI_TOILET1);
		double distT2 = space.getDistance(mypos, Constants.POI_TOILET2);
		double minTdist = Math.min(Math.min(distT0, distT1), distT2);
		
		double distB0 = space.getDistance(mypos, Constants.POI_BAR);
		double distB1 = space.getDistance(mypos, Constants.POI_BAR1);
		double distB2 = space.getDistance(mypos, Constants.POI_BAR2);
		double minBdist = Math.min(Math.min(distB0, distB1), distB2);
		
		boolean crowdedT0 = Misc.crowded(space, Constants.POI_TOILET, 3);
		boolean crowdedT1 = Misc.crowded(space, Constants.POI_TOILET1, 3);
		boolean crowdedT2 = Misc.crowded(space, Constants.POI_TOILET2, 3);
		
		boolean crowdedB0 = Misc.crowded(space, Constants.POI_BAR, 3);
		boolean crowdedB1 = Misc.crowded(space, Constants.POI_BAR1, 3);
		boolean crowdedB2 = Misc.crowded(space, Constants.POI_BAR2, 3);
		
		if (d_thisAgent.getDomPhysio().equalsIgnoreCase(Constants.BLADDER)){
			
			if (distT0 == 0 || distT1 == 0 || distT2 == 0) {
				return randomHeading();
			}
			if (crowdedT0 && crowdedT1 && crowdedT2){
				int random = (int) (Math.random() % 2);
				switch (random) {
				case 0: return Misc.heading2point(mypos, Constants.POI_TOILET);
				case 1: return Misc.heading2point(mypos, Constants.POI_TOILET1);
				case 2: return Misc.heading2point(mypos, Constants.POI_TOILET2);
				default: return Misc.heading2point(mypos, Constants.POI_TOILET2); 
				} 
			}
			else {	
			if (minTdist == distT0){
				if (minTdist < 5 & crowdedT0){
					return Misc.heading2point(mypos, Constants.POI_TOILET1);
				} else {
					return Misc.heading2point(mypos, Constants.POI_TOILET);
				}
			} 
			else if (minTdist == distT1){
				if (minTdist < 5 & Misc.crowded(space, Constants.POI_TOILET1, 3)){
					return Misc.heading2point(mypos, Constants.POI_TOILET2);
				} else {
					return Misc.heading2point(mypos, Constants.POI_TOILET1);
				}
			} else {
			        return Misc.heading2point(mypos, Constants.POI_TOILET2);
				}
			}
	}
		
		else { // go to the bar (when not already there)			
			if (distB0 == 0 || distB1 == 0 || distB2 == 0) {
				return randomHeading();
			}
			if (crowdedB0 && crowdedB1 && crowdedB2){
				int random = (int) (Math.random() % 2);
				switch (random) {
				case 0: return Misc.heading2point(mypos, Constants.POI_BAR);
				case 1: return Misc.heading2point(mypos, Constants.POI_BAR1);
				case 2: return Misc.heading2point(mypos, Constants.POI_BAR2);
				default: return Misc.heading2point(mypos, Constants.POI_BAR2); 
				} 
			}

			if (minBdist == distB0){
				if (minTdist < 5 & crowdedB0){
					return Misc.heading2point(mypos, Constants.POI_BAR1);
				} else {
					return Misc.heading2point(mypos, Constants.POI_BAR);
				}
			} 
			else if (minBdist == distB1){
				if (minTdist < 5 & crowdedB1){
					return Misc.heading2point(mypos, Constants.POI_BAR2);
				} else {
					return Misc.heading2point(mypos, Constants.POI_BAR1);
				}
			}
			else return Misc.heading2point(mypos, Constants.POI_BAR2);
		} 
	}
	
	/**
	 * Changes the heading of the agent towards a specific point
	 * 	
	 * @param poiPos
	 */	

	
	/**
	 *  Changes the current heading of the agent in a random direction
	 *  
	 *  The random number generator generates a uniform value between [0,1) 
	 *  Meaning that the change of getting a number between 0 and 1 is just as likely (never becomes 1)
	 *  By multiplying with 360 degrees, the range is scaled to [0,360)
	 *  
	 *  	
	 */
		public double randomHeading(){
			return MiscSingleton.getInstance().getMyUniform().nextDoubleFromTo(0, Constants.DEGREES);
		}
	
		
	/**
	 * MoveSimple describes movement in the most simple form.
	 * Given a current heading and velocity the new target
	 * position is calculated. Given the walkability of that
	 * position the agent will move, otherwise it will stand still.
	 * 
	 * @param velocity
	 */
		
	public void moveSimple(double v){
		
		/* Get the context in which the agent is residing */
		Context context = RunState.getInstance().getMasterContext();
		/* Get the patch grid from the context */
		Grid patch = (Grid) context.getProjection("Simple Grid");
		/* Get the continuous space from the context */
		ContinuousSpace space = (ContinuousSpace) context.getProjection("Continuous Space");
		NdPoint pos = d_thisAgent.getMyPos();
		double myheading = d_thisAgent.getHeading();
		double xpos = pos.getX();
		double ypos = pos.getY();
		
		/* calculate target position */
		NdPoint target = MathUtils.calcNewPosition(xpos, ypos, v, myheading);
		GridPoint targetPatch = Misc.nd2gridpoint(target);
		int x = targetPatch.getX();
		int y = targetPatch.getY();
		
		// check whether the target location is walkable
		boolean walkableTarget = Misc.walkablePatch(context, d_thisAgent, target);
		//System.out.println("patch: " + target + " is beloopbaar: " + walkableTarget);
		
		if (walkableTarget) {
			/* Move the agent on the space by one unit according to its new heading */
			double r = target.getX(); 
			double s = target.getY();
			
			space.moveTo(d_thisAgent, target.getX(), target.getY(), 0);
			/* Move the agent to its new patch (note the patch may not actually change)*/
			patch.moveTo(d_thisAgent, x, y); 

			d_thisAgent.setCannotMove(false);	
		} else {
			d_thisAgent.setCannotMove(true);
		}

	}
	
	
	/**
	 * MoveBraitenBerg describes movement still very simple,
	 * however if the target position ahead isn't walkable, the
	 * free spots +/-45 degrees the agent will move to that spot
	 * 
	 * @param velocity
	 */
		
	public void move(double v){

		/* Get the context in which the agent is residing */
		Context context = RunState.getInstance().getMasterContext();
		/* Get the patch grid from the context */
		Grid patch = (Grid) context.getProjection("Simple Grid");
		/* Get the continuous space from the context */
		ContinuousSpace space = (ContinuousSpace) context.getProjection("Continuous Space");
		
		NdPoint pos = d_thisAgent.getMyPos();
		double myheading = d_thisAgent.getHeading();
		double tmpHeading = myheading;
		double xpos = pos.getX();
		double ypos = pos.getY();		
		
		d_thisAgent.heading2point = pos;
		
		/* calculate target position */
		NdPoint target = MathUtils.calcNewPosition(xpos, ypos, v, myheading);
		GridPoint targetPatch = Misc.nd2gridpoint(target);
		
		/* check whether the target location is walkable */
		boolean walkableTarget = Misc.walkablePatch(context, d_thisAgent, target); // whether the agent can walk directly in my most preferred direction
				
		if (!(walkableTarget)) { // Do da braitenberg
			
			tmpHeading = checkSurrounding(v);
 			target = MathUtils.calcNewPosition(xpos, ypos, v, tmpHeading);
 		
 			/* in case this doesn't work, try to walk 5 times@random */
 			for (int i = 0; i < 5; ++i){
 				if (!Misc.walkablePatch(context, d_thisAgent, target)){
 					tmpHeading = (Math.random() * Constants.DEGREES);
 					target = MathUtils.calcNewPosition(xpos, ypos, v, tmpHeading);
 				}
 			}
		} 
		
		walkableTarget = Misc.walkablePatch(context, d_thisAgent, target);
		if (walkableTarget){  					// Move!
			targetPatch = Misc.nd2gridpoint(target);
			int x = targetPatch.getX();
			int y = targetPatch.getY();
			
			space.moveTo(d_thisAgent, target.getX(), target.getY(), 0);
			patch.moveTo(d_thisAgent, x, y); // this is not necessary
			
			d_thisAgent.setCannotMove(false);
		} else {								//cannot move, just stand still
			d_thisAgent.setCannotMove(true);
		}
		
		
	}
		
		/**
		 *  braitenberg style surrounding check
		 */
		public double checkSurrounding(double v){
			
			Context context = RunState.getInstance().getMasterContext();
			
			NdPoint pos = d_thisAgent.getMyPos();
			double myheading = d_thisAgent.getHeading();
			double xpos = pos.getX();
			double ypos = pos.getY();
			
			double tmpHeading = myheading; //this is the intended/preferred direction
			int noise = 4; // braitenberg approach has a problem if both sides have the same input. Agent in a corner :) common solution: use noise)
			
			// sensors within vision range [180 degrees]
			NdPoint leftSensor1 = MathUtils.calcNewPosition(xpos, ypos, v, myheading - 90);
			NdPoint leftSensor2 = MathUtils.calcNewPosition(xpos, ypos, v, myheading - 75);
			NdPoint leftSensor3 = MathUtils.calcNewPosition(xpos, ypos, v, myheading - 60);
			NdPoint leftSensor4 = MathUtils.calcNewPosition(xpos, ypos, v, myheading - 45);
			NdPoint leftSensor5 = MathUtils.calcNewPosition(xpos, ypos, v, myheading - 30);
			NdPoint leftSensor6 = MathUtils.calcNewPosition(xpos, ypos, v, myheading - 15);
			
			NdPoint rightSensor1 = MathUtils.calcNewPosition(xpos, ypos, v, myheading + 90);
			NdPoint rightSensor2 = MathUtils.calcNewPosition(xpos, ypos, v, myheading + 75);
			NdPoint rightSensor3 = MathUtils.calcNewPosition(xpos, ypos, v, myheading + 60);			
			NdPoint rightSensor4 = MathUtils.calcNewPosition(xpos, ypos, v, myheading + 45);
			NdPoint rightSensor5 = MathUtils.calcNewPosition(xpos, ypos, v, myheading + 30);
			NdPoint rightSensor6 = MathUtils.calcNewPosition(xpos, ypos, v, myheading + 15);
			
			// doesn't matter in what order this is done, as the overall influence determines the heading (influence adds up)

			tmpHeading = (!Misc.walkablePatch(context,d_thisAgent, leftSensor5)) ? tmpHeading + 30 : tmpHeading;
			tmpHeading = (!Misc.walkablePatch(context,d_thisAgent, leftSensor3)) ? tmpHeading + 30 : tmpHeading;
			tmpHeading = (!Misc.walkablePatch(context,d_thisAgent, leftSensor1)) ? tmpHeading + 30 : tmpHeading;
			
			tmpHeading = (!Misc.walkablePatch(context,d_thisAgent, rightSensor5)) ? tmpHeading - 30 : tmpHeading;
			tmpHeading = (!Misc.walkablePatch(context,d_thisAgent, rightSensor3)) ? tmpHeading - 30 : tmpHeading;
			tmpHeading = (!Misc.walkablePatch(context,d_thisAgent, rightSensor1)) ? tmpHeading - 30 : tmpHeading;
						
			tmpHeading = tmpHeading % 360; // keep the rang [0-360] to be able to compare
			tmpHeading = (tmpHeading < 0 ? 360 + tmpHeading : tmpHeading);
			
			if (tmpHeading == myheading){
				tmpHeading = (Math.random() % 2 == 0 ? tmpHeading + 30 : tmpHeading - 30);
				}
			
		return tmpHeading;
	}
	
		
	/**
	  * Describes the movement of an agent. 
	  * Given a heading and a walking speed an approachable target location is found.
	  * 
	  */

	public void moveOriginalMess(double v){
		/* Get the context in which the agent is residing */
		Context context = RunState.getInstance().getMasterContext();
		/* Get the patch grid from the context */
		Grid patch = (Grid) context.getProjection("Simple Grid");
		/* Get the continuous space from the context */
		ContinuousSpace space = (ContinuousSpace) context.getProjection("Continuous Space");
		NdPoint pos = d_thisAgent.getMyPos();
		double myheading = d_thisAgent.getHeading();
		double xpos = pos.getX();
		double ypos = pos.getY();
		
		/* calculate target position */
		NdPoint target = MathUtils.calcNewPosition(xpos, ypos, v, myheading);
		GridPoint targetPatch = Misc.nd2gridpoint(target);
		int x = targetPatch.getX();
		int y = targetPatch.getY();
		
		/* check whether the target location is walkable */
		boolean walkableTarget = Misc.walkablePatch(context, d_thisAgent, target);
		
		if (walkableTarget) {
			/* Move the agent on the space by one unit according to its new heading */
			double r = target.getX(); 
			double s = target.getY();
						
			space.moveTo(d_thisAgent, target.getX(), target.getY(), 0);
			/* Move the agent to its new patch (note the patch may not actually change) */
			patch.moveTo(d_thisAgent, x, y);  

			d_thisAgent.setCannotMove(false);	
		}
		
		else { // if straight ahead is not possible
			
			// Sensor array
			NdPoint leftSensor1 = MathUtils.calcNewPosition(xpos, ypos, v, myheading - 45);
			NdPoint leftSensor2 = MathUtils.calcNewPosition(xpos, ypos, v, myheading - 30);
			NdPoint leftSensor3 = MathUtils.calcNewPosition(xpos, ypos, v, myheading - 15);
			
			NdPoint rightSensor1 = MathUtils.calcNewPosition(xpos, ypos, v, myheading + 45);
			NdPoint rightSensor2 = MathUtils.calcNewPosition(xpos, ypos, v, myheading + 30);
			NdPoint rightSensor3 = MathUtils.calcNewPosition(xpos, ypos, v, myheading + 15);
			
			
			// Do da braitenberg (note: still problem if both sides have the same input. Agent in een hoekje :) common solution: use noise)
			double tmpHeading = myheading; //this is the intended/preferred direction
			int noise = 4;	
			
			/* no preference for the right or left side to check */
			if ((Math.random() % 2) == 0){
				tmpHeading = (!Misc.walkablePatch(context,d_thisAgent, leftSensor3)) ? tmpHeading + (30 + noise) : tmpHeading;
				tmpHeading = (!Misc.walkablePatch(context,d_thisAgent, rightSensor3)) ? tmpHeading - (30 + noise) : tmpHeading;
			}
			else {
				tmpHeading = (!Misc.walkablePatch(context,d_thisAgent, rightSensor3)) ? tmpHeading - (30 + noise) : tmpHeading;
				tmpHeading = (!Misc.walkablePatch(context,d_thisAgent, leftSensor3)) ? tmpHeading + (30 + noise) : tmpHeading;
			}
			
			if ((Math.random() % 2) == 0){
			tmpHeading = (!Misc.walkablePatch(context,d_thisAgent, leftSensor2)) ? tmpHeading + (20 + noise) : tmpHeading;
			tmpHeading = (!Misc.walkablePatch(context,d_thisAgent, rightSensor2)) ? tmpHeading - (20 + noise) : tmpHeading;
			}
			else {
				tmpHeading = (!Misc.walkablePatch(context,d_thisAgent, rightSensor2)) ? tmpHeading - (20 + noise) : tmpHeading;
				tmpHeading = (!Misc.walkablePatch(context,d_thisAgent, leftSensor2)) ? tmpHeading + (20 + noise) : tmpHeading;
			}
			
			if ((Math.random() % 2) == 0){
				tmpHeading = (!Misc.walkablePatch(context,d_thisAgent, leftSensor1)) ? tmpHeading + (10 + noise) : tmpHeading;
				tmpHeading = (!Misc.walkablePatch(context,d_thisAgent, rightSensor1)) ? tmpHeading - (10 + noise) : tmpHeading;
			}
			else {
				tmpHeading = (!Misc.walkablePatch(context,d_thisAgent, rightSensor1)) ? tmpHeading - (10 + noise) : tmpHeading;
				tmpHeading = (!Misc.walkablePatch(context,d_thisAgent, leftSensor1)) ? tmpHeading + (10 + noise) : tmpHeading;
			}
				
			// So new position will be....
			NdPoint newPos = MathUtils.calcNewPosition(xpos, ypos, v, tmpHeading);
			
			// Tyr to walk 5 times random, otherwise stand still
			for (int i = 0; i < 5; ++i){
				if (!Misc.walkablePatch(context,d_thisAgent, newPos)){
			
					tmpHeading = (Math.random() * Constants.DEGREES);
					newPos = MathUtils.calcNewPosition(xpos, ypos, v, tmpHeading);
				}
			}
			if (!Misc.walkablePatch(context,d_thisAgent, newPos)){
				d_thisAgent.setCannotMove(true);

			} else {
			
				// Move it....
				GridPoint opt = Misc.nd2gridpoint(newPos);
			
				double a = newPos.getX();
				double b = newPos.getY();
			
				if (a < 0 || a >= Constants.WORLD_WIDTH){
					System.out.println("LocRule.move: x valt buiten de range, zou niet moeten: " + a);
				}
				if (b < 0 || b >= Constants.WORLD_HEIGHT){
					System.out.println("LocRule.move: y valt buiten de range, zou niet moeten" + b);
				}

				space.moveTo(d_thisAgent, newPos.getX(), newPos.getY(), 0);
				patch.moveTo(d_thisAgent, opt.getX(), opt.getY()); 
				d_thisAgent.setCannotMove(false);
			}
			
			
			double turn = 1.00 * Constants.GAZE_WIDTH;
			
			NdPoint option1 = MathUtils.calcNewPosition(xpos, ypos, v, myheading - turn);
			NdPoint option2 = MathUtils.calcNewPosition(xpos, ypos, v, myheading + turn);
			boolean walkableOption1 = Misc.walkablePatch(context,d_thisAgent, option1);
			boolean walkableOption2 = Misc.walkablePatch(context,d_thisAgent, option2);
			
			ArrayList<NdPoint> options = new ArrayList<NdPoint>();
			
			if (walkableOption1){options.add(option1);}
			if (walkableOption2){options.add(option2);}
			
			if (!options.isEmpty()){
				int i = (int) Math.random() % options.size();  // random number within the range of options
				NdPoint option = (NdPoint) options.get(i);
				GridPoint opt = Misc.nd2gridpoint(option);
				
				double a = option.getX();
				double b = option.getY();
				
				if (a < 0 || a >= Constants.WORLD_WIDTH){
					System.out.println("LocRule.move: x valt buiten de range, zou niet moeten: " + a);
				}
				if (b < 0 || b >= Constants.WORLD_HEIGHT){
					System.out.println("LocRule.move: y valt buiten de range, zou niet moeten" + b);
				}

				space.moveTo(d_thisAgent, option.getX(), option.getY(), 0);
				patch.moveTo(d_thisAgent, opt.getX(), opt.getY()); //redundant and causes problems
				d_thisAgent.setCannotMove(false);
			}
			else
				d_thisAgent.setCannotMove(true);
			/* else stay where you are */
			
		}
	}
	
	public void moveTest(){
		Context context = RunState.getInstance().getMasterContext(); 
		//Context subcontext = ContextUtils.getContext(d_thisAgent);
		Grid patch = (Grid) context.getProjection("Simple Grid");
		ContinuousSpace space = (ContinuousSpace) context.getProjection("Continuous Space");
		
		
		NdPoint testPos = new NdPoint(20,20);		
		NdPoint pos = d_thisAgent.getMyPos();
		d_thisAgent.setHeading(Misc.heading2point(pos, testPos));
		
		NdPoint target = MathUtils.calcNewPosition(pos.getX(), pos.getY(), Constants.V_DEFAULT, d_thisAgent.getHeading());
		GridPoint targetPatch = Misc.nd2gridpoint(target);
		
		double x = target.getX();
		double y = target.getY();
		
		if (Misc.walkablePatch(context, d_thisAgent, target)){
			space.moveTo(d_thisAgent, x,y,0);
			patch.moveTo(d_thisAgent, (int) x, (int) y); 
		}
	}
	
	/**
	 * @return the mostclose
	 */
	public double getMostClose() {
		return mostclose;
	}
	/**
	 * @return the nearest agents
	 */

	public CrowdAgent getNearestAgent() {
		return d_nearest;
	}
	
	/**
	 * Returns the position this agent is heading to
	 * when moving social
	 * @return
	 */
	public NdPoint getHpos(){
		return d_hpos;
	}
	
	/**
	 * Sets the position where this agent is heading
	 * to when moving social
	 * 
	 * @param hpos
	 */
	public void setHpos(NdPoint hpos){
		d_hpos = hpos;
	}
	
}
	
