package crowdmodel.agents;

import java.util.*;

import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.parameter.Parameters;
import repast.simphony.space.graph.Network;
import repast.simphony.context.DefaultContext;
import repast.simphony.context.space.graph.NetworkBuilder;
import crowdmodel.Constants;
import crowdmodel.gui.GUIparams;
import crowdmodel.utils.Misc;
import crowdmodel.utils.MiscSingleton;

/**
 * Memory is a main part of the model structure. The main types of the model
 * structure consists out of a sensor, motor, memory and physiology part.
 * All there is to describe (behaviour, influences factors, processes,
 * characteristics etc.) need to be described in either one of the (derived)
 * main parts of the model structure
 * 
 * Memory: describes the functional part of the model structure This is
 * where behaviour is chosen and (mainly) is influenced. Memory consists out
 * of memoryElements that represent the knowledge (goals, facts & rules)
 * 
 * The memory exists only in an agent, this is why it is not an CObject and not
 * a part of the overall context. Memory is a context on its own, in which the memory
 * elements are the members of the context and a projection describes their 
 * underlying relationship (in an association network)
 * 
 * The elements stored in memory are stored in a Map. Which makes them retrievable by name.
 * This is different from the usual context in which this is stored in a Collection.
 * To be able to work with this I overloaded some functions: <br>
 * <ul>
 * <li> add(MemoryElement element) : overloads Collection.remove(Object o) </li> 
 * <li> remove(MemoryElement element) : overloads Collection.remove(Object o) </li>
 * <li> contains(MemoryElement element) : overloads AbstractContext.contains(Object o) </li> 
 * </ul>
 * But also added some functions as you want some of the collection functionality: <br>
 * <ul>
 
 * <li> getIterator </li>
 * </ul>
 * 
 * This makes the usage of some other functions abundant, however still retrievable. Due to lack of
 * time I cannot catch these exceptions or to write a new AbstractContext that implements the Collections framework
 * that that allows me to override the add/remove function or just specifically for this Memory. So causion in using:<br>
 * - Iterable() : creates an iterable over an empty collection, that doesn't represent memory as everything is stored in memoryMap. Use getIterator instead <br>
 * - add or remove (Object o)  % would be nice to overload this one <br>
 * - actually any function of the Collections framework as it doesn't refers to a collection of MemoryElements
 * 
 * 
 * 
 * @author Nanda Wijermans
 */

public class Memory extends DefaultContext {

	private CrowdAgent d_thisAgent;  		//Refers to the agent this memory belongs to
	private boolean d_thisAgentFlag;		//indicates whether d_thisAgent has already been set before
	
	private Network<MemoryElement> d_associations;
	
	private HashMap<String, MemoryElement> d_memoryMap; // container that represents memory: collection of memoryElements are stored in here
	private HashMap<String, BehaviourFact> d_behaviourFacts; // contains the facts related to a behaviour rule
	private HashMap<String, PersonFact> d_hashFriends; // contains the friends an individual has
	private LinkedList<BehaviourRule> d_orderedBehaviours; // contains all the behaviourRules, ordered by their activationLevel
	private ArrayList<PersonFact> d_friends;  	// contains the friends an individual has
	private ArrayList<PersonFact> d_leaders;  	// contains the agents that are regarded as leader
	private ArrayList<PersonFact> d_persons;	// contains all the persons this agent knows
	private Iterator<BehaviourRule> d_behaviours;
	
	private double d_time;
	private Goal d_domGoal; 	// the dominant goal: id, social, safety, or subsistence

	private Random d_rng;		// Random number generator
	
	private Goal d_identity, d_social, d_safety, d_subsistence, d_empty;
	protected LocomotionRule d_walk, d_run, d_dance, d_sitDown, d_standUp, d_kneel, d_lieDown;  // om de functies te testen
	private BehaviourRule d_other, d_nothing;
	private BehaviourFact d_otherExp, d_nothingExp, d_walkExp, d_runExp, d_danceExp, d_sitExp, d_standExp, d_kneelExp, d_lieExp;
	private PersonFact d_self, d_ingroup, d_outgroup, d_unknown;
	
	/**
	 * Association map represent the links/connections between memoryElements
	 * they are represented here and not locally in the memoryElements for
	 * computational reasons Every association in here has a key and a slot. The
	 * key is needed to get to the slot (content) The slot contains the
	 * association strength between the two memoryElements [0,1]
	 * 
	 * Each association is easily found by the key that incorporates the
	 * ID-number of both memoryElements key = highesIDtnumber [sign]
	 * lowestIDnumber slot =
	 * 
	 * Note: At this point these associations are fixed, set while initialising
	 * an agent with its memory. However in experiencing (events, or just
	 * perceiving during time) these association can change (strengthen or weaken).
	 * this is a form of learning that is not implemented here yet. However
	 * is constructed as such that is can change/learn.
	 * 
	 */
	

	/**
	 * The memory constructor initialises memory by creating goals, other
	 * memory elements, and their associations
	 */

	public Memory() {	
		d_memoryMap = new HashMap<String, MemoryElement>();
		d_behaviourFacts = new HashMap<String, BehaviourFact>();
		d_hashFriends = new HashMap<String, PersonFact>();
		d_friends = new ArrayList<PersonFact>();
		d_leaders = new ArrayList<PersonFact>();
		d_orderedBehaviours = new LinkedList<BehaviourRule>();
		d_thisAgentFlag = false;
		d_empty = new Goal("empty", 0, 0);

		createGoals();
		setDomGoal();
		createBehaviours();
		setBehaviourIterator();                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    
		createFacts();
	
		NetworkBuilder<MemoryElement> builder = new NetworkBuilder<MemoryElement>("memory", this, false);  // network projection
		d_associations = builder.buildNetwork();
		setupAssociations();		
	}

	/**
	 * This function overloads the general add function that is specified in Collection and
	 * (overridden by Context)
	 * The elements of Memory are collected within a Map instead of a Collection. As they
	 * are easily called based on their name.
	 * 
	 * @param name
	 * @param element
	 * @return boolean, whether the addition worked or not
	 */

	public boolean add(MemoryElement element){
		String name = element.d_name;
		d_memoryMap.put(name, element);
		return (d_memoryMap.containsKey(name));
	}
	
	
	/**
	 * This function overloads the general remove function that is specified in Collection and
	 * (overridden by Context)
	 * The elements of Memory are collected within a Map instead of a Collection. As they
	 * are easily called based on their name.
	 * 
	 * @param name
	 * @param element
	 * @return boolean, whether the addition worked or not
	 */
	public boolean remove(MemoryElement element){
		String name = element.d_name;
		d_memoryMap.remove(name);
		return (d_memoryMap.containsValue(element));
	}
	
	/**
	 * Function that checks whether a MemoryElement is stored in Memory
	 * This overloads the contains function of AbstractContext, which isn't suitable here
	 * as an abstractContext stores its objects in a Collections, whereas we do so in a Map.
	 * 
	 * @param me
	 * @return
	 */
	 
	public boolean contains(MemoryElement me){
		return d_memoryMap.containsValue(me);
	}
	
	
	/**
	 * Function that returns an iterator of all MemoryElements in Memory.
	 *
	 * @return iterator of the elements in Memory
	 */
	public Iterator getIterator(){
		return d_memoryMap.values().iterator();
	}
	
	/**
	 * The step function defines what an agent does internally every simulation step/tick.
	 * 
	 * This is a temporarily solution as I want the processes to be going parallel.
	 * Furthermore, I want to relate this to the internal time of an agent, that not necessarily need
	 * to be synchrony with the world time.
	 * 
	 */
	public void step(){
		Iterator<MemoryElement> memory = d_memoryMap.values().iterator();
		while (memory.hasNext()) {
			MemoryElement element = (MemoryElement) memory.next();
			element.approxB();
			element.updateActivation();
		}
	}
	
	/**
	 * Sets the Agent to whom this memory belongs
	 * This allows for the 'direct' access of methods/vars concerning memory
	 * by the functions in Memory;
	 *  
	 *  The function checks whether this has already been assigned, at this is
	 *  only done once while initialising. As this cannot be done in the constructor
	 *  this is the 'ugly' solution.
	 * @param thisAgent
	 */
	public void setThisAgent(CrowdAgent thisAgent){
		boolean flag = d_thisAgentFlag;
		CrowdAgent agent = thisAgent;
		if (!d_thisAgentFlag){
			d_thisAgent = thisAgent;
			d_thisAgentFlag = true;
		} else {
			System.out.println("this is an illegal operation");
			//make Syst.err;
		}
	}
	
	/**
	 * Sets for all memoryElements that they belong to which memory and
	 * corresponding agent.
	 * This allows for the 'direct' access of methods/vars concerning memory and
	 * agent by the	by the functions in MemoryElement;
	 *  
	 *  The function checks whether this has already been assigned, at this is
	 *  only done once while initialising. As this cannot be done in the constructor
	 *  this is the 'ugly' solution.
	 * @param thisAgent
	 */
	public void setElementsWithAgentAndMemory(){
		Iterator<MemoryElement> memoryElements = d_memoryMap.values().iterator();
		while (memoryElements.hasNext()){
			MemoryElement element = memoryElements.next();
			element.setThisAgent(d_thisAgent);
			element.setThisMemory(this);	
		}
	}
	
	/**
	 * Function sets the scenario in the memory of an agent
	 * This incorporates an individual having friends
	 * by having a personFact for each one of them
	 */
	
	public void setSocialScenario(Network socNetwork){
		Iterator<CrowdAgent> myFriends = socNetwork.getAdjacent(d_thisAgent).iterator();
		
		while (myFriends.hasNext()){
			CrowdAgent friend = myFriends.next();
			int friendID = friend.getAgentID();
			String name = Constants.P_NAME_FRIEND + friendID;
			PersonFact friendFact = new PersonFact(name, true, false, Constants.IN_GROUP, friendID);
			d_memoryMap.put(friendFact.getName(), friendFact);
			d_friends.add(friendFact);	
		}
	}
	
	
	/**
	 * Sets in memory which agents this agent perceives as a leader
	 * 
	 * NOTE: need to change this structure, PersonFacts need to be in one set, not separate for
	 * friends and leaders
	 * 
	 */
	public void setMyLeaders(HashSet<CrowdAgent> leaders){
		Iterator<CrowdAgent> myLeaders = leaders.iterator();
		
		while (myLeaders.hasNext()){
			CrowdAgent leader = myLeaders.next();
			int leaderID = leader.getAgentID();
			String name = Constants.P_NAME_LEADER + leaderID;
			PersonFact leaderFact = new PersonFact(name, true, true, Constants.IN_GROUP, leaderID);
			d_memoryMap.put(leaderFact.getName(), leaderFact);
			d_leaders.add(leaderFact);	
		}
	}
	
	/**
	 * Creates the goals an individual has in its memory.
	 * 
	 * In this structure an agent has four abstract goals (identity, social,
	 * safety, subsistence) that represent the drives/desired states of an
	 * agent.
	 */
	private void createGoals() {
				
		/* Creates the identity goal when it is indicated in
		 * the GUI to play a role in the behaviour of the agent
		 * given the settings
		 */
		if ((Boolean) GUIparams.getInstance().getIdGoal()){	
			double initID = GUIparams.getInstance().getInitId();
			double idPref = 0;
			String prefDist = GUIparams.getInstance().getIdDist();
			if (prefDist.equalsIgnoreCase(Constants.DIST_NORMAL)){
				idPref = MiscSingleton.getInstance().getMyBetaNormal().nextDouble();  //Beta distribution alpha = 5 and beta = 5; approximates the normal distribution		
			} else if (prefDist.equalsIgnoreCase(Constants.DIST_UNIFORM)){
				idPref = MiscSingleton.getInstance().getMyUniform().nextDouble();  // Value between 0 and 1, following a uniform distribution
			} else if (prefDist.equalsIgnoreCase(Constants.DIST_CUSTOM)){
				idPref = GUIparams.getInstance().getCustoId();
				}
			d_identity = new Goal(Constants.GOAL_NAME_IDENTITY, initID, idPref);
			this.add(d_identity);
		} else {
			d_identity = d_empty;
		}
		
		
		
		/* Creates the social goal when it is indicated in
		 * the GUI to play a role in the behaviour of the agent
		 * given the settings
		 */
		if ((Boolean) GUIparams.getInstance().getSocGoal()){				 
			double initSoc = GUIparams.getInstance().getInitSoc();
			double socPref = 0;
			String prefDist = GUIparams.getInstance().getSocDist();
			if (prefDist.equalsIgnoreCase(Constants.DIST_NORMAL)){
				socPref = MiscSingleton.getInstance().getMyBetaNormal().nextDouble();  //Beta distribution alpha = 5 and beta = 5; approximates the normal distribution
			} else if (prefDist.equalsIgnoreCase(Constants.DIST_UNIFORM)){
				socPref = MiscSingleton.getInstance().getMyUniform().nextDouble();  // Value between 0 and 1, following a uniform distribution
			} else if (prefDist.equalsIgnoreCase(Constants.DIST_CUSTOM)){
				socPref = GUIparams.getInstance().getCustoSoc();
			}	
			d_social = new Goal(Constants.GOAL_NAME_SOCIAL, initSoc, socPref);
			this.add(d_social);
		} else {
			d_social = d_empty;
		}
		
		/*
		 * Creates the safety goal when it is indicated in
		 * the GUI to play a role in the behaviour of the agent
		 * given the settings
		 */
		
		if ((Boolean) GUIparams.getInstance().getSafeGoal()){
			double initSafe = GUIparams.getInstance().getInitSafe();			
			double safePref = 0;
			String prefDist = GUIparams.getInstance().getSafeDist();
			if(RunEnvironment.getInstance().isBatch()) {
				Parameters p = RunEnvironment.getInstance().getParameters();
				prefDist = (String) p.getValue("safety_pref_dist");
			}	
	
			if (prefDist.equalsIgnoreCase(Constants.DIST_NORMAL)){
				safePref = MiscSingleton.getInstance().getMyBetaNormal().nextDouble();  //Beta distribution alpha = 5 and beta = 5; approximates the normal distribution
			} else if (prefDist.equalsIgnoreCase(Constants.DIST_LEFT_SKEW)){ // corresponds with friends crowd composition
				safePref = MiscSingleton.getInstance().getMyBetaLeft().nextDouble(); // sort of skewed distribution, most values lie between 0.2 and 0.3
			} else if (prefDist.equalsIgnoreCase(Constants.DIST_RIGHT_SKEW)){ // corresponds with family crowd composition
				safePref =  MiscSingleton.getInstance().getMyBetaRight().nextDouble(); // sort of skewed distribution, most values lie between 0.7 and 0.8
			} else if (prefDist.equalsIgnoreCase(Constants.DIST_UNIFORM)){
				safePref = MiscSingleton.getInstance().getMyUniform().nextDouble();  // Value between 0 and 1, following a uniform distribution
			} else if (prefDist.equalsIgnoreCase(Constants.DIST_CUSTOM)){
				safePref = GUIparams.getInstance().getCustoSafe();
			}
			d_safety = new Goal(Constants.GOAL_NAME_SAFETY, initSafe, safePref);
			this.add(d_safety);
		} else {
			d_safety = d_empty;
		}
		
		/*
		 * Creates the subsistence goal when it is indicated in
		 * the GUI to play a role in the behaviour of the agent
		 * given the settings
		 */
		if ((Boolean) GUIparams.getInstance().getSubsGoal()){		
			
			double initSubs = 0;  // is done separately as it is based on physiology
			double subsPref = 0;
			String prefDist = GUIparams.getInstance().getSubsDist();
			if (prefDist.equalsIgnoreCase(Constants.DIST_NORMAL)){
				subsPref = MiscSingleton.getInstance().getMyBetaNormal().nextDouble();  //Beta distribution alpha = 5 and beta = 5; approximates the normal distribution
			} else if (prefDist.equalsIgnoreCase(Constants.DIST_UNIFORM)){
				subsPref = MiscSingleton.getInstance().getMyUniform().nextDouble();;  // Value between 0 and 1, following a uniform distribution
			} else if (prefDist.equalsIgnoreCase(Constants.DIST_CUSTOM)){
				subsPref = GUIparams.getInstance().getCustoSubs();
			}
			d_subsistence = new Goal(Constants.GOAL_NAME_SUBSISTENCE, initSubs, subsPref);
			d_memoryMap.put(d_subsistence.getName(), d_subsistence);
			this.add(d_subsistence);
		} else {
			d_subsistence = d_empty;
		}
	}
	
	public void initSubs(double initVal){
		d_subsistence.setSatisfaction(initVal);
	}
	
	/**
	 * Creates the facts an individual has in its memory
	 * There are 3 types of facts: behaviour, person, (event) facts
	 */
	public void createFacts() {
		
		/* person facts */
		d_self = new PersonFact(Constants.P_NAME_SELF, true, false, Constants.IN_GROUP, -1);
		d_ingroup = new PersonFact(Constants.P_NAME_INGROUP, false, false, Constants.IN_GROUP, -1);
		d_outgroup = new PersonFact(Constants.P_NAME_OUTGROUP, false, false, Constants.OUT_GROUP, -1);
		d_unknown = new PersonFact(Constants.P_NAME_UNKNOWN, false, false, Constants.UNKNOWN, -1);
		
		this.add(d_self);
		this.add(d_ingroup);
		this.add(d_outgroup);
		this.add(d_unknown);
		
		
		/* behaviour facts - locomotion */
		
		
		d_nothingExp = new BehaviourFact(Constants.F_NAME_ENOTHING, 0,0,0,0);
		
		d_walkExp = new BehaviourFact(Constants.F_NAME_EWALK, 0.3, 0.3, 0.2, 0.1);
		d_runExp = new BehaviourFact(Constants.F_NAME_ERUN, 0.1, 0.1, 0.4, 0.1);
		d_danceExp = new BehaviourFact(Constants.F_NAME_EDANCE, 0.3, 0.3, 0, 0);

		this.add(d_nothingExp);
		
		this.add(d_walkExp);
		this.add(d_runExp);
		this.add(d_danceExp);
	}
	
	/**
	 * Creates the internal representation of 
	 * a behaviour (= a behaviourRule) 
	 * There are 3 general types of behaviour: locomotion, manipulation and communication
	 */
	public void createBehaviours() {
		
		
		/* Locomotion Behaviour Rules */
		d_walk = new LocomotionRule(Constants.B_NAME_WALK);
		d_run = new LocomotionRule(Constants.B_NAME_RUN);
		d_dance = new LocomotionRule(Constants.B_NAME_DANCE);

		this.add(d_walk);
		this.add(d_run);
		this.add(d_dance);
		
		d_orderedBehaviours.add(d_walk);
		d_orderedBehaviours.add(d_run);
		d_orderedBehaviours.add(d_dance);
	}

	/**
	 * Sets d_domGoal with the most dominant goal for the given time
	 * There is a order of importance when the dominance values are identical
	 * then the goals become dominant in the order of: safety, subsistence, social (i doubt about the order of the last two in our setting)
	 */
	public void setDomGoal() {
		
		d_domGoal = d_social;
		if (d_identity.getName() != "empty"){
			if(d_social.getName() == "empty"){
				if (d_identity.getD_dominance() >= d_domGoal.getD_dominance()) {
					d_domGoal = d_identity;
				}
			} else {
				if (d_identity.getD_dominance() > d_domGoal.getD_dominance()) {
					d_domGoal = d_identity;
				}
			}
		}
		if (d_subsistence.getName() != "empty"){
			if (d_subsistence.getD_dominance() >=  d_domGoal.getD_dominance()) {
				d_domGoal = d_subsistence;
			}  
		}
		if (d_safety.getName() != "empty"){
			if (d_safety.getD_dominance() >= d_domGoal.getD_dominance()) {
				d_domGoal = d_safety;
			}
		}
		if (d_domGoal.getName() == "empty"){
			System.out.println("Memory.setDomGoal: this cannot be, agents have no goal");
		}

	}
	
	/**
	 * Creates the links between the memory element with a certain initial
	 * association value
	 * 
	 * Now I directly connected behaviourRules with behaviourFacts. I am going
	 * to change this later, but I need this connection now to make it work.
	 * So it is a matter of restructuring the implementation, however this connections
	 * works as the model describes it.
	 * 
	 */
	public void setupAssociations() {
		// goals linked to behaviours (rules)
		// behaviourRules and behaviourFacts associated with each other
			
		d_associations.addEdge(d_walk, d_walkExp, 1.0);
		d_associations.addEdge(d_run, d_runExp, 1.0);
		d_associations.addEdge(d_dance, d_danceExp, 1.0);
			}

	/**
	 * Returns the internal time an agent has to select a behaviour
	 * @return d_time
	 * 
	 */
	public double getTime() {
		return d_time;
	}

	/**
	 * Sets the internal time an agent has, to select a behaviour 
	 * @param time
	 */
	protected void setTime(double time) {
		d_time = time;
	}

	/**
	 * Returns the dominant goal that is currently dominant and influencing
	 * behaviour in the cognitive system.
	 * 
	 * @return d_domGoal
	 */
	public Goal getDomGoal() {
		return d_domGoal;
	}

	/**
	 * Returns the identity goal
	 * 
	 * @return d_identity - the identity goal
	 */
	public Goal getIdentity() {
		return d_identity;
	}
	
	/**
	 * Returns the social goal
	 * 
	 * @return d_social - the social goal
	 */
	public Goal getSocial() {
		return d_social;
	}

	/**
	 * Returns the safety goal
	 * 
	 * @return d_safety - the safety goal
	 */
	public Goal getSafety() {
		return d_safety;
	}

	/**
	 * Returns the subsistence goal
	 * 
	 * @return d_subsistence - the subsistence goal
	 */
	public Goal getSubsistence() {
		return d_subsistence;
	}

	/**
	 * Returns the HashMap in which all the memoryElements are stored
	 * 
	 * @return d_memoryMap - the collection of memoryElements
	 */
	public HashMap<String, MemoryElement> getMemoryElements(){
		return d_memoryMap;
	}
	
	/**
	 * @return a string containing all the names of the memoryElements currently in memory
	 */
	public String getNamesMemoryElements(){
		String elements = new String();
		Set<Map.Entry<String, MemoryElement>> memorySet = d_memoryMap.entrySet();
		for (Map.Entry<String, MemoryElement> element : memorySet) {
			elements += element.getValue().getName() + ", "; 
		}
		return elements;
	}
	
	/**
	 * Returns the behaviourFact corresponding to a behaviour name.
	 * 
	 * Behaviour facts are currently stored within a hashmap of behaviourFacts. 
	 * I want to associate behaviourRules with Facts in another way, however 
	 * that takes more time as I have to create a subcontext for every agent
	 * in which all memory elements are connected. 
	 * 
	 * At this point I relate the behaviourRules with the facts by this HashMap where
	 * the name labels of the behaviourRules are the keys and the Facts are in the value slots
	 * 
	 * @return d_behaviourFact - a fact about a behaviour
	 */
	public BehaviourFact getBehaviourFact(String bName){
		String bFactName = Misc.bName2fName(bName);
		return (BehaviourFact) d_memoryMap.get(bFactName); 
	}
	
	
	/**
	 * Returns the friends an individual has. 
	 * 
	 * @return d_friends - a hashSet of friends (personFacts). 
	 */
	public ArrayList<PersonFact> getFriends(){
		return d_friends;
	}
	
	/**
	 * Returns the agent this agent perceives as a leader 
	 * 
	 * @return d_friends - a hashSet of friends (personFacts). 
	 */
	public ArrayList<PersonFact> getLeaders(){
		return d_leaders;
	}
	
	/**
	 * Sets the iterator that returns behaviours in order of their current activation level
	 * As this is a class variable, it needs to be set before starting to compare behaviours
	 * as during comparing the iterator while first return the highest activation, then the
	 * second highest, etc. 
	 */
	public void setBehaviourIterator(){
		d_behaviours = d_orderedBehaviours.iterator();
	}
	
	/**
	 * Returns the next most active behaviour. As behaviours have an order based on their activation
	 * level. This order is used to compare behaviours with one and another.
	 * 
	 * @return behaviour - next behaviour with the highest activation
	 */
	public BehaviourRule getNextActiveB(){
		BehaviourRule b = (BehaviourRule) d_behaviours.next();
		d_thisAgent.decreaseInternalTime(Constants.BRULE_RETRIEVE_TIME);
		return b ;
	}
	
	/**
	 * Returns whether there is a next active behaviour in memory left
	 * @return availability of a behaviour in the iterator
	 */
	public boolean hasNextB(){
		return d_behaviours.hasNext();
	}
	
	/**
	 * Returns the behaviourRule given a name
	 * 
	 * @param name
	 * @return
	 */
	public BehaviourRule getBehaviour(String name){
		return (BehaviourRule) d_memoryMap.get(name);
	}
	
	/**
	 * Returns an element from memory based on its name
	 * In naming elements we distinct betwee
	 *
	 * @param eName
	 * @return
	 */
	
	public MemoryElement getElement(String eName){
		return d_memoryMap.get(eName);
	}
	
	/**
	 * Returns whether memory has this element
	 * This is an overloading function. As the context
	 * memory has an function contains as well, however takes
	 * an object as an argument instead of a String (element name).
	 * 
	 * @param eName
	 * @return
	 */
	public Boolean contains(String eName){
		return d_memoryMap.containsKey(eName);
	}
	
	
	/**
	 * This function is called when one of the behaviourRules activation has been changed.
	 * As d_orderedBehaviours is a list that contains all the behaviours ordered based on their activity
	 * This function keeps on having this ordering, allowing for changes.
	 * 
	 * @param b
	 */
	public void updateBehaviourOrdering(BehaviourRule b){
		d_orderedBehaviours.remove(b);
		d_orderedBehaviours.add(b);
	}

	public double getIDPref() {
		return d_identity.getPreference();
	}
	
	public double getSocialPref() {
		return d_social.getPreference();
	}
	public double getSafePref() {
		return d_safety.getPreference();
	}
	public double getSubsPref() {
		return d_subsistence.getPreference();
	}
	
}
