 package crowdmodel.agents;

import java.util.LinkedList;

import repast.simphony.engine.environment.RunEnvironment;
import crowdmodel.utils.MiscSingleton;

/**
 * A memoryElement is a component of the Memory.
 * Represents an atom of knowledge in this structure.
 * The forms of knowledge are either a goal, a fact or a rule
 * specified in the derived classes
 * 
 * In general, every memoryElement can be activated, which makes
 * it more salient (approachable), and is linked to other memory elements
 * in an associative way (i.e. stronger, weaker links).
 * 
 */

public abstract class MemoryElement {
	
	int d_id;
	CrowdAgent d_thisAgent;		//refers to the CrowdAgent object it belongs to
	Memory d_thisMemory;		//refers to the Memory object it belongs to
	boolean d_thisAgentFlag;
	boolean d_thisMemoryFlag;
	
	String d_name;

	double d_activation;
	protected double d_baseLevel;			// part of activation based on objective perception (priming), direct influence
	protected double d_contextLevel;		// part of activation based on context/subjective perception, experience, indirect influence

	
	/* class variables for the Base-level approximation function */
	
	private final static int d_K = 1; 	// depth-variable: number of registered moments in the past. Every time a memory element is used, this time is registered. d_K indicates the last d_K moment of usage that are stored.
	private double d_Tfirst;			// time-step that the element is used for the first time

	private LinkedList<Double> d_T;		// the last k times an element is used: the times are registered in the LinkedList. Updated according to a queuing principle (FIFO) 
	private double d_n;					// total number of times this element has been used/primed | 
	
	/* test vars */
	private double d_tLastPrime;
	
	
	/**
	 * Constructor
	 * 
	 */
	public MemoryElement(){
		d_id = MiscSingleton.getInstance().nextID();
		d_thisAgentFlag = false;
		d_thisMemoryFlag = false;
		d_activation = 0;
		d_baseLevel = 0;
		d_contextLevel = 0;
		
		d_Tfirst = 0; // meaning not used yet!, is going to be noted when used for the first time
		d_n = 0;
		d_T = new LinkedList<Double>();
		
		d_tLastPrime = 0;
	}
	
	/**
	 * Primes a memoryElement
	 * Psy terms: This means that a concept in memory is getting more salient/dominant.
	 * Coding: the (base-level) activation of this element increases given a function
	 *   
	 *  When an element is primed it 'uses' the element, resulting in an increased activation
	 *  It stores the time in d_T
	 */
	public void prime(){
		double currentTime = RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
		d_tLastPrime = currentTime;
		d_T.addFirst(currentTime); 
		++d_n;  // increment the number of total uses
		
		if (d_n > d_K) {  //when list d_T exceeds the number of storage slots, delete the oldest
			d_T.removeLast();
		} 
		else if (d_n == 1){
			d_Tfirst = currentTime;
		}
		approxB();
		updateActivation();
		
	}
	
	public void contextActivation(){
	}
	

	/**
	 * Function calculates the approximation of the base-level activation of a memoryElement.
	 * Base-level activation is a part of the total activation of a memory element that describes
	 * the recency and frequency of use of this particular element.
	 * 
	 * This computational efficient approximation function (Petrov 2006) generates the typical base-level activation dynamics: 
	 * sharp peak directly after usage, decay in absence of use, and gradual accretion with frequent use. It is more efficient as it does not need to register every usage of the memory element, it is only stored the last 
	 * k-times it was used. 
	 * 
	 * B = ln (SUM(1/sqrt(ti) + ((2(n-k))/(sqrt(tn) + sqrt(tk)));
	 * 
	 * ti = registered time (of usage) with index i <br>
	 * n  = number of times the element is used since it exists <br>
	 * k =  depth variable. Indicates the number items to be stored that represent the historic time of usage of this element. Here d_K = 1, as Petrov (2006) indicates that it works already very good.<br>
	 * i = Index (i) allows for finding the stored time-step (ti) of history element usage. <br>
	 * tn = lifespan, corresponds with the time a memory element started to exist, whereas (n) represents the total number of uses. <br>
	 * tk =  usespan, time since the Kth-storage. t_now - t_kth_usage = d_t - d_T[k]. As the history of usage is kept in a linkedList it is the last element that represents kth-usage.<br>
	 * 
	 * 
	 * Implementation:
	 * 
	 * Time registrations are stored in a linked list (d_T) that is restricted in having k slots. Follows a FIFO queueing principle.<br>
	 * d_K = k (depth variable)
	 * lifeSpan = tn ;
	 * useSpan = tk
	 * d_Tfirst = time on which the element started to exist
	 * currentTime = current simulation time
	 * 
	 */
	
	public void approxB(){
		double currentTime = RunEnvironment.getInstance().getCurrentSchedule().getTickCount(); //when I use multiple schedulers this could cause problems
		
		/* correction for the first steps where the element 'doesn't exist' yet  */
		if (currentTime < d_Tfirst || currentTime == 0){
			d_baseLevel = 0;
		}
		else { 			
			double lifeSpan = (currentTime <= d_Tfirst ? currentTime - d_Tfirst : 0); // life-span: time the element exists since first usage (t_now - t_first = d_t - d_Tfirst)
		 	double bsum = 0;
		 	if (!d_T.isEmpty()){
		 		for (int i = 0; i < d_K; ++i){
				double ti = d_T.get(i);   			//i-th stored time of past usage    
				double useSpan = currentTime - d_T.getLast(); // use-span: time interval between Kth-usage and now (t_now - t_kth_usage = d_t - d_T[k])
					bsum += (1/Math.sqrt((ti) + ((2 * (d_n - d_K))) / (Math.sqrt(lifeSpan) + Math.sqrt(useSpan))));
		 		}
		 	}
		 	d_baseLevel = Math.log(bsum);  //log is natural logarithm with groundnumber e
		}
		
	}
	
	
	public void updateActivation(){
		d_activation = d_baseLevel + d_contextLevel;
	}
	
	/**
	 * Sets the Agent where this MemoryElement belongs to
	 * This allows for the 'direct' access of methods/vars concerning the Agent
	 * by the functions in the MemoryElements;
	 * 
	 * 	The function checks whether this has already been assigned, at this is
	 *  only done once while initialising. As this cannot be done in the constructor
	 *  this is the 'ugly' solution. Cannot be done because the agent gives itself as a parameter
	 *  and in the constructor the creation of this object is not ready yet, so 
	 *  it is not possible to give itself as an argument.
	 *  
	 * @param thisAgent
	 */
	public void setThisAgent(CrowdAgent thisAgent){
		if (!d_thisAgentFlag){
			d_thisAgent = thisAgent;
			d_thisAgentFlag = true;
		} else {
			System.out.println("this is an illegal operation");
			//make Syst.err;
		}
	}
	
	/**
	 * Sets the Memory where this MemoryElement belongs to
	 * This allows for the 'direct' access of methods/vars concerning memory
	 * by the functions in the MemoryElements;
	 * 
	 * 	The function checks whether this has already been assigned, at this is
	 *  only done once while initialising. As this cannot be done in the constructor
	 *  this is the 'ugly' solution.
	 *  
	 * @param thisAgent
	 */
	public void setThisMemory(Memory thisMemory){
		if (!d_thisMemoryFlag){
			d_thisMemory = thisMemory;
			d_thisMemoryFlag = true;
		} else {
			System.out.println("this is an illegal operation");
			//make Syst.err;
		}
	}
	
	/**
	 * @return Name of the memory element
	 */
	public String getName() {
		return d_name;
	}
		
	/**
	 * 
	 * @return activation value of the memory element
	 */
	public double getActivation(){
		return d_activation;
	}
	
	/**
	 * 
	 * @return the unique ID number of this Object
	 */
	public int getID(){
		return d_id;
	}
	
	public String toString(){
		return d_name;
	}
	
	/**
	 * Returns the tick in which this memoryElement was lastly primed
	 */
	public double getLastPrimeTime(){
		return d_tLastPrime;
	}
}
