package crowdmodel.agents;

import crowdmodel.Constants;
import crowdmodel.utils.MiscSingleton;

/**
 * 
 * Physiology is one of the architecture main types.
 * Architecture consists out of a sensor, motor, memory and physiology part.
 * All there is to describe behaviour, influences factors, processes, characteristics etc. 
 * need to be described in either one of the (derived) main parts of the architecture
 * 
 * Physiology: describes the bio-state of an individual.  
 * It represents the bodily state, characteristics of a human being. Here those
 * elements are described that are important to represent all non-mental aspects
 * apart from sensing (e.g. arousal, energy, etc).
 * Physiology is affected by the external world as well as the mental state of an
 * individual. On its turn it affects the internal state of an individual.
 * 
 * In the current version physiology includes 3 factors:<br>
 * - arousal : indicates the level of awareness/alertness (indicated by increased heartbeat + bloodpressure) 
 * - bladder : related to subsistence, based on our festival context: physiological link to go to the toilet
 * - stomach : related to subsistence, based on our festival context: physiological link to go to the bar
 * 
 * @author Nanda Wijermans
 * 
 */

public class Physiology {

	private CrowdAgent d_thisAgent;  		//Refers to the agent this memory belongs to
	private boolean d_thisAgentFlag;		//indicates whether d_thisAgent has already been set before

	private double d_energy; 				//not used yet
	private double d_arousal;
	private double d_bladder;				// [0,1] full bladder (1) leads to urge to go to the toilet via increase in subsistence dominance
	private double d_stomach;				// [0,1] empty stomach (0) leads to urge to eat/drink something at the bar via subsistence dominance
	
	private String d_domPhysioString;
	private double d_domPhysioValue;
	
	/**
	 * Constructor sets the initial values of arousal, bladder and stomach
	 */
	public Physiology(){
		d_thisAgentFlag = false;	
		d_energy = 1;	// [0,1] not used yet
		d_arousal = 0.5;  // [0,1]

		d_bladder = MiscSingleton.getInstance().getMyBetaNormal().nextDouble();  // most people don't have to go to the toilet in the beginning
		d_stomach = MiscSingleton.getInstance().getMyBetaNormal().nextDouble();  // most people do want to go to the bar in the beginning

		setDomPhysiology();
	}
	
	/**
	 * Sets the Agent to whom this physiology belongs
	 * This allows for the 'direct' access of methods/vars concerning physiology
	 * by the functions in Physiology; As Inheritance works only in one direction, this works around
	 * it by being able to access the agent it self
	 *  
	 *  The function checks whether this has already been assigned, at this is
	 *  only done once while initialising. As this cannot be done in the constructor
	 *  this is the 'ugly' solution.
	 * @param thisAgent
	 */
	protected void setThisAgent(CrowdAgent thisAgent){
		boolean flag = d_thisAgentFlag;
		CrowdAgent agent = thisAgent;
		if (!d_thisAgentFlag){
			d_thisAgent = thisAgent;
			d_thisAgentFlag = true;
		} else {
			System.out.println("this is an illegal operation");
			//make Syst.err;
		}
		

	}
	
	/**
	 * The level of the bladder. [0,1]
	 * Full bladder (1) leads to urge to go to the toilet via increase in subsistence dominance
	 * 
	 * @return d_bladder
	 */
	public double getBladderLevel(){
		return d_bladder;
	}
	
	/**
	 * Sets the bladderLevel
	 * 
	 * @param bladder
	 */
	protected void setBladderLevel(double bladderLevel){
		d_bladder = bladderLevel;
	}
	
	/**
	 *  Updates the bladder level in case nothing is affecting the bladder externally.
	 *  This means that the bladder content is increasing during time.
	 *  
	 */
	protected void updateBladder(){
		d_bladder += 0.0010;    // after 1000 ticks an bladder is full
		d_bladder = (d_bladder < 0 ? 0 : d_bladder);
		d_bladder = (d_bladder > 1 ? 1 : d_bladder);
	}
	
	
	/**
	 * The level of the stomach. [0,1]
	 * Empty stomach (0) leads to urge to eat/drink something at the bar via subsistence dominance
	 * 
	 * @return
	 */
	
	public double getStomachLevel(){
		return d_stomach;
	}
	
	/**
	 * Sets the stomachLevel
	 */
	protected void setStomachLevel(double stomachLevel){
		d_stomach = stomachLevel;
	}
	
	/**
	 *  Updates the stomach level in case nothing is affecting the stomach externally.
	 *  This means that the stomach content is decreasing during time following:
	 *  
	 *  C = constant = 0.0013 
	 *  d_stomach -= C 
	 *  
	 */
	protected void updateStomach(){
		d_stomach -= 0.0010;    // after 750 ticks a stomach is empty
		d_stomach = (d_stomach < 0 ? 0 : d_stomach);
		d_stomach = (d_stomach > 1 ? 1 : d_stomach);
	}
	
	/**
	 * Returns the energy level of an individual
	 * Note: is not used right now!
	 * @return
	 */
	public double getEnergy(){
		return d_energy;
	}
	
	
	/**
	 * Sets the energy level of an individual
	 * Note: is not used right now!
	 * 
	 */
	protected void setEnergy(double energy){
		d_energy = energy;
	}
	
	/**
	 * Returns the arousal level of an individual
	 * 
	 * @return arousal level
	 */
	public double getArousal(){
		return d_arousal;
	}
	
	/**
	 * Sets the arousal level of an individual
	 * 
	 * @param arousal
	 */
	protected void setArousal(double arousal){
		d_arousal = arousal;
	}
	
	/**
	 * Sets the dominant influence given (subsistence-related) physiology
	 * Here the bladder (urge to go to the toilet) and
	 * stomach (urge to have eat or drink) are the competitors.
	 * 
	 */
	protected void setDomPhysiology(){

		if ((1 - d_bladder) >= d_stomach){
			d_domPhysioString = Constants.BLADDER;
			d_domPhysioValue = 1 - d_bladder;
		} else {
			d_domPhysioString = Constants.STOMACH;
			d_domPhysioValue = d_stomach;	
		}

		
	}
	
	/**
	 * Returns the dominant influence given (subsistence-related) physiology
	 * 
	 * @return dominant physio factor: bladder or stomach
	 */
	public String getDomPhysiology(){
		return d_domPhysioString;
	}
	
	/**
	 * Returns the dominant influence value given (subsistence-related) physiology
	 * This is used to set the satisfaction level of the subsistence goal.
	 * 
	 * returns the values of the stomach and bladder, normalised by returning
	 * a value corresponding to the satisfaction value (meaning that 1-stomach
	 * represent the stomach satisfaction)
	 * 
	 * @return dominant value
	 */
	public double getDomPhysioLevel(){
		return d_domPhysioValue;
	}

	/**
	 * Describes the level of density purely based upon the locally perceived density.
	 * It is a very simple, linear, threshold rule, limited by a minimum as the arousal level
	 * will not be low in the simulations we are dealing with. 
	 * 
	 * 
	 * @param density
	 */
	
	public void updateArousal(double density) {
		if (density < 3) {
			d_arousal = 0.3;
		} else if (density > 10){
			d_arousal = 1.0;
		} else {
			d_arousal = density / 10;
		}
		
	}
	
	
}
