package crowdmodel.environment;

import java.awt.Polygon;
import java.util.Vector;
import java.util.Iterator;

import crowdmodel.environment.CellObject;
import crowdmodel.utils.*;
import repast.simphony.context.Context;
import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.grid.Grid;

/**
 * Describes certain scenario specific areas on a physical terrain in the world
 * An area represents a set of cellObjects/gridCells that cover a part of the
 * environmental space
 * 
 * @author Nanda Wijermans
 *
 */

public class Area {
		
	Context d_context;     
	
	String d_name;
	boolean d_walkable;
	Polygon d_areaPolygon;
	int d_xmin, d_ymin, d_xmax, d_ymax;
	Vector<NdPoint> d_pois;   // points of interests of this area
	
	/**
	 * Constructor of Area
	 * Area defines parts of the world with a specific attribute
	 * in terms of walkability and name (which has a semantic given a scenario)
	 * 
	 * Festival scenario areas: stage, bar and toilet
	 * 
	 */
	
	public Area(Context context, String name, Polygon areaPolygon, boolean walkable){
		d_name = name;
		d_context = context;
		d_areaPolygon = areaPolygon;
		d_pois = new Vector<NdPoint>();
		
		int[] xminmax = MathUtils.getMinAndMax(d_areaPolygon.xpoints, d_areaPolygon.npoints);
		int[] yminmax = MathUtils.getMinAndMax(d_areaPolygon.ypoints, d_areaPolygon.npoints);
		
		d_xmin = xminmax[0]; 
		d_xmax = xminmax[1];

		d_ymin = yminmax[0];
		d_ymax = yminmax[1];
		
		setWalkable(walkable);		
		setArea();
	}	
	
	/**
	 * setArea really sets a part of a grid as walkable or non-walkable
	 * changes those cells that are (festival) area cells.
	 * This information is also used to update the valueLayer 'terrain'
	 * to visualise the terrain on the screen
	 */
	private void setArea(){		
		Grid patch = (Grid)d_context.getProjection("Simple Grid");
		CellObject cell = null;
		
		for (int x = d_xmin; x < (d_xmax + 1); ++x){ 
			for (int y = d_ymin; y < (d_ymax + 1); ++y){ 
				for (Object o : patch.getObjectsAt(x, y)){
					if (o instanceof CellObject) {
						cell = (CellObject)o;
						setCell(cell); 
					}
				}
			}
		}
	}
	
	/**
	 * Sets the point-of-interest (POIs) by changing the
	 * name of the cell object representing an environmenttype (type area (terrain, stage, toilet,bar and poi))
	 */
	public void setPOIs(){
		Grid patch = (Grid)d_context.getProjection("Simple Grid");
		CellObject cell = null;
		
		Iterator<NdPoint> pois = d_pois.iterator();
		while (pois.hasNext()){
			NdPoint poi = pois.next();
			int x = (int) Math.rint(
					poi.getX());
			int y = (int) Math.rint(poi.getY());
			for (Object o : patch.getObjectsAt(x, y)){
				if (o instanceof CellObject) {
					cell = (CellObject)o;
					cell.setName("poi");
					cell.setWalkable(true);
					cell.setValue(d_context);
				}
			}
		}
	}
	
	/**
	 * Sets every cell belonging to that Area
	 * by defining the name, walkability and value in valueLayer (for visualisation purposes)
	 */
	public void setCell(CellObject cell){
		cell.setName(d_name);
		cell.setWalkable(d_walkable);
		cell.setValue(d_context);
	}
	
	/**
	 * Returns the name-label of this area
	 */
	public String getName(){
		return d_name;
	}

	/**
	 * Sets the walkability of this area
	 */
	public void setWalkable(boolean walkable) {
		d_walkable = walkable;
	}

	/**
	 * Returns the walkability of this area
	 */
	public boolean getWalkable() {
		return d_walkable;
	}

	
	/**
	 * Creates a bar of points of interest
	 * This is used for the festival scenario
	 * for the stage
	 */
	public void createPois(){
		int delta = (d_xmax - d_xmin);
		int poiA = (int)Math.floor(d_xmin + (delta * 0.25));
		int poiB = (int)Math.floor(d_xmax - (delta * 0.25));
		
		for (int i = poiA; i < (poiB + 1); ++i){
			addPoi(new NdPoint(i, d_ymin));
		}
	}
	
	
	/**
	 * Adds a point of interest (poi) to collection of
	 * pois for this specific type of area
	 * 
	 */
	public void addPoi(NdPoint poi){
		d_pois.add(poi);
	}
	
	/**
	 * Returns point(s) of interest (pois) of this
	 * area
	 */
	public Vector<NdPoint> getPoi(){
		return d_pois;
	}
		
	
}
