package crowdmodel.gui;

import crowdmodel.Constants;

/**
 * A singleton class of GUI parameters
 * This allows for the creation of just one instantiation of the class
 * That is accessible for all.
 * 
 * The class is used to connect the parameters in the GUI with
 * the simulation parameters and vise versa
 * 
 * @author Nanda Wijermans
 * 
 */

public class GUIparams {
	
	private static GUIparams d_instance = null;
	
	boolean d_idGoal, d_socGoal, d_safeGoal, d_subsGoal;  // indicates which goals should play a role in the agents
	double d_initId, d_initSoc, d_initSafe;// d_initSubs;   // slider values for the initial value of the goals
	double d_custoId, d_custoSoc, d_custoSafe, d_custoSubs; // slider values for the customised preference value of the goals
	String d_Iddist, d_SocDist, d_SafeDist, d_SubsDist; //d_initSubsDist;		// combobox values for the distribution

	boolean d_visDomGoal;   //indicates whether the simulation visualisation is set to display goals
	
	/**
	 * A private constructor as this is a singleton class
	 * can only be called by getInstance, which makes sure
	 * that this class is only instantiated once
	 */
	private GUIparams(){
		d_idGoal = true;
		d_socGoal = true;
		d_safeGoal = true;
		d_subsGoal = true;
		
		d_initId = Constants.DEFAULT_INIT_IDENTITY;
		d_initSoc = Constants.DEFAULT_INIT_SOCIAL;
		d_initSafe = Constants.DEFAULT_INIT_SAFETY;
		
		d_custoId = Constants.DEFAULT_PREF_IDENTITY;
		d_custoSoc = Constants.DEFAULT_PREF_SOCIAL;
		d_custoSafe = Constants.DEFAULT_PREF_SAFETY;
		d_custoSubs = Constants.DEFAULT_PREF_SUBSISTENCE;
		
		d_Iddist = Constants.DIST_NORMAL;
		d_SocDist = Constants.DIST_NORMAL;
		d_SafeDist = Constants.DIST_NORMAL;
		d_SubsDist = Constants.DIST_UNIFORM;   
		
		d_visDomGoal = true;
	}
	
	/**
	 * This function returns the instantiation of this class
	 * As this is a singleton, there is only one object created
	 * which is referred to by instance.
	 */
	public static GUIparams getInstance(){
		if (d_instance == null){
			d_instance = new GUIparams();
		}
		return d_instance;
	}

	/**
	 * Sets the boolean value that indicates the participation 
	 * and thus influence of the identity goal in the simulation.
	 * 
	 * @param bool 
	 */
	public void setIdGoal(boolean bool){
		d_idGoal = bool;
	}
	
	/**
	 * Returns a boolean value that indicates the particaption
	 * and thus influence role of the identity goal.
	 * 
	 * @return d_idGoal
	 */
	public boolean getIdGoal(){
		return d_idGoal;
	}

	/**
	 * Sets the boolean value that indicates the participation 
	 * and thus influence of the social goal in the simulation.
	 * 
	 * @param bool 
	 */
		public void setSocGoal(boolean bool){
		d_socGoal = bool;
	}

	/**
	 * Returns a boolean value that indicates the particaption
	 * and thus influence role of the social goal.
	 * 
	 * @return d_socGoal
	 */

	public boolean getSocGoal(){
		return d_socGoal;
	}

	/**
	 * Sets the boolean value that indicates the participation 
	 * and thus influence of the safety goal in the simulation.
	 * 
	 * @param bool 
	 */
	
	public void setSafeGoal(boolean bool){
		d_safeGoal = bool;
	}
	
	/**
	 * Returns a boolean value that indicates the participation
	 * and thus influence role of the social goal.
	 * 
	 * @return d_safeGoal
	 */

	public boolean getSafeGoal(){
		return d_safeGoal;
	}
	
	
	/**
	 * Sets the boolean value that indicates the participation 
	 * and thus influence of the subsistence goal in the simulation.
	 * 
	 * @param bool 
	 */
	
	public void setSubsGoal(boolean bool){
		d_subsGoal = bool;
	}
	
	/**
	 * Returns a boolean value that indicates the participation
	 * and thus influence role of the social goal.
	 * 
	 * @return d_subsGoal
	 */

	public boolean getSubsGoal(){
		return d_subsGoal;
	}

	/**
	 * Returns the initial value of the identity goal
	 * this represents the current satisfaction level
	 * 
	 * @return d_initId
	 */
	public double getInitId() {
		return d_initId;
	}
	
	/**
	 * Sets the initial value for the identity goal
	 * this represents the current satisfaction level
	 * 
	 * @param initid
	 */
	public void setInitId(double initid) {
		d_initId = initid;
	}

	
	/**
	 * Returns the initial value of the social goal
	 * this represents the current satisfaction level
	 * 
	 * @return d_initSoc
	 */
	public double getInitSoc() {
		return d_initSoc;
	}

	/**
	 * Sets the initial value for the social goal
	 * this represents the current satisfaction level
	 * 
	 * @param soc
	 */

	public void setInitSoc(double soc) {
		d_initSoc = soc;
	}

	/**
	 * Returns the initial value of the safety goal
	 * this represents the current satisfaction level
	 * 
	 * @return d_initSafe
	 */
	public double getInitSafe() {
		return d_initSafe;
	}

	/**
	 * Sets the initial value for the safety goal
	 * this represents the current satisfaction level
	 * 
	 * @param safe
	 */

	public void setInitSafe(double safe) {
		d_initSafe = safe;
	}
	

	/**
	 * Returns the preference level of the identity goal
	 * in case is a customised distribution
	 * 
	 * The preference level is a attribute of an individual
	 * which can represent in the case of the identity goal
	 * that an individual has a high/low tendency or focus 
	 * for identity => higher/lower probability that the
	 * identity goal is dominant
	 * 
	 * @return d_custoID
	 */
	public double getCustoId() {
		return d_custoId;
	}
	
	/**
	 * Sets the preference level of the identity goal
	 * in case of a customised distribution. Which means
	 * that all agents get this same preference level.
	 * 
	 * @param d_custoid
	 */
			
	public void setCustoId(double d_custoid) {
		d_custoId = d_custoid;
	}

	/**
	 * Returns the preference level of the social goal
	 * in case of a customised distribution
	 * 
	 * The preference level is a attribute of an individual
	 * which can represent in the case of the social goal
	 * that an individual has a high/low tendency or focus 
	 * for social => higher/lower probability that the
	 * social goal is dominant. (example: more/less susceptible
	 * for social influences)
	 * 
	 * @return d_custoSoc
	 */
	
	public double getCustoSoc() {
		return d_custoSoc;
	}

	/**
	 * Sets the preference level of the social goal
	 * in case of a customised distribution. Which means
	 * that all agents get this same preference level.
	 * 
	 * @param soc
	 */

	public void setCustoSoc(double soc) {
		d_custoSoc = soc;
	}

	/**
	 * Returns the preference level of the safety goal
	 * in case is is a customised distribution
	 * 
	 * The preference level is a attribute of an individual
	 * which can represent in the case of the safety goal
	 * that an individual has a high/low tendency or focus 
	 * for safety => higher/lower probability that the
	 * safety goal is dominant. (example: easily/hardly scared)
	 * 
	 * @return d_custoSafe
	 */
	public double getCustoSafe() {
		return d_custoSafe;
	}

	/**
	 * Sets the preference level of the safety goal
	 * in case of a customised distribution. Which means
	 * that all agents get this same preference level.
	 * 
	 * @param safe
	 */
	public void setCustoSafe(double safe) {
		d_custoSafe = safe;
	}
	/**
	 * Returns the preference level of the subsistence goal
	 * in case is is a customised distribution
	 * 
	 * The preference level is a attribute of an individual
	 * which can represent in the case of the subsistence goal.
	 * This is more or less the same for individuals as it represents
	 * the energy level. 
	 * 
	 * 
	 * @return d_custoSubs
	 */
	public double getCustoSubs() {
		return d_custoSubs;
	}

	/**
	 * Sets the preference level of the subsistence goal
	 * in case of a customised distribution. Which means
	 * that all agents get this same preference level.
	 * 
	 * @param subs
	 */
	public void setCustoSubs(double subs) {
		d_custoSubs = subs;
	}

	/**
	 * Returns the type of distribution used to set
	 * the preferred level of identity satisfaction for a crowd (agent set)
	 * This allows for a heterogeneous set of agents
	 * 
	 * @return d_IDdist
	 */
	public String getIdDist() {
		return d_Iddist;
	}

	/**
	 * Sets the type of distribution that will be used to set
	 * the preferred level of identity satisfaction for a crowd (agent set)
	 * This allows for a heterogeneous set of agents
	 * 
	 * @param ddist
	 */
	public void setIdDist(String ddist) {
		d_Iddist = ddist;
	}

	/**
	 * Returns the type of distribution used to set
	 * the preferred level of social satisfaction for a crowd (agent set)
	 * This allows for a heterogeneous set of agents
	 * 
	 * @return d_SocDist
	 */
	public String getSocDist() {
		return d_SocDist;
	}

	/**
	 * Sets the type of distribution that will be used to set
	 * the preferred level of social satisfaction for a crowd (agent set)
	 * This allows for a heterogeneous set of agents
	 * 
	 * @param socDist
	 */
	public void setSocDist(String socDist) {
		d_SocDist = socDist;
	}

	/**
	 * Returns the type of distribution used to set
	 * the preferred level of safety satisfaction for a crowd (agent set)
	 * This allows for a heterogeneous set of agents
	 * 
	 * @return d_SafeDist
	 */
	public String getSafeDist() {
		return d_SafeDist;
	}

	/**
	 * Sets the type of distribution that will be used to set
	 * the preferred level of safety satisfaction for a crowd (agent set)
	 * This allows for a heterogeneous set of agents
	 * 
	 * @param safeDist
	 */
	public void setSafeDist(String safeDist) {
		d_SafeDist = safeDist;
	}

	/**
	 * Returns the type of distribution used to set
	 * the preferred level of subsistence satisfaction for a crowd (agent set)
	 * This allows for a heterogeneous set of agents
	 * 
	 * @return d_SubsDist
	 */
	public String getSubsDist() {
		return d_SubsDist;
	}
	
	/**
	 * Sets the type of distribution that will be used to set
	 * the preferred level of subsistence satisfaction for a crowd (agent set)
	 * This allows for a heterogeneous set of agents
	 * 
	 * @param subsDist
	 */
	public void setSubsDist(String subsDist) {
		d_SubsDist = subsDist;
	}
	
	
	/**
	 * Returns information for the visualisation of the agents
	 * Based on the boolean: visDomGoal the dominant goal of each 
	 * agent is visualised or the behaviour an agent shows
	 * 
	 * @return d_visDomGoal 
	 */
	public boolean getVisDomGoal() {
		return d_visDomGoal;
	}

	/**
	 * Sets the visualisation of the agent to be representing
	 * the dominant goal of an agent.
	 * 
	 * @param domGoal
	 * 
	 */
	public void setVisDomGoal(boolean domGoal) {
		d_visDomGoal = domGoal;
	}

	
	
	
}
