package crowdmodel.test;

import java.util.Iterator;
import java.util.Vector;

import crowdmodel.CObject;
import crowdmodel.Constants;
import crowdmodel.agents.CrowdAgent;
import crowdmodel.agents.MemoryElement;
import crowdmodel.utils.MiscSingleton;
import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.environment.RunState;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridPoint;

/**
 * This class describes a test of the function of the implemented CrowdModel
 * It concerns one of the major processes: perception
 * By checking this process, the functioning of the structure components
 * is also tested.
 * 
 * It is not the most thorough way of code testing (as I am not testing every
 * method) it is a pragmatic way of checking whether my theoretical model
 * is working the way it is supposed to.
 * 
 * 
 * @author Nanda Wijermans
 *
 */


public class PerceiveTest extends CObject{
	
	Context d_world;
	CrowdAgent d_testDummy;
	
	public PerceiveTest(CrowdAgent agent) {
		d_id = MiscSingleton.getInstance().nextID();
		d_testDummy = agent;
	}


	/**
	 * Tests all aspects of perception for every time step.
	 * 
	 */
	
//@ScheduledMethod(start = 1, interval = 10, priority = 10.0) // high priority = high number, earlier ; low priority = low number, later
	public void testIt(){
		d_world = RunState.getInstance().getMasterContext();
		if (perceivedPersons()){
			//System.out.println("PerceiveTest: persons are correctly observed");
		}
		else {
			System.out.println("PerceiveTest: persons are NOT correctly observed");
		}
		
	}
	
	
	/**
	 * Function checks whether the agents that are situated on POIs (point-of-interest's)
	 * have their internal settings correctly updated. This implies:
	 * - BAR-POIS : check for a full stomach
	 * - TOILET-POIS : check for an empty bladder
	 * - STAGE-POI: check for a random agent whether the id goal was correctly updated
	 * 
	 * Note that this could be programmed more generic, however didn't do this because
	 * as I want to be able to specify the origin of the error in case something is not working well.
	 */
	
	public boolean perceivedPOIs(){		
		return (checkBarPOIs() && checkToiletPOIs() && checkStagePOI());
			
	}
	
	/**
	 * 	Checks whether the agents currently standing on at a bar (on a bar patch)
	 * 	have their physiology settings correctly updated
	 * 
	 * @return effect drinking beer takes place
	 */
	public boolean checkBarPOIs(){
		//Context context = RunState.getInstance().getMasterContext();
		ContinuousSpace space = (ContinuousSpace) d_world.getProjection("Continuous Space");
		Grid patch = (Grid) d_world.getProjection("Simple Grid");

		boolean perceivedPOIs = true;
		
		/* 
		 * Check fulfilment barPOI 1
		 */
		GridPoint bar1 = Constants.GPOI_BAR;
		Iterable objectsAtBar1 = patch.getObjectsAt(bar1.getX(),bar1.getY());
		for (Object o : objectsAtBar1) {
			if (o instanceof CrowdAgent){
				CrowdAgent ca = (CrowdAgent) o;
				if (fullStomach(ca)){
					//System.out.println("Tests.PerceveTest.poiBar1: Agent" + ca.getID() + " at the bar, correctly filled stomach");
				}else {
					System.out.println("Tests.PerceveTest.poiBar1: Agent" + ca.getID() + " at the bar, incorrectly handled! stomach isn't full");
					//System.out.println();
					perceivedPOIs = false;
				}
			}
		}
		
		/* 
		 * Check fulfilment barPOI 2
		 */
		GridPoint bar2 = Constants.GPOI_BAR1;
		Iterable objectsAtBar2 = patch.getObjectsAt(bar2.getX(),bar2.getY());
		for (Object o : objectsAtBar2) {
			if (o instanceof CrowdAgent){
				CrowdAgent ca = (CrowdAgent) o;
				if (fullStomach(ca)){
					//System.out.println("Tests.PerceveTest.poiBar2: Agent" + ca.getID() + " at the bar, correctly filled stomach");
				}else {
					perceivedPOIs = false;
					System.out.println("Tests.PerceveTest.poiBar2: Agent" + ca.getID() + " at the bar, incorrectly handled! stomach isn't full");
				}
			}
		}
		
		/* 
		 * Check fulfilment barPOI 3
		 */
		GridPoint bar3 = Constants.GPOI_BAR2;
		Iterable objectsAtBar3 = patch.getObjectsAt(bar3.getX(),bar3.getY());
		for (Object o : objectsAtBar3) {
			if (o instanceof CrowdAgent){
				CrowdAgent ca = (CrowdAgent) o;
				if (fullStomach(ca)){
					//System.out.println("Tests.PerceveTest.poiBar3: Agent" + ca.getID() + " at the bar, correctly filled stomach");
				}else {
					perceivedPOIs = false;
					System.out.println("Tests.PerceveTest.poiBar3: Agent" + ca.getID() + " at the bar, incorrectly handled! stomach isn't full");
				}
			}
		}		
		return  perceivedPOIs;
	}
	
	/**
	 * Checks whether the agents currently at a toilet (are situated on a toilet patch)
	 * 	have their physiology settings correctly updated (empty bladder)
	 * 
	 * @return whether going to the toilet has (physiological) effect
	 */
	public boolean checkToiletPOIs(){
		
		//Context context = RunState.getInstance().getMasterContext();
		ContinuousSpace space = (ContinuousSpace) d_world.getProjection("Continuous Space");
		Grid patch = (Grid) d_world.getProjection("Simple Grid");

		boolean perceivedPOIs = true;
		/* 
		 * Check fulfilment toiletPOI 1
		 */
		GridPoint toilet1 = Constants.GPOI_TOILET;
		Iterable objectsAtToilet1 = patch.getObjectsAt(toilet1.getX(),toilet1.getY());
		for (Object o : objectsAtToilet1) {
			if (o instanceof CrowdAgent){
				CrowdAgent ca = (CrowdAgent) o;
				if (emptyBladder(ca)){
					//System.out.println("Tests.PerceveTest.poiToilet1: Agent" + ca.getID() + " at the toilet, correctly emptied bladder");
				}else {
					perceivedPOIs = false;
					System.out.println("Tests.PerceveTest.poiToilet1: Agent" + ca.getID() + " at the toilet, incorrectly handled! bladder isn't empty");
				}
			}
		}

		/* 
		 * Check fulfilment toiletPOI 2
		 */
		GridPoint toilet2 = Constants.GPOI_TOILET1;
		Iterable objectsAtToilet2 = patch.getObjectsAt(toilet2.getX(),toilet2.getY());
		for (Object o : objectsAtToilet2) {
			if (o instanceof CrowdAgent){
				CrowdAgent ca = (CrowdAgent) o;
				if (emptyBladder(ca)){
					//System.out.println("Tests.PerceveTest.poiToilet2: Agent" + ca.getID() + " at the toilet, correctly emptied bladder");
				}else {
					perceivedPOIs = false;
					System.out.println("Tests.PerceveTest.poiToilet2: Agent" + ca.getID() + " at the toilet, incorrectly handled! bladder isn't empty");
				}
			}
		}
		
		/* 
		 * Check fulfilment toiletPOI 3
		 */
		GridPoint toilet3 = Constants.GPOI_TOILET2;
		Iterable objectsAtToilet3 = patch.getObjectsAt(toilet3.getX(),toilet3.getY());
		for (Object o : objectsAtToilet3) {
			if (o instanceof CrowdAgent){
				CrowdAgent ca = (CrowdAgent) o;
				if (emptyBladder(ca)){
					//System.out.println("Tests.PerceveTest.poiToilet3: Agent" + ca.getID() + " at the toilet, correctly emptied bladder");
				}else {
					perceivedPOIs = false;
					System.out.println("Tests.PerceveTest.poiToilet3: Agent" + ca.getID() + " at the toilet, incorrectly handled! bladder isn't empty");
				}
			}
		}
		return perceivedPOIs;
	}
	
	/**
	 * Checks the effect of the stagePOI,<br>
	 * whether the vicinity of the stage has the proper effect on the id goal satisfaction
	 * level of the agent.<br><br>
	 * 
	 * based on the difference in registration the distance towards this poi can be 
	 * Selects a random agent and based on its distance from the stage it checks whether
	 * the identity goal was correctly updated.
	 * 
	 * @return proper effect of the Stage POI
	 */
	
	public boolean checkStagePOI(){
	
		//Context context = RunState.getInstance().getMasterContext();
		ContinuousSpace space = (ContinuousSpace) d_world.getProjection("Continuous Space");
		Grid patch = (Grid) d_world.getProjection("Simple Grid");
		Context agentContext = d_world.getSubContext("Agent Subcontext");
		
		boolean perceivedPOIs = true;
		
		NdPoint stage = Constants.POI_STAGE;
		double dist2poi = space.getDistance(d_testDummy.getPrevPos(), stage);
		double prevLevel = d_testDummy.getPrevIdSatis();
		double diffIdSatis = prevLevel - d_testDummy.getIDlevel();

		if (diffIdSatis == 0){
			if (prevLevel != 0 && prevLevel != 1)
				perceivedPOIs = false;
		}
		else if (diffIdSatis == 0.01){
			if (dist2poi > 3){
				perceivedPOIs = false;
			}
		} else if (diffIdSatis == 0.05){
			if (dist2poi > 7 && dist2poi < 3) {
				perceivedPOIs = false;
			}
		} else if (diffIdSatis == -0.025){
			if (dist2poi > 15 && dist2poi < 7){
				perceivedPOIs = false;
			}
		} else if ((diffIdSatis == -0.01)){
			if (dist2poi < 15){
				perceivedPOIs = false;
			}
	    } 

		if (!perceivedPOIs){
			System.out.println("PerceiveTest.checkStagePOI IdSatis is incorrect: dist2poi = " + dist2poi + ", and diffSatis = " + diffIdSatis); 
		}
		return perceivedPOIs;
	}
	
	/**
	 * Checks whether the perception of behaviours that are supposably seen by an agent are correctly handled by:<br>
	 * <ul>
	 * <li> priming the corresponding behaviourRule in memory</li>
	 * </ul>
	 * 
	 * This is done for all agents. So every agent is tested based upon it's current location and visible agents.
	 * @returns outcome of the behaviour perception test
	 */
	
	public boolean perceivedBehaviours(){
		
		Context agentContext = d_world.getSubContext(Constants.ID_AGENT_CONTEXT);
		ContinuousSpace space = (ContinuousSpace) d_world.getProjection("Continuous Space");
		Iterator<CrowdAgent> allAgents = agentContext.iterator();
		boolean bPerceived = true;
		
		while (allAgents.hasNext()) {
			CrowdAgent testDummy = (CrowdAgent) allAgents.next();
			NdPoint pos = space.getLocation(testDummy);
			Vector<CrowdAgent> visibleAgents = crowdmodel.utils.Misc.agentsInCone(d_world, testDummy, crowdmodel.Constants.GAZE_WIDTH, crowdmodel.Constants.GAZE_DEPTH);
			Iterator<CrowdAgent> perceivedAgents = visibleAgents.iterator();
			while(perceivedAgents.hasNext()){
				CrowdAgent ca = (CrowdAgent) perceivedAgents.next();
				String observedB = ca.d_myBehaviour;
				if (observedB != Constants.B_NAME_NOTHING && observedB != Constants.B_NAME_INHIBITED){
					if (primingTest(observedB,ca)){
						//System.out.println("PerceiveTest.perceivedBehaviours(): Behaviours are correctly primed by " + testDummy);
					} else {
						bPerceived = false;
						System.out.println("PerceiveTest.perceivedBehaviours():" + observedB + "wasn't primed");
					}
				}
			}
		}
		return bPerceived;
	}
	
	
	/**
	 * Tests whether a memory element was primed during the current time step.
	 * 
	 * @param elementName
	 * @return whether element is primed t = now 
	 */
	
	public boolean primingTest(String elementName, CrowdAgent testDummy){
		double currentTime = RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
		MemoryElement mElement =  testDummy.d_memory.getElement(elementName);
		if (mElement == null){
			System.out.println("PerceiveTest.primingTest(): " + elementName + " is null");
		}
		return mElement.getLastPrimeTime() == currentTime;
	}
	
	
	/**
	 * Checks whether perceiving a person is correctly handled:
	 * 
	 * 1) by priming the corresponding personFact in memory
	 * 2) density levels affecting physiology
	 * 3) satisfaction of the social goal
	 * @return proper effect of stimuli on the agent 
	 */
	public boolean perceivedPersons(){
		
		boolean pCheck = checkPrimingPersonFacts();
		
		
		/* density - arousal check */
		
		double arousal = d_testDummy.d_physiology.getArousal();
		double density = d_testDummy.getDensity();
		boolean aCheck = true;
		
		if (arousal > 0.3 && arousal < 1){
			if (arousal*10 != density){
				aCheck = false;
			}		
		} else if (arousal == 0.3){
			if (!(density <= 3)){
				aCheck = false;
			}
		} else if (arousal == 1.0){
			if (!(density >= 10)){
				aCheck = false;
			}
		} else {
			System.out.println("PerceiveTest.perceivedPersons(): deze waarde van arousal kan niet: " + arousal);
		}
		
		if (aCheck){
			//System.out.println("PerceiveTest.perceivedPersons(): arousal is correctly set");
		} else {
			System.out.println("PerceiveTest.perceivedPersons(): arousal is incorrectly set. arousal = " + arousal + " density = " + density);
		}
				
		/* safety satisfaction check */
		double safeSatis = d_testDummy.getSafelevel();
		boolean sCheck = true;
		if ((safeSatis > 0.5) && (density > 5.0)){ //density should be < 5
			sCheck = false;
			System.out.println("PerceiveTest.perceivedPersons(): safety satisaction is not done right. safesatis = " + safeSatis + " density = " + density);
		} else if ((safeSatis < 0.5) && (density < 5.0)){ // density should be > 5
			sCheck = false;
			System.out.println("PerceiveTest.perceivedPersons(): safety satisaction is not done right. safesatis = "+ safeSatis + " density = " + density);
		}
		
		/* social satisfaction check */
		double socSatis = d_testDummy.getSoclevel();
		double prevSocSatis = d_testDummy.getPrevSocSatis();
		double diff = socSatis - prevSocSatis;
		boolean sscheck = true;
		int nearby = d_testDummy.getNearbyAgents().size();
		if (diff < 0 && nearby != 0){
			sscheck = false;
			System.out.println("PerceiveTest.perceivedPersons(): social satic not correct fulfilled. diffSatis = " + diff + "nearby = " + nearby );
		} else if (diff >= 0 && nearby < 0){
			sscheck = false;
			System.out.println("PerceiveTest.perceivedPersons(): social satic not correct fulfilled. diffSatis = " + diff + "nearby = " + nearby );
		}
		
		return (pCheck && aCheck && sCheck && sscheck);	
	}
	
	/**
	 * Function that checks whether the perception of other agents (that are supposed to be seen)
	 * have their correct impact on the personFacts in terms of being primed. <br>
	 * When observing an other agents the corresponding facts need to be primed:
	 * - if known: the corresponding person fact (friends)
	 * - depending on membership: in or out group fact need to be primed
	 * 
	 * @return correctness of priming given perception of other agents
	 */
	public boolean checkPrimingPersonFacts(){
	
		ContinuousSpace space = (ContinuousSpace) d_world.getProjection("Continuous Space");
		Context agentContext = d_world.getSubContext(Constants.ID_AGENT_CONTEXT);
		Grid patch = (Grid) d_world.getProjection("Simple Grid");
		
		
		Iterator<CrowdAgent> allAgents = agentContext.iterator();
		boolean pPerceived = true;
		
		while (allAgents.hasNext()) {
			CrowdAgent testDummy = (CrowdAgent) allAgents.next();
			NdPoint pos = space.getLocation(testDummy);
			Vector<CrowdAgent> visibleAgents = crowdmodel.utils.Misc.agentsInCone(d_world, testDummy, crowdmodel.Constants.GAZE_WIDTH, crowdmodel.Constants.GAZE_DEPTH);
			Iterator<CrowdAgent> perceivedAgents = visibleAgents.iterator();
			while(perceivedAgents.hasNext()){
				CrowdAgent other = (CrowdAgent) perceivedAgents.next();
				String agentName = Constants.P_NAME_FRIEND + other.getAgentID();
				
				if (testDummy.d_memory.contains(agentName)){ // only facts in memory about friends?
					if (primingTest(agentName, testDummy)){
						//System.out.println("PerceiveTest.perceivedPersons(): person is seen and corresponding pFact primed");
					} else {
						pPerceived = false;
						System.out.println("PerceiveTest.perceivedPersons(): pFact wasn't primed!");
					}	
				}
				
				if (primingTest(Constants.P_NAME_INGROUP,testDummy)){
					if (testDummy.d_groupA == other.d_groupA){
						//System.out.println("PerceiveTest.perceivedPersons(): ingroup is correctly primed");	
					} else {
						pPerceived = false;
						System.out.println("PerceiveTest.perceivedPersons(): ingroup is primed, but shouldn't be primed");
					}
				} else {
					if (primingTest(Constants.P_NAME_OUTGROUP, testDummy)){
						pPerceived = false;
						System.out.println("PerceiveTest.perceivedPersons(): in/outgroup membership hasn't been primed correctly");
					} else {
						//System.out.println("PerceiveTest.perceivedPersons(): in/outgroup membership correctly primed");
					}
				}
			}	
		}
		return pPerceived;
	}
	
	
	/**
	 * Function that checks whether the stomach of an agent is full (equals 1)
	 * 
	 * @param agent
	 * @return whether the stomach is full
	 */
	public boolean fullStomach(CrowdAgent ca){
		double stomach = ca.d_physiology.getStomachLevel();
		return stomach == 1	;
	}
	
	/**
	 * Function that checks whether the bladder of an agent is empty (equals 0)
	 * 
	 * @param agent
	 * @return whether bladder is empty
	 */
	public boolean emptyBladder(CrowdAgent ca){
		double bladder = ca.d_physiology.getBladderLevel();
		return bladder == 0	;
	}

}
