package crowdmodel.utils;

import crowdmodel.Constants;
import repast.simphony.space.continuous.NdPoint;

/**
 * Class that describes static method functions
 * = functions that do not need a class instantiation to be used
 * = mathematical functions that are convenient for my model, however are not part of the model
 *
 * @author Nanda Wijermans
 */

public class MathUtils{

	/**
	 * Returns the distance between two NdPoints
	 */
	public static double getDistance(NdPoint pos1, NdPoint pos2){
		/* Returns the distance between two NdPoints (two positions on the continuous grid */

		return Math.sqrt((Math.abs(pos1.getX() - pos2.getX()) * 
							Math.abs(pos1.getX() - pos2.getX())) +
								(Math.abs(pos1.getY() - pos2.getY()) *
										Math.abs(pos1.getY() - pos2.getY())));
	}
	
	/**
	 * Calculates what the heading(angle) is given a position and the heading to the new heading should be, given a point it should be directed to 
	 * 
	 * In repast the coordinate system allows mixed ranges:
	 * - from 0 (top), 90 (right), 180/-180 (bottom), -90 (left).
	 * - [0,360]
	 * - [0,infinite]
	 * 
	 * Here we choose to generate the angles within the range [0,360]
	 * 
	 * The heading is calculated as follows: <br>
	 * <br>
	 * Based on the relative position from one point A (me) to another point B(other) there are
	 * either positioned:
	 * I) on the same axe (either: left, right, top or bottom position)<br>
	 * II) in one of the Quadrants (1,2,3,4)<br>
	 * <br>
	 * Given the distances (height and width of a vector between the points) and the quadrants the other point can
	 * lie in relative from point A (me)<br>
	 * deltaX = x1 (me) - x2 (other) <br>
	 * deltaY = x2 (me) - y2 (other) <br>
	 * <br>
	 * Quadrant 1 (Q1): top-right, (deltaX < 0 && deltaY < 0) <br>
	 * Quadrant 2 (Q2): top-left, (deltaX > 0 && deltaY < 0)  <br>
	 * Quadrant 3 (Q3): bottom-left, deltaX > 0 && deltaY < 0) <br>
	 * Quadrant 4 (Q4): bottom-right, deltaX < 0 && deltaY < 0) <br>
	 * <br>
	 * In condition I : <br>
	 * If deltaX == 0 : point B is on the same vertical height. <br>
	 * <li>
	 * 	<ul>	if deltaY > 0 : heading = 180, B is above A </ul>
	 * 	<ul>	if deltaY < 0 : heading = 0, B is below A </ul>
	 * </li>
	 * else if deltaY == 0 : point be is on the same horizontal height <br>
	 * <li>
	 * 	<ul>	if deltaX > 0 : heading = 270, B is left from A </ul>
	 * 	<ul>	if deltaX < 0 : heading = 90, B is right from A </ul>
	 * </li>
	 *
	 * The angle is calculated in the same way for each B, independent from the quadrant
	 * B is situated in. To calculate the heading is based on the the quadrant the other point is lying in. <br>
	 * <br>
	 * alpha = shift tan(deltaX / deltaY)
	 * Q1: heading = alpha
	 * Q2: heading = 180 - alpha
	 * Q3: heading = 180 + alpha
	 * Q4: heading = 360 - alpha
	 * 
	 * @param pos1 - my position (A)
	 * @param pos2 - position where I am heading to (B)
	 * 
	 */

	public static double calcHeading(double heading, NdPoint pos1, NdPoint pos2){
		
		double x1 = pos1.getX();
		double y1 = pos1.getY();
		double x2 = pos2.getX();
		double y2 = pos2.getY();
		
		double dx = x1 - x2;
		double dy = y1 - y2;
		double angle = 0;
		
		if (dx==0 && dy == 0){
			angle = heading;
		}
		if(dx == 0){
			if (dy > 0){ // other (poi) is positioned under me (pos)
				angle = 180;
				//System.out.println("du, == : " + angle);
			} else {		 // other (poi) is positioned above me (pos)	
				angle = 0;
				//System.out.println("dx ==0, maar dy < 0: " + angle);
			}
		} else if(dy == 0){ 
			if (dx < 0){ 	// other (poi) is right from me (pos)
				angle = 270;
				//System.out.println("du, == : " + angle);
			} else {			// other (poi) is left from me (pos)
				angle = 90;
				//System.out.println("du, == : " + angle);
			}
		} else {   // other point doesn't lie upon one of the axes
			double alpha = Math.atan(Math.abs(dx) / Math.abs(dy)); // [radians]
			alpha *= 180/Math.PI; //[degrees]
			
			if (dx < 0 && dy < 0){   // other (poi) lies in Quadrant 1, ie upper-left from me (pos)
				angle = alpha;
			} else if (dx < 0 && dy > 0){ // other (poi) lies in Quadrant 4, ie bottom-left from me (pos)
				angle = 180 - alpha;
			} else if (dx > 0 && dy > 0){ // other (poi) lies in Quadrant 3, ie bottom-right from me (pos) 
				angle = 180 + alpha;
			} else if (dx > 0 && dy < 0){ // other (poi) lies in Quadrant 2, ie upper-right from me (pos)  
				angle = 360 - alpha;
			} else {
				//System.errrrr 
				 System.out.println("MathUtils.getAngle(): else case that shouldn't be reached");
			}
		}
		return angle;
		
	}

	/**
	 * Function that decides whether a heading (beta) lies within line of sight given a
	 * current heading (myheading) and a range inbetween the value is supposed to lie. 
	 * It assumes heading values between [0,360].
	 * 
	 * @param myheading
	 * @param alpha
	 * @param gazeWidth
	 * @return whether a value lies within a certain range
	 */
	public static boolean withinLineOfSight(double myheading, double alpha, double gazeWidth){
		
		double diff  = 0;
		
		if (myheading > alpha){
			diff = myheading - alpha; 
		} else {  // beta <= alpha
			diff = alpha - myheading;
		}
		
		//check whether this is the smallest angle between alpha and beta 
		double diffInv = 360 - diff;
		diff = Math.min(diff, diffInv);
		return (diff <= (gazeWidth/2));
	}
	

	/**
	 * Returns the angle between two headings (in degrees)
	 * this is used to see whether a specific agent is positioned within a certain gaze-width
	 */
	public static double getAngleDiff(double h0, double h1){
			if (h0 > h1) {
				return AngleSubtraction(h0, h1);
			} else {
				return AngleSubtraction(h1, h0);
			}
	}
		
	/**
	 * Adds two angles with each other and corrects for non-existing angles [0-360]
	 * Repast corrects itself for this. 
	 * @param angle0
	 * @param angle1
	 * @return added angles within the[0-360] boundary
	 */
	public static double AngleAddition(double angle0, double angle1){
		return (angle0 + angle1) % 360;
	}
		
	/**
	 * Subtracts two angles with each other and corrects for non-existing angles [0-360]
	 * @param angle0
	 * @param angle1
	 * @return added angles within the[0-360] boundary
	 */
	public static double AngleSubtraction(double angle0, double angle1){
		return (angle0 - angle1) % 360;
	}
	/**
	 * Calculates a new position (NdPoint) on the continuous space  
	 * Given the current position, heading and speed.
	 */
	public static NdPoint calcNewPosition(double x, double y, double v, double heading){
		double h = heading * Math.PI / 180; // heading in radians
	    double displacement = v * 1;
		double dx = Math.sin(h) * displacement; // 
	    double dy = Math.cos(h) * displacement;
		
	    return new NdPoint (x + dx, y + dy);
	}
		

	/**
	 * Calculates the transposition (the distance in moving given a certain velocity) 	
	 * returns the distance in terms of simulation distance given a velocity in 
	 * real (m/s) terms
	 * 
	 * Physical space is related to a real physical space
	 * 1 grid cell / unit = 40x40cm (0.4x0.4m)
	 * movement of 1 is  40cm in real
	 * v [m/s] = [2.5 unit / s]  // need to think how simulation time relates to real time 
	 */
	
	public static double dist(double v){
		return (v * Constants.METER2UNIT);
	}
	
	/**
	 * getMin returns the minimal value of an Integer array.
	 */
	public static int getMin(int[] ar, int length){
		int min = ar[1]; //M: huh??? was precies de bedoeling hier? waarom [1]
		for (int i = 0; i < length; ++i){    
			if (min > ar[i]){
				min = ar[i];
			}
		}
		return min;
	}

	/**
	 * returns the maximal value of an Integer  array.
	 * overloaded function (for double arrays)
	 * 
	 * @param ar, length
	 * @return max 
	 */
	public static int getMax(int[] ar){
		int length = ar.length;
		int max = ar[1];
		for (int i = 0; i < length; ++i){    
			if (max < ar[i]) {
				max = ar[i];
			}
		}
		return max;
	}

	/**
	 * returns the maximal value of an double array.
	 * overloaded function (for integer arrays)
	 * 
	 * @param ar
	 * @return max value in array
	 */
	public static double getMax(double[] ar){
		int length = ar.length;
		double max = ar[0];
		for (int i = 0; i < length; ++i){    
			if (max < ar[i]) {
				max = ar[i];
			}
		}		
		return max;
	}
	
	/**
	 * Returns the index of the maximum value given an array of values
	 * 
	 * @param ar
	 * @param length
	 * @return index of the largest value in an array
	 */
	public static int getMaxIndex(double[] ar){
		double max = 0;
		int length = ar.length;
		int index = length;
		for (int i = 0; i < length; ++i){    
			if (max < ar[i]) {
				index = i;
				max = ar[i];
			}	
		}
		return index;
	}
	
	
	/**
	 * getMinAndMax returns the min and max value of an Integer array.
	 * Also in this order: ar[0] = min, ar[1] = max
	 * This is a bit more efficient if one needs min and max values
	 * in an array. 
	 */

 	public static int[] getMinAndMax(int[] ar){
		int length = ar.length;
		int min = Integer.MAX_VALUE;
		int max = 0;
		
		for (int i = 0; i < length; ++i){    
			max = ((max < ar[i]) ? ar[i] : max );
			min = ((min > ar[i]) ? ar[i] : min );
		}
		return new int[]{min, max};
	}
	
	/**
	 * Overloaded function that returns the min and max value of an Integer array.
	 * Overloaded because in some cases not the whole array needs to be considered
	 * such as in Area to look at the points in a Polygon
	 * 	
	 * @param ar
	 * @param length
	 * @return
	 */
	public static int[] getMinAndMax(int[] ar, int length){
		int min = Integer.MAX_VALUE;
		int max = 0;
		
		for (int i = 0; i < length; ++i){    
			max = ((max < ar[i]) ? ar[i] : max );
			min = ((min > ar[i]) ? ar[i] : min );
		}
		return new int[]{min, max};
	}
	
/**
 * Function calculates the area of a triangle 
 * Calculates the opposite, given the angle and Adjacent side
 * 
 *  opposite = tan(angle/2) * adjacent
 *  area = 2(opposite * adjacent)
 *  
 * @param angle - in degrees
 * @param az - adjacent side 
 * @return area 
 */
	
	public static double calcArea(double angle, double az){		
		double oz = Math.tan(angle/2)* az;
		return 2 *(oz * az);
		
	}
	
}
