package crowdmodel.utils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;
import repast.simphony.context.Context;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.graph.Network;
import repast.simphony.space.grid.GridPoint;
import repast.simphony.space.grid.Grid;
import crowdmodel.CObject;
import crowdmodel.Constants;
import crowdmodel.agents.CrowdAgent;
import crowdmodel.environment.CellObject;

/**
 * Class that describes static method functions
 * = functions that do not need a class instantiation to be used
 * = functions that are convenient for my model, however are not part of the model
 * 
 * These involve functions that describe perception functions like the agents that 
 * can be perceived by an individual, or to check whether a position is walkable or not,
 * functions to initialise the social structure etc.
 * 
 * @author Nanda Wijermans
 */

public class Misc{

	/** 
	 * Converts an String array into an ArrayList
	 * Used by the behaviourRules classes
	 * 	
	 */	
	
	public static ArrayList<String> fill(String[] list){

		ArrayList<String> behavioursList = new ArrayList<String>();
		for (int i = 0; i < list.length; ++i){
			behavioursList.add(list[i]);
			}
		return behavioursList;
	}	

	/**
	 * Calculates the number of agents that are situated in a cone
	 * given a position, angle, depth and width.
	 * 
	 * Used to calculate the density of number of agents observed by an agent
	 * 
	 * For future development: 
	 * - incorporate the vertical dimension/restrictions in perceiving to define whether an agent is observable or not.
	 * - refine this for the type of sensor (hearing has a different range then viewing)
	 */
	
	public static int nrAgentsInCone(Context context, CrowdAgent me, double angle, double distance){
	
		Context agentContext = (Context) context.getSubContext(Constants.ID_AGENT_CONTEXT);
		ContinuousSpace space = (ContinuousSpace)context.getProjection("Continuous Space");
				
		NdPoint myPos = space.getLocation(me);
		double heading = me.getHeading();
		int cnt = 0;
		
		Iterator<CrowdAgent> crowd = agentContext.iterator();
		
		while(crowd.hasNext()){
			CrowdAgent other = crowd.next();
			if (other != me){
				NdPoint otherPos = space.getLocation(other);
				if (MathUtils.getDistance(myPos, otherPos) < distance){  //checks whether agent is within gaze-depth
					if (myPos == otherPos){System.out.println("Misc.nrAgentsInCone - dat ken nie he!");}
					double angleTmp = MathUtils.calcHeading(heading, myPos, otherPos); 
					double dAngle = MathUtils.getAngleDiff(heading, angleTmp);
					if (Math.abs(dAngle) < angle / 2){
						++cnt;
					}
				}
			}
		}
		return cnt;
	}
	
	/**
	 * Indicated whether a point is crowded based on the occupancy of the 8 surrounding patches (Moore's neighbourhood).
	 * When the amount of people exceed 4, than is is considered crowded
	 * 
	 * This is used for the toilet and bar to indicate whether they are crowded or not.
	 * 
	 * @param space
	 * @param point
	 * @return
	 */
	public static boolean crowdedOld(ContinuousSpace space, NdPoint point){
		
		double x = point.getX();
		double y = point.getY();
		
		int peopleCnt = 0;
		
		
		/* neighbor (x+1,y)  (east)  */	
		for (Object o: space.getObjectsAt(x+1,y)) {		
			if (o instanceof CrowdAgent){
				++peopleCnt;
			}
		}
		
		/* Neighbour (x+1,y-1)  (south-east)  */	
		for (Object o: space.getObjectsAt(x+1,y-1)) {		
			if (o instanceof CrowdAgent){
				++peopleCnt;
			}
		}
		
		/* Neighbour (x,y-1)  (south)  */	
		for (Object o: space.getObjectsAt(x,y-1)) {		
			if (o instanceof CrowdAgent){
				++peopleCnt;
			}
		}
		
		/* Neighbour (x-1,y-1)  (south-west)  */	
		for (Object o: space.getObjectsAt(x-1,y-1)) {		
			if (o instanceof CrowdAgent){
				++peopleCnt;
			}
		}
		
		/* Neighbour (x-1,y)  (west)  */	
		for (Object o: space.getObjectsAt(x-1,y)) {		
			if (o instanceof CrowdAgent){
				++peopleCnt;
			}
		}
		
		/* Neighbour (x-1,y+1)  (north-west)  */	
		for (Object o: space.getObjectsAt(x-1,y+1)) {		
			if (o instanceof CrowdAgent){
				++peopleCnt;
			}
		}
		
		
		/* neighbor (x,y+1)  (north)  */	
		for (Object o: space.getObjectsAt(x,y+1)) {		
			if (o instanceof CrowdAgent){
				++peopleCnt;
			}
		}
		
		/* neighbor (x+1,y+1)  (north-east)  */	
		for (Object o: space.getObjectsAt(x+1,y+1)) {		
			if (o instanceof CrowdAgent){
				++peopleCnt;
			}
		}
		
		return (peopleCnt > 4 ? true : false);
	}

	
	public static boolean crowded(ContinuousSpace space, NdPoint point, double dist){
		
		double distance = dist;
		int cnt = 0;
		for (Object o : space.getObjects()){ 
			if (o instanceof CrowdAgent){
				CrowdAgent agent2 = (CrowdAgent) o;
				NdPoint otherPos = space.getLocation(agent2);
				if (MathUtils.getDistance(point, otherPos) < distance){  //checks whether agent is within gaze-depth
					++cnt;
				}
			}
		}
		return (cnt > 4);
	}
		
	public static Integer crowdedness(ContinuousSpace space, NdPoint point, double dist){
		
		double distance = dist;
		int cnt = 0;
		for (Object o : space.getObjects()){ 
			if (o instanceof CrowdAgent){
				CrowdAgent agent2 = (CrowdAgent) o;
				NdPoint otherPos = space.getLocation(agent2);
				if (MathUtils.getDistance(point, otherPos) < distance){  //checks whether agent is within gaze-depth
					++cnt;
				}
			}
		}
		return cnt;
	}
	
	/**
	 * Returns the agents that are situated in a cone given a position, angle, width and depth. 
	 * Other then that is is identical to the nrAgentsIncone function.
	 * 
	 * It is used to scan the observable agents for friends, or direction to walk to. But also to
	 * discern between the people (i.e. observable behaviours) that affect an agent.
	 * 
	 * For future development: 
	 * - incorporate the vertical dimension/restrictions in perceiving to define whether an agent is observable or not.
	 * - refine this for the type of sensor (hearing has a different range then viewing)
	 */

	public static Vector<CrowdAgent> agentsInCone(Context context, CrowdAgent me, double angle, double distance){
	
	/*
	 * A context needs to be given via a parameter as all variables in a static function need to be instantiated because
	 * the class is never instantiated. So all elements of the static function are static as well. (apart from the content
	 * parameters of course)
	 * 	
	 */
		
		Context agentContext = (Context) context.getSubContext(Constants.ID_AGENT_CONTEXT);
		ContinuousSpace space = (ContinuousSpace)context.getProjection("Continuous Space");
				
		NdPoint myPos = space.getLocation(me);
		double heading = me.getHeading();
		Vector<CrowdAgent> visibleAgents = new Vector<CrowdAgent> ();

		Iterator<CrowdAgent> crowd = agentContext.iterator();
		
		while(crowd.hasNext()){
			CrowdAgent other = crowd.next();
			if (other != me){
				NdPoint otherPos = space.getLocation(other);
				if (MathUtils.getDistance(myPos, otherPos) < distance){  //checks whether agent is within gaze-depth
					if (myPos == otherPos){System.out.println("Misc.nrAgentsInCone - dat ken nie he!");}
					double angleTmp = MathUtils.calcHeading(heading, myPos, otherPos); 
					if (MathUtils.withinLineOfSight(heading, angleTmp, Constants.GAZE_WIDTH)){
						visibleAgents.add(other);
					}
				}
			}
		}
 	    return visibleAgents;
	}

	/**
	 * Returns the location of the nearest positioned agent
	 * given a set of agents and my own position.
	 * 
	 * @param myPos - the position of the agent who is concerned with the distance of other agents
	 * @param agents - set of agents that I want to extract the position of the nearest person
	 * @return position of the nearest agents
	 */
	public static NdPoint posNearestAgent(Context context, NdPoint myPos, Iterator<CrowdAgent> agents, CrowdAgent me){

		ContinuousSpace space = (ContinuousSpace)context.getProjection("Continuous Space");
		NdPoint pos = new NdPoint(0,0,0);
		double minDist = Integer.MAX_VALUE;
	
		while (agents.hasNext()) {
			CrowdAgent ca = agents.next();
			if (ca != me){  
				NdPoint pos2 = space.getLocation(ca);
				double dist = space.getDistance(myPos, pos2);
				if (dist < minDist){
					minDist = dist;
					pos = pos2;
				}
			}
		}
		return pos;
	}

	/**
	 * Returns the agent that is physically the nearest
	 * 
	 * @param myPos - the position of the agent who is concerned with the distance of other agents
	 * @param agents - set of agents that I want to extract the position of the nearest person
	 * @return the agent that is most nearby
	 */
	public static CrowdAgent nearestAgent(Context context, CrowdAgent me, Iterator<CrowdAgent> agents){

		ContinuousSpace space = (ContinuousSpace) context.getProjection("Continuous Space");
		NdPoint mypos = space.getLocation(me);
		NdPoint pos = new NdPoint(0,0,0);
		CrowdAgent nearestAgent = null;
		double minDist = Integer.MAX_VALUE;
		
		while (agents.hasNext()) {
			CrowdAgent ca = agents.next();
			if (ca!= me){
				NdPoint pos2 = space.getLocation(ca);
				double dist = space.getDistance(mypos, pos2);
				if (dist < minDist){
					minDist = dist;
					pos = pos2;
					nearestAgent = ca;
				}
			}
		}			
		return nearestAgent;		
	}
	

	/**
	 * Indicates whether a GridPoint lies within the world
	 * 
	 * @param target
	 * @return a boolean value indicating whether a GridPoint lies within the word
	 */
	public static boolean withinWorld(GridPoint target){
		
		int x = target.getX();
		int y = target.getY();
		
		if (x < 0 || x >= Constants.WORLD_WIDTH){
			return false;
		} else
		if (y < 0 || y >= Constants.WORLD_HEIGHT){
			return false;
		} else
		return true;
	}
	
	/**
	 * Indicates whether a NdPoint lies within the world or not
	 * 
	 * @param target
	 * @return a boolean value indicating whether a NdPoint lies within the word
	 */
	public static boolean withinWorld(NdPoint target){
		
		double x = target.getX();
		double y = target.getY();
		
		if (x < 0 || x >= Constants.WORLD_WIDTH){
			return false;
		} else
		if (y < 0 || y >= Constants.WORLD_HEIGHT){
			return false;
		} else
		return true;
	}
	
	
	/**
	 * Given a (x,y) coordinate this function returns whether this gridcell is
	 * walkable or not based on the position being located within the world and
	 * whether the cell is walkable (not a building (stage,bar,toilet))
	 * 
	 * Note overloaded function!
	 * 
	 * @param px
	 * @param py
	 * @param sspace
	 * @return whether a patch is walkable or not
	 */
	public static boolean walkablePatch(Context context, CrowdAgent thisAgent, NdPoint target){
				
		Grid patch = (Grid) context.getProjection("Simple Grid");
		ContinuousSpace space = (ContinuousSpace) context.getProjection("Continuous Space");
		boolean walkable = true;
		boolean occupied = false;
		double x = target.getX();
		double y = target.getY();
		
		GridPoint targetP = Misc.nd2gridpoint(target);
		int px = targetP.getX(); 
		int py = targetP.getY();
		
		
		if (withinWorld(target)) {
			int agentsOnPatch = 0;
			for (Object o : patch.getObjectsAt(px, py)) {
				/* discern walkable from non-walkable patches */ 
				if (o instanceof CellObject) {
					CellObject cell = (CellObject) o;
					walkable = cell.getWalkable();
					//System.out.println("Misc.walkablePatch(): cell (" + cell.getX() + "," + cell.getY() + ") is niet beloopbaar");
				} 
				else if (o instanceof CrowdAgent){ 
				/* discern whether a patch is walkable based on agent occupation */
					CrowdAgent ca = (CrowdAgent) o;
					if (ca.getAgentID() != thisAgent.getAgentID()){
						++agentsOnPatch; 
					}
				}			
			}
			walkable = (agentsOnPatch > 0 ? false : walkable);
			return walkable;
		}
		else	
			return false;		
	}

	
	public static Vector<GridPoint> getMooresPatches(GridPoint gp){
		Vector<GridPoint> mooresPatches = new Vector<GridPoint>();
		int x = gp.getX();
		int y = gp.getY();
		
		GridPoint moore1 = new GridPoint(x+1, y); 	//right
		GridPoint moore2 = new GridPoint(x+1, y-1); //right-bottom
		GridPoint moore3 = new GridPoint(x, y-1); 	//bottom
		GridPoint moore4 = new GridPoint(x-1, y-1); //bottom - left
		GridPoint moore5 = new GridPoint(x-1, y); 	//left
		GridPoint moore6 = new GridPoint(x-1, y+1);	//left - top
		GridPoint moore7 = new GridPoint(x, y+1);	//top
		GridPoint moore8 = new GridPoint(x+1, y+1);	//top - right
		
		mooresPatches.add(gp);
		mooresPatches.add(moore1);
		mooresPatches.add(moore2);
		mooresPatches.add(moore3);
		mooresPatches.add(moore4);
		mooresPatches.add(moore5);
		mooresPatches.add(moore6);
		mooresPatches.add(moore7);
		mooresPatches.add(moore8);
		
		return mooresPatches;
	}
	
	
	

	/**
	 * Given a GridPoint (x,y) coordinate this function returns whether this gridcell is
	 * walkable or not (so if CellObjects are on it)
	 * 
	 * Note verloaded function!
	 * 
	 * @param px
	 * @param py
	 * @param sspace
	 * @return whether a patch is walkable or not
	 */
	public static boolean walkablePatch(Context context, GridPoint target){
		
		Grid patch = (Grid) context.getProjection("Simple Grid");
		ContinuousSpace space = (ContinuousSpace) context.getProjection("Continuous Space");
		boolean walkable = true;
		double dist = Constants.HUMAN_DIAMETER;
			
		int px = target.getX(); 
		int py = target.getY();
		
		int numAgentsOnPatch = 0;
		
		if (withinWorld(target)) {
			for (Object o : patch.getObjectsAt(px, py)) {
				/* discern walkable from non-walkable patches */ 
				if (o instanceof CellObject) {
					CellObject cell = (CellObject) o;
					walkable = cell.getWalkable();
				} 
				else if (o instanceof CrowdAgent){ 
				/* discern whether a patch is walkable based on agent occupation */
					++numAgentsOnPatch;
				}
				  //TODO syst.err
			}
			if (numAgentsOnPatch > 2){
				walkable = false;
			}
			return walkable;
		} else
			return false;
	}

	
	/**
	 * Calculates what the new heading should be, given a point it should be directed to 
	 * 
	 * In repast the coordinate system goes from 0 (top), 90 (right), 180/-180 (bottom), -90 (left).
	 * We calculate a Repast heading as follows: <br>
	 * <br>
	 * Based on the relative position from one point A (me) to another point B(other) there are
	 * either positioned:
	 * I) on the same axe (either: left, right, top or bottom position)<br>
	 * II) in one of the Quadrants (1,2,3,4)<br>
	 * <br>
	 * Given the distances (height and width of a vector between the points) and the quadrants the other point can
	 * lie in relative from point A (me)<br>
	 * deltaX = xpos (me) - xpoi (other) <br>
	 * deltaY = ypos (me) - ypoi (other) <br>
	 * <br>
	 * Quadrant 1 (Q1): top-right, (deltaX < 0 && deltaY < 0) <br>
	 * Quadrant 2 (Q2): top-left, (deltaX > 0 && deltaY < 0)  <br>
	 * Quadrant 3 (Q3): bottom-left, deltaX > 0 && deltaY < 0) <br>
	 * Quadrant 4 (Q4): bottom-right, deltaX < 0 && deltaY < 0) <br>
	 * <br>
	 * In condition I : <br>
	 * If deltaX == 0 : point B is on the same vertical height. <br>
	 * <li>
	 * 	<ul>	if deltaY > 0 : heading = 180 or -180, B is above A </ul>
	 * 	<ul>	if deltaY < 0 : heading = 0, B is below A </ul>
	 * </li>
	 * else if deltaY == 0 : point be is on the same horizontal height <br>
	 * <li>
	 * 	<ul>	if deltaX > 0 : heading = -90, B is left from A </ul>
	 * 	<ul>	if deltaX < 0 : heading = 90, B is right from A </ul>
	 * </li>
	 *
	 * The angle is calculated in the same way for each B, independent from the quadrant
	 * B is situated in. To calculate the heading is based on the the quadrant the other point is lying in. <br>
	 * <br>
	 * alpha = shift tan(deltaX / deltaY)
	 * Q1: heading = alpha
	 * Q2: heading = -alpha
	 * Q3: heading = -180 + alpha
	 * Q4: heading = 180 - alpha
	 * 
	 * @param pos - my posistion (A)
	 * @param poi - position I want to set my heading towards (B)
	 */
	public static double heading2point(NdPoint pos, NdPoint poi){
		
		double heading = 0;

		double posx = pos.getX();
		double posy = pos.getY();
		double poix = poi.getX();
		double poiy = poi.getY();
		
		double dx = posx - poix;
		double dy = posy - poiy;

		
		if (dx==0 && dy == 0){
			//System.err
		}
		if(dx == 0){
			if (dy > 0){ // other (poi) is positioned under me (pos)
				heading = 180;
			} else {		 // other (poi) is positioned above me (pos)	
				heading = 0;
			}
		} else if(dy == 0){ 
			if (dx < 0){ 	// other (poi) is right from me (pos)
				heading = -90;
			} else {			// other (poi) is left from me (pos)
				heading = 90;
			}
		} else {   // other point doesn't lie upon one of the axes
			double alpha = Math.atan(Math.abs(dx) / Math.abs(dy)); // [radians]
			alpha *= 180/Math.PI; //[degrees]
			
			if (dx < 0 && dy < 0){   // other (poi) lies in Quadrant 1, ie top-right from me (pos)
				heading = alpha;
			} else if (dx < 0 && dy > 0){ // other (poi) lies in Quadrant 4, ie bottom-right from me (pos)
				heading = 180 - alpha;
			} else if (dx > 0 && dy > 0){ // other (poi) lies in Quadrant 3, ie bottom-left from me (pos) 
				heading = - 180 + alpha;
			} else if (dx > 0 && dy < 0){ // other (poi) lies in Quadrant 2, ie top-left from me (pos)  
				heading = 360 - alpha;
			} else {
				 System.out.println("Misc.heading2point(): else case that shouldn't be reached");
			 }
		}
		return heading;
	}
	

	/**
	 * Returns the corresponding patch that a NdPoint belongs to
	 * (this is of course a approximation, as we use a rounding function
	 * for the double coordinate values to return an integer coordinate value)
	 * 
	 * @param target
	 * @return gridPoint given a NdPoint
	 */

	public static GridPoint nd2gridpoint(NdPoint target){
		int x = (int) Math.rint(target.getX());
		int y = (int) Math.rint(target.getY());
		
		x = (x == Constants.WORLD_WIDTH ? Constants.WORLD_WIDTH - 1 : x);
		y = (y == Constants.WORLD_HEIGHT ? Constants.WORLD_HEIGHT - 1 : y);
		
		return new GridPoint(x, y);
	}
		
	/**
	 * The function makeFriends defines the initial social structure.
	 * It describes the smallest unit of connection between
	 * individuals in terms of with whom am I coming to this event
	 * 
	 * Such a group is defined as a closed and full connected group.
	 * 
	 * The size and amount of groups is distinctive between the scenarios. We assume the following
	 * social structure settings given the a scenario:
	 * festival: mostly duo's, trio's and quartets (couples, sets of couples, friends, family)
	 * soccer: a group of (20-30) of more hard core supporters (given an in-group) and the rest of the 
	 * crowd consists again out of duo's, trio's  and quartets. (mental note: makes me think, as within this hardcore setting
	 * one also has the duos, triplets and quartets in friends, however have an extra group they feel verbonden with)
	 * demonstration:  They arrive in big groups, one can define department level so like 20, 30 people. However here one has also
	 * bigger clusters like 5, 6 . Sight, haven't made up my mind about this.  (mental note: can I avoid the multiple layers of groups one belong to
	 * in this setting? I mean in a festival it is so easy as they there is no link between the subgroups, there is no overall group or multiple overall
	 * groups, and is this not exactly what I define as an important indicator for the difference between more probable settings for a riot (focussed group))
	 * 
	 * @param crowdsize
	 * @param duoPerc
	 * @param trioPerc
	 * @param quartetPerc
	 * @return matrix that represents connections between agents
	 */
	
	public static int[][] makeFriends(int crowdsize, int duoPerc, int trioPerc, int quartetPerc){
		/* all variables are static variables in a static function */
		
		final int QUARTET = 4;		// to get a quartet (4) you need 3 friends
		final int TRIO = 3;			// to get a trio (3) you need 2 friends
		final int DUO = 2; 			// to get a duo (2) you need 1 friend
		
		int sum = duoPerc + trioPerc + quartetPerc; 
		
		if(sum > 100) { 
			System.out.println("sum of the percentages exceeds 100");
			//System.err van maken
		}		
		
		int numQuartets = (int) Math.rint(crowdsize * ((double) quartetPerc / 100));
		numQuartets = numQuartets / QUARTET;
		
		int numTrios = (int) Math.rint(crowdsize * ((double) trioPerc/100));
		numTrios =  numTrios / TRIO;
		
		int freeAgents = crowdsize - (numQuartets * QUARTET + numTrios * TRIO);;
		
		if (freeAgents % 2 == 1 && freeAgents > 2){ /* odd number */
			++numTrios;
			freeAgents -= 3;
		}	
		
		int numDuos = freeAgents / DUO;
		
		
		/* initialise matrix - nobody knows anybody */
		int[][] friendsMatrix = new int[crowdsize + 1][crowdsize];
		for (int i = 0; i < friendsMatrix.length; ++i) {
			for (int j = 0; j < friendsMatrix.length - 1; ++j){
				friendsMatrix[i][j] = 0;
			}
		}
		
		/* create the quartets, triplets and duos, respectively groups of four, three and two people */
		friendsMatrix = createGroup(friendsMatrix, QUARTET, numQuartets, crowdsize);
		friendsMatrix = createGroup(friendsMatrix, TRIO, numTrios, crowdsize);
		friendsMatrix = createGroup(friendsMatrix, DUO, numDuos, crowdsize);
		return friendsMatrix;
	}
	
	
	/**
	 * Function creates a matrix that represents a social structure.
	 * The matrix is filled by defining a (set of) group(s), based on the 
	 * number of groups with a certain group size.
	 * 
	 * An element in the matrix indicates whether an agent is connect (1) to another agent or not (0)
	 * Where the last column represents the number of ties/friends an individual has.
	 * 
	 *    | a1 | a2 | a3 | #ties \n
	 * -------------------------- \n
	 * a1 |  0 |  0 |  1 |  2 \n
	 * a2 |  0 |  0 |  1 |  2 \n
	 * a3 |  1 |  1 |  0 |  2 \n
	 * 
	 * Example shows that agent 1 and 2 have 1 connection and agent 3 has to as this one is connected
	 * to both agent 1 and 2.
	 *   
	 * @param matrix
	 * @param groupsize
	 * @param numgroups
	 * @param crowdsize
	 * @return matrix that represents connections between agents
	 * 
	 */
	public static int[][] createGroup(int[][] matrix, int groupsize, int numgroups, int crowdsize){
		
		final int FRIEND = 1;			
		final int NO_FRIEND = 0;
		final int NUM_TIES_INDEX = crowdsize;  /* last column of the matrix */

		int[][] friendsMatrix = matrix;
		  
		int agentIndex = 0;
		
		/* while not all groups are formed and there are still agents to form a group */
		while ((numgroups > 0) && (agentIndex < crowdsize)){		
			
			/* look for an agent to start forming a group */
			while ((agentIndex < crowdsize) && (friendsMatrix[NUM_TIES_INDEX][agentIndex] > 0)){
				++agentIndex;
			}
			
			/* find up to groupsize friends */
			int counter = friendsMatrix[NUM_TIES_INDEX][agentIndex]; /* number of friends/ties the given agent has */
			int friendIndex = 0;
			HashSet<Integer> gmembers = new HashSet<Integer>();
			gmembers.add(agentIndex);
			
			/* while the group is not complete given the current and there are still agent available to form a group */
			while ((counter < groupsize - 1) && (friendIndex < crowdsize)){   /* as long as you didn't find enough friends and there are people available */ 	
				if (friendIndex != agentIndex) {
				// cannot become friends with myself ;)	
					if ((friendsMatrix[friendIndex][agentIndex] == NO_FRIEND)  && (friendsMatrix[NUM_TIES_INDEX][friendIndex] == NO_FRIEND) ){
						// we aren't already friends AND this person has room for more friends :) ==> connect this person with the group!
						Iterator<Integer> newfriends = gmembers.iterator();
						while (newfriends.hasNext()){
							int friend = newfriends.next();
							friendsMatrix[friendIndex][friend] = FRIEND;
							++friendsMatrix[NUM_TIES_INDEX][friend];
							/* and vise versa */
							friendsMatrix[friend][friendIndex] = FRIEND;
							++friendsMatrix[NUM_TIES_INDEX][friendIndex];
						}
						gmembers.add(friendIndex);
						++counter;
					}
				}	
				++friendIndex;
			}
			--numgroups;
		}		
		return friendsMatrix;
	}
	 
	/**
	 * Prints a matrix on the console, specifically a matrix for friends
	 * with the first row being printed outside the matrix...
	 * 
	 * TODO update the name and explanation of this function
	 * 
	 * * - overloading function depending on the type of matrix
	 * here a matrix filled with Integer values
	 * 
	 * @param matrix
	 */
	public static void matrixPrint(int[][] matrix){
		for (int y = 0; y < matrix.length - 1; y++) {
			if(y < 10 ){
				System.out.print("a0"+ y + " | " + matrix[matrix.length-1][y] + ": [");
			}
			else {
				System.out.print("a"+ y + " | " + matrix[matrix.length-1][y] + ": [");
			}
			for (int x = 0; x < matrix.length - 1; x++) {
				System.out.print(matrix[x][y] + ", ");
			}
			System.out.println("]");
		}
	}

	/**
	 * Prints a matrix on the console
	 * 
	 * - overloading function depending on the type of matrix
	 * here a matrix filled with Double values
	 * @param matrix
	 */
	public static void matrixPrint(double[][] matrix){
		for (int y = 0; y < matrix.length; y++) {
			System.out.print("[");
			for (int x = 0; x < matrix.length; x++) {
				System.out.print(matrix[x][y]);
				if (x < matrix.length - 1){
					System.out.print(", ");
				}
			}
			System.out.println("]");
		}
	}
	
	/**
	 * Sets the initial social structure in the repast context in terms of a Network projection.
	 * The agents (nodes) that are connected (given a matrix) are set by adding an Edge to the
	 * network projection.
	 *  
	 * @param agentContext
	 * @param links
	 */
	
	public static void setSocialNetwork(Context agentContext, int[][] links){
		final int FRIEND = 1;			
		
		Network<CrowdAgent> socialNetwork = (Network<CrowdAgent>) agentContext.getProjection("Social Network");
		Iterator<CrowdAgent> agents = (Iterator<CrowdAgent>) agentContext.iterator();
		
		HashMap<Integer, CrowdAgent> friend2Agent = new HashMap<Integer, CrowdAgent>();
		
		int index = 0;
		while(agents.hasNext()){
			friend2Agent.put(index, agents.next());
			++index;
		}
		int lastAgent = links.length - 1;
		/* test */
		if ((index-1) == lastAgent){System.out.println("Misc.setSocNet: aantal agents komt niet overeen met het aantal slots in de friendsMatrix");}
		
		for (int i = 0; i < lastAgent; i++) {
			CrowdAgent cai = friend2Agent.get(i);
			for (int j = 0; j < lastAgent; j++) {
				if (links[i][j] == FRIEND){
					CrowdAgent caj = friend2Agent.get(j);
					socialNetwork.addEdge(cai, caj);
				}
			}
		}
	}
	
	/**
	 * This function returns whether an agent is standing close to one of its friends 
	 * 
	 * This is done by checking the distance between the agent and all of its friends.
	 * When the distance is closer than 0.5 meter a friend is being considered as physically close.
	 * 
	 * This function can be made in a computational cheaper way by checking whether
	 * the people that are close to you (within a radius of 0.5 meter) are your friends.
	 * However, I need to have some sort of collection to make this friend-check (friends.contains(closeAgent))
	 * and this is not possible using the Network information (as this returns an Iterable, which forces me
	 * to go through all my friends which makes the whole attempt obsolete given my current method) and I can
	 * use it by accessing memory, however I need to make this available and usable in terms of retrieving a
	 * subset of memory elements. this is not that important now. Furthermore the size of the groups are currently
	 * max. 4 people so not that expensive.
	 * 
	 * @param context
	 * @param friends
	 * @param myPos
	 * @return boolean whether an agent is physically close to one of its friends
	 */
	
	public static boolean close2friend(Context<CObject> context, Iterator<CrowdAgent> friends, CrowdAgent me){
		
		ContinuousSpace space = (ContinuousSpace) context.getProjection("Continuous Space");
		boolean close = false;
		double closeDist = 0.5 * Constants.METER2UNIT;
		NdPoint myPos = space.getLocation(me);
		
		while(friends.hasNext()){
			NdPoint friendPos = space.getLocation(friends.next());
			if (space.getDistance(myPos, friendPos) <= closeDist){
				close = true;
			}
		}
		return close;
	}

	/**
	 * Function returns a position close a given position (here the position of a friend is used)
	 * It searches for a position in the Moore's neighbourhood (the eight surrounding patches)
	 * When no position can be found in the neighbourhood that is available for an agent to stand
	 * on (depending on walkablility: occupiedness (other objects/agents) and positioned within the world)
	 * the function returns null.
	 * 
	 * @param context
	 * @param posFriend
	 * @param me
	 * @return a NdPoint near to a given position. Null is returned when no (available) position is found. 
	 */
	public static NdPoint posClose2pos(Context<CObject> context, NdPoint posFriend, CrowdAgent me) {
		ContinuousSpace space = (ContinuousSpace) context.getProjection("Continuous Space");
		Grid patch = (Grid) context.getProjection("Simple Grid");
		GridPoint centre = nd2gridpoint(posFriend);
		
		int xcentre = centre.getX();
		int ycentre = centre.getY();
		/* check Moore's neighbourhood (the eight surrounding patches */
		
		LinkedList<Integer> patches = new LinkedList<Integer>();
		for (int i = 0; i < 8; i++) {  // fill it with the patch numbers
			patches.add(i);
		}
		Collections.shuffle(patches);
		int dist = 1; 
		
		int x = xcentre;
		int y = ycentre;
	    int index = 0;
	    boolean foundAspot = false;
	    NdPoint target = new NdPoint(x,y,0);
	    
		while(!(foundAspot) && index < patches.size()){	
			int p = patches.get(index);
			switch (p) {
			case 0 : {		/* patch right from centre */
				x = xcentre + dist;
				y = ycentre;
				target = new NdPoint(x,y,0);
				break;
			}
			case 1 : {		/* patch left from centre */
				x = xcentre - dist;
				y = ycentre;
				target = new NdPoint(x,y,0);
				if (walkablePatch(context, me, target)){
					return new NdPoint(x,y,0);
				}
				break;	
			}
			case 2 : {		/* patch above the centre */
				x = xcentre;
				y = ycentre + dist;
				target = new NdPoint(x,y,0);
				break;
			}
			case 3 : {		/* patch under the centre */
				x = xcentre;
				y = ycentre - dist;
				target = new NdPoint(x,y,0);
				break;
			}
			case 4 : {		/* patch diagonal topright from the centre */
				x = xcentre + dist;
				y = ycentre + dist;
				target = new NdPoint(x,y,0);
				break;
			}
			case 5 : {		/* patch diagonal topleft from the centre */
				x = xcentre - dist;
				y = ycentre + dist;
				target = new NdPoint(x,y,0);
				break;
			}
			case 6 : {		/* patch diagonal bottomleft from the centre */
				x = xcentre - dist;
				y = ycentre - dist;
				target = new NdPoint(x,y,0);
				break;
			}
			case 7 : {		/* patch diagonal bottomright from the centre */
				x = xcentre + dist;
				y = ycentre - dist;
				target = new NdPoint(x,y,0);
				break;
			}
			default: target = null;
				break;
			}
			if (target != null){
				foundAspot = walkablePatch(context, me, target);
			}
			++index;
		}
		
		if (foundAspot){
			return new NdPoint(target.getX(),target.getY(),0);
		} else
			return null;
		
	}	
	
	/**
	 * 
	 * @param context
	 * @param pos
	 * @return
	 */
	
	public static GridPoint findWalkableNeighborPatch(Context<CObject> context, GridPoint pos) {
		ContinuousSpace space = (ContinuousSpace) context.getProjection("Continuous Space");
		Grid patch = (Grid) context.getProjection("Simple Grid");
		
		int xpos = pos.getX();
		int ypos = pos.getY();
		
		Vector<GridPoint> targets = new Vector<GridPoint>();
		
		/* patch right from centre */
		GridPoint target = new GridPoint(xpos + 1, ypos, 0);
		if (walkablePatch(context, target)){
			targets.add(target);
		}
		/* patch left from centre */
		target = new GridPoint(xpos - 1, ypos, 0);
		if (walkablePatch(context, target)){
			targets.add(target);
		}
		/* patch above the centre */
		target = new GridPoint(xpos, ypos + 1, 0);
		if (walkablePatch(context, target)){
			targets.add(target);
		}
		/* patch under the centre */
		target = new GridPoint(xpos,ypos - 1, 0);
		if (walkablePatch(context, target)){
			targets.add(target);
		}
		/* patch diagonal topright from the centre */
		target = new GridPoint(xpos + 1,ypos + 1, 0);
		if (walkablePatch(context, target)){
			targets.add(target);
		}
		/* patch diagonal topleft from the centre */
		target = new GridPoint(xpos - 1, ypos + 1, 0);
		if (walkablePatch(context, target)){
			targets.add(target);
		}
		/* patch diagonal bottomleft from the centre */
		target = new GridPoint(xpos - 1, ypos - 1, 0);
		if (walkablePatch(context, target)){
			targets.add(target);
		}
		/* patch diagonal bottomright from the centre */
		target = new GridPoint(xpos + 1, ypos - 1, 0);
		if (walkablePatch(context, target)){
			targets.add(target);
		}
		
		int range = targets.size();  // number of elements in a vector, here number of patches
		if (range ==  0){
			return null;
		} else {
			int randomIndex = MiscSingleton.getInstance().getMyUniform().nextIntFromTo(0,range-1);
			return targets.get(randomIndex);
		}
	}
	
	
	/**
	 * Given a behaviourName the corresponding factName is returned.
	 * Currently this is done a bit expensive, needs to be changed into
	 * a global variable
	 * 
	 * @param behaviourName
	 * @return
	 */
	public static String bName2fName(String behaviourName){
		HashMap<String, String> bname2fname = new HashMap<String, String>();  // contains the related names: behaviourRule - behaviourFact

		 bname2fname.put(Constants.B_NAME_WALK, Constants.F_NAME_EWALK);
		 bname2fname.put(Constants.B_NAME_RUN, Constants.F_NAME_ERUN);
		 bname2fname.put(Constants.B_NAME_DANCE, Constants.F_NAME_EDANCE);
		 bname2fname.put(Constants.B_NAME_STANDUP, Constants.F_NAME_ESTANDUP);
		 bname2fname.put(Constants.B_NAME_SITDOWN, Constants.F_NAME_ESITDOWN);
		 bname2fname.put(Constants.B_NAME_KNEEL, Constants.F_NAME_EKNEEL);
		 bname2fname.put(Constants.B_NAME_LIEDOWN, Constants.F_NAME_ELIEDOWN);
		 
		 bname2fname.put(Constants.B_NAME_CARRY, Constants.F_NAME_ECARRY);
		 bname2fname.put(Constants.B_NAME_LIFT, Constants.F_NAME_ELIFT);
		 bname2fname.put(Constants.B_NAME_PASS, Constants.F_NAME_EPASS);
		 bname2fname.put(Constants.B_NAME_STRIKE, Constants.F_NAME_ESTRIKE);
		 bname2fname.put(Constants.B_NAME_THROW, Constants.F_NAME_ETHROW);
		 bname2fname.put(Constants.B_NAME_PULL, Constants.F_NAME_EPULL);
		 bname2fname.put(Constants.B_NAME_PUSH, Constants.F_NAME_EPUSH);
		 bname2fname.put(Constants.B_NAME_EMBRACE, Constants.F_NAME_EEMBRACE);
		 bname2fname.put(Constants.B_NAME_RESTRAIN, Constants.F_NAME_ERESTRAIN);
		 bname2fname.put(Constants.B_NAME_CLAPPING, Constants.F_NAME_ECLAPPING);
		 bname2fname.put(Constants.B_NAME_SNAPPING, Constants.F_NAME_ESNAPPING);
		 
		 bname2fname.put(Constants.B_NAME_OTHER, Constants.F_NAME_EOTHER);
		 bname2fname.put(Constants.B_NAME_NOTHING, Constants.F_NAME_ENOTHING);
		 return bname2fname.get(behaviourName);
	}
	
	/**
	 * Given a factName the corresponding behaviourtName is returned.
	 * Currently this is done a bit expensive, needs to be changed into
	 * a global variable
	 * 
	 * @param factName
	 * @return
	 */
	
	public static String fName2bName(String factName){
		HashMap<String, String> fname2bname = new HashMap<String, String>();  // contains the related names: behaviourRule - behaviourFact

		 fname2bname.put(Constants.B_NAME_WALK, Constants.F_NAME_EWALK);
		 fname2bname.put(Constants.B_NAME_RUN, Constants.F_NAME_ERUN);
		 fname2bname.put(Constants.B_NAME_DANCE, Constants.F_NAME_EDANCE);
		 fname2bname.put(Constants.B_NAME_STANDUP, Constants.F_NAME_ESTANDUP);
		 fname2bname.put(Constants.B_NAME_SITDOWN, Constants.F_NAME_ESITDOWN);
		 fname2bname.put(Constants.B_NAME_KNEEL, Constants.F_NAME_EKNEEL);
		 fname2bname.put(Constants.B_NAME_LIEDOWN, Constants.F_NAME_ELIEDOWN);
		 
		 fname2bname.put(Constants.B_NAME_CARRY, Constants.F_NAME_ECARRY);
		 fname2bname.put(Constants.B_NAME_LIFT, Constants.F_NAME_ELIFT);
		 fname2bname.put(Constants.B_NAME_PASS, Constants.F_NAME_EPASS);
		 fname2bname.put(Constants.B_NAME_STRIKE, Constants.F_NAME_ESTRIKE);
		 fname2bname.put(Constants.B_NAME_THROW, Constants.F_NAME_ETHROW);
		 fname2bname.put(Constants.B_NAME_PULL, Constants.F_NAME_EPULL);
		 fname2bname.put(Constants.B_NAME_PUSH, Constants.F_NAME_EPUSH);
		 fname2bname.put(Constants.B_NAME_EMBRACE, Constants.F_NAME_EEMBRACE);
		 fname2bname.put(Constants.B_NAME_RESTRAIN, Constants.F_NAME_ERESTRAIN);
		 fname2bname.put(Constants.B_NAME_CLAPPING, Constants.F_NAME_ECLAPPING);
		 fname2bname.put(Constants.B_NAME_SNAPPING, Constants.F_NAME_ESNAPPING);
	
		 fname2bname.put(Constants.F_NAME_EOTHER, Constants.B_NAME_OTHER);
		 fname2bname.put(Constants.F_NAME_ENOTHING, Constants.B_NAME_NOTHING);
		 
		 return fname2bname.get(factName);
	}
	
	public static ArrayList iterator2array(Iterator iterator, int size){
		
		ArrayList array = new ArrayList(size);
		
		int i = 0;
		while (iterator.hasNext()) {
			array.add(i, iterator.next());
			++i;
		}
		return array;
	}	
	
	public static int getDensityCondition(int crowdsize){
	
		if(crowdsize == 227){
			return 1; //low
		} else if(crowdsize == 681){
			return 2; //medium
		} else if(crowdsize == 1275){
			return 3;
		} else
			return 0;
		}
	
	/**
	 * Relates the safety condition with a number.
	 * 1 = friends (crowd composition) = low preference safety goal = B(3,7)
	 * 2 = mixed (crowd composition) = average preference safety goal = B(5,5)
	 * 3 = family (crowd composition) = high preference safety goal = B(7,3)
	 * @param distribution
	 * @return numerical safety condition
	 */
	public static int getSafetyCondition(String distribution){
		
		if (distribution.equals(Constants.DIST_LEFT_SKEW)){
			return 1;
		} else if (distribution.equals(Constants.DIST_NORMAL)){
			return 2;
		} else if (distribution.equals(Constants.DIST_RIGHT_SKEW)){
			return 3;
		} else {
			return 0;
		}
	}
	
	
	public static int getLeadershipCondition(int leaderRatio){
		
		switch (leaderRatio) {
			case 0: return 1;   //zero
			case 10: return 2;  //
			case 25: return 3;	//
			case 50: return 4;  //
			case 75: return 5;  //
			case 100: return 6; //
		default: return 0;
		}
	}
}
	
