package crowdmodel.utils;

import cern.jet.random.Beta;
import cern.jet.random.Normal;
import cern.jet.random.Uniform;
import repast.simphony.random.RandomHelper;

/**
 * This is a singleton class to make sure that there are unique ID's for the objects in the crowd model
 * 
 * A singleton class is a construction in which one is sure that there is only one unique
 * instantiation of that class. Furthermore, it can be called by any class only via a
 * getInstance.
 * 
 * - ID number generator
 * - Random number generators (normal, beta and uniform distributions)
 * 
 * @author Nanda Wijermans
 *
 */

public class MiscSingleton {

	private static MiscSingleton d_instance = null;    //reference to itself
	
	private int d_nextID;			// a unique id-number for objects in crowdmodel		
	
	private Normal d_myNormal;
	private Uniform d_myUniform;
	private Beta d_myBetaNormal; // B(5,5)
	private Beta d_myBetaLeft; 	// B(3,7)     
	private Beta d_myBetaRight; // B(7,3)
	
	
	/**
	 * A private constructor as this is a singleton class
	 * can only be called by getInstance, which makes sure
	 * that this class is only instantiated once
	 */
	private MiscSingleton(){
		d_nextID = 0;
		d_myNormal = RandomHelper.createNormal(0.5, 0.1);
		d_myUniform = RandomHelper.createUniform(0, 1);
		d_myBetaNormal = RandomHelper.createBeta(5, 5);
		d_myBetaLeft = RandomHelper.createBeta(3, 7);
		d_myBetaRight = RandomHelper.createBeta(7, 3);
	}
	
	/**
	 * This function returns the instantiation of this class
	 * As this is a singleton, there is only one object created
	 * which is referred to by instance.
	 * 
	 */
	public static MiscSingleton getInstance(){
		if (d_instance == null){
			d_instance = new MiscSingleton();
		}
		return d_instance;
	}

	
	/**
	 * Returns an unique ID value used to give
	 * each created object a fixed number for identification
	 */
	
	public int nextID(){
		++d_nextID;
		return d_nextID;
	}

	/**
	 * Returns the (unique) normal distribution stream used in this simulation
	 * mean = 0.5
	 * std dev = 0.1
	 */
	public Normal getMyNormal() {
		return d_myNormal;
	}

	/**
	 * Returns the (unique) uniform distribution stream used in this simulation
	 */
	public Uniform getMyUniform() {
		return d_myUniform;
	}

	/**
	 * Returns the (unique) Beta distribution stream used in this simulation
	 * for the initial settings of the safety goal. Shaped as a left-skewed distribution (positive skew)
	 * 
	 */
	public Beta getMyBetaLeft() {
		return d_myBetaLeft;
	}

	/**
	 * Returns the (unique) Beta distribution stream used in this simulation
	 * for the initial settings of the safety goal. Shaped as a right-skewed distribution (negative skew)
	 * 
	 */
	public Beta getMyBetaRight() {
		return d_myBetaRight;
	}
	
	/**
	 * Returns the (unique) Beta distribution stream used in this simulation
	 * for the initial settings of the identity, social and safety goal. Shaped as a approximation of the normal distribution.
	 * 
	 */
	public Beta getMyBetaNormal() {
		return d_myBetaNormal;
	}
	
	
}
